#!/usr/bin/perl
# Takes an input dictionary containing entries for compund words 
# and generates a lex routine which
# substitutes such compounds by their equivalents, at least for the
# root form.
# For example the dictionary entries look like:
#  "ADA_lA","UNKN","vt","1.wakarAra/JagadZA_kara","d1"
#  "AI_calAI","UNKN","nf","1.sAxA_rahanasahana","d1"
# It would generate lex which sustitutes 'ADA  lA' by 'wakarAra', etc.

#print <<EOT;
#WORD [A-Za-z]*
#SEPARATOR [\",:;'\?><\]\[\}\{\)\( \n\t-_=!\@#%&\*\.\xea\\\/]
#SEP_COMPND [\*\t_\- \@\\\/]
#SEP_COMPND [\*\t\n_\- \@\\\/]  # Old
#VOW1 [aeiouAEIOUqMH]
#VOW2 [eiouEIOUqMH]
#CONS [kKgGfcCjJFtTdDNwWxXnpPbBmyrlvSRsh]
#%%
#EOT

# Instead of above commented out text, copy from a file.
open (PREAMB, "/home/bharathi/PNJ1/usr3/sangal/tools/src/pdgm_dict/preamble.lex") ||
  die "Unable to open file /home/bharathi/PNJ1/usr3/sangal/tools/src/pdgm_dict/preamble.lex: $!\n";
while (<PREAMB>) { print; }

$sep_full = '{SEPARATOR}';
$sep = '{SEP_COMPND}';

while (<>) {
  chop;
  s/"//g;
  @flds =  split (/,/);
  $in = $out = '';
  if ($flds[0] =~ /_/) {   # A compound entry found.
    $found = 1;
    @ivals = split (/_/, $flds[0]);
    $iter = 0;
    foreach $iv (@ivals) {
      # print STDERR "DEBUG: Val: $iv\n";
      if ($iter == 0) { $in = $sep_full . $iv;   $iter = 1; }
      else { $in .= $sep . '*' . $iv; }
    }
    $in .= '/' . $sep_full ;
    @vals = split (/\//, $flds[3]);
    foreach $v (@vals) {
      # print STDERR "DEBUG: Val: $v\n";
      if ($v =~ /^[0-9]+\./) { # First word for a distinct meaning.
        $match = $&;
        $first = $';  
        if ($first =~ /\/$/) { $first = $`; } # Remove trailing '/'
        if ($match =~ /1\./) { # First word for the first meaning.
          $out = $first;
        }
        else { $out .= '/' . $first; }
      }
    }
    #print STDERR 'DEBUG:', $in, ' -> ', $out, "\n";
    print  $in, '   { printf("%c', $out, '", yytext[0]); }', "\n";
  }
}
if (!($found)) { print STDERR "compnd.pl: No compound entries found.\n"; }

