#!/usr/bin/perl
# Takes a sorted dictionary containing entries from two authors
# (usually obtained by joining two dictionaries).
# The dictionary entry contains: "SRCWORD","TARGET_WORD","SOURCE"
# When two entries are found to have the same headword,
# marks the entries other than LATEST in the SOURCE field
# as old, by putting '####'
# at the end.
# Options:
# -l	-  Symbol to mark which entries are latest. 
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $Finname, $Foutname);
local ($latest);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$latest = 'LATEST'; # Default  Symbol to mark which entries are latest. 

# Look up .dict_joinrc file. Override defaults.
if (open (RC, '.dict_joinrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/anusAraka/punjabi/dict/frq_wrd/dict_join.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'l') {
        $latest = substr ($_, 2);
      }
      else { 
          print STDERR "dict_join.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/anusAraka/punjabi/dict/frq_wrd/dict_join.cmd');
          die;
      }
    }
    else { # 
      print STDERR "dict_join.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
     # Command line args override those of the rc file.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/anusAraka/punjabi/dict/frq_wrd/dict_join.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1, 1) eq  'l') {
      $latest = substr ($ARGV[$i], 2);
    }
    else { 
        print STDERR "dict_join.cmd: Unrecognized param: $ARGV[$i].\n";
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/anusAraka/punjabi/dict/frq_wrd/dict_join.cmd');
        die;
    }
  }
  else { # Argument found. Parameters end.
    $argbegpos = $i;
    last;
  }
}

# Process the arguments in the command line then call dict_join.
# 1. Can be used as a filter. OR
# 2. file args can be given instead of filters.

if ($argbegpos < 0) { # No args.
  &dict_join(STDIN, STDOUT);
}
elsif (($#ARGV - $argbegpos) == 0) { # Only one arg given. Execute the command with arg and stdout.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "dict_join.cmd: Unable to open file $ARGV[$argbegpos]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = '';
  &dict_join(INFILE, STDOUT);
  close (INFILE);
}
elsif (($#ARGV - $argbegpos) == 1) { # Two args given. Execute the command with arg1 and arg2.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "dict_join.cmd: Unable to open input file $ARGV[$argbegpos]\n" ;
  open (OUTFILE, ">$ARGV[$argbegpos + 1]") ||
     die "dict_join.cmd: Unable to open output file $ARGV[$argbegpos + 1]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = $ARGV[$argbegpos+1];  
  &dict_join(INFILE, OUTFILE);
  close (OUTFILE);
  close (INFILE);
}
else { print STDERR "dict_join.cmd: Too many arguments.\n"; }
#------------------------------------------------------

sub dict_join {
local ($W_fin, $W_fout) = @_;
select($W_fout);
#%COMMENT
# Takes a sorted dictionary containing entries from two authors
# (usually obtained by joining two dictionaries).
# The dictionary entry contains: "SRCWORD","TARGET_WORD","SOURCE"
# When two entries are found to have the same headword,
# marks the entries other than LATEST in the SOURCE field
# as old, by putting '####'
# at the end.
#%CMD TYPE 11

  
$found_latest = 0;
while (<$W_fin>) {
  chop;
  ($srcwrd,$targwrd, $author) = split(/,/);
  if ($srcwrd eq $prev_srcwrd) {
    #print STDERR "DEB: Head word matched: $srcwrd in $_.\n";
    if ($found_latest) { # If latest entry already found for this word.
      print $_, ",###\n";
    }
    elsif ($author =~ /"$latest"/) { # new entry is the latest
      $found_latest = 1;
      &empty_buffer ('not_latest');  # Empty buffer suffixing with '###'
      print $_, "\n";
    }
    else { # Latest not found yet. Put in buffer.
      &add_buffer ($_);
    }
  }
  else {  # New head word found.
    $found_latest = 0;
    &empty_buffer ('latest');
    ($prev_srcwrd,$prev_targwrd, $prev_author) = ($srcwrd,$targwrd, $author);
    &add_buffer($_);
  }
}
&empty_buffer('latest');  # Check whether argument.

# Empty the buffer, and suffix with '###' if not latest.
sub empty_buffer {
  local ($nature) = @_;
  local ($suff, $i);
  #print STDERR "DEB: empty_buffer $nature: $_\n";
  if ($nature eq 'latest') { $suff = ''; } 
  else { $suff = ',###'; }
  for ($i=0; $i <= $#buff; $i++) { print $buff[$i], $suff, "\n"; }
  $#buff = -1;
}

# add to the buffer
sub add_buffer {
  local ($line) = @_;
  $#buff++;  $buff[$#buff] = $line;
}
}
1;
