#!/usr/bin/perl
#%COMMENT
# Takes a sorted dictionary containing entries from two authors
# (usually obtained by joining two dictionaries).
# The dictionary entry contains: "SRCWORD","TARGET_WORD","SOURCE"
# When two entries are found to have the same headword,
# marks the entries other than LATEST in the SOURCE field
# as old, by putting '###'
# at the end.
#%CMD TYPE 11

$latest = 'LATEST';  # Symbol to mark which entries are latest. #OPT -l
  
$found_latest = 0;
while (<>) {
  chop;
  ($srcwrd,$targwrd, $author) = split(/,/);
  if ($srcwrd eq $prev_srcwrd) {
    #print STDERR "DEB: Head word matched: $srcwrd in $_.\n";
    if ($found_latest) { # If latest entry already found for this word.
      print $_, ",###\n";
    }
    elsif ($author =~ /"$latest"/) { # new entry is the latest
      $found_latest = 1;
      &empty_buffer ('not_latest');  # Empty buffer suffixing with '###'
      print $_, "\n";
    }
    else { # Latest not found yet. Put in buffer.
      &add_buffer ($_);
    }
  }
  else {  # New head word found.
    $found_latest = 0;
    &empty_buffer ('latest');
    ($prev_srcwrd,$prev_targwrd, $prev_author) = ($srcwrd,$targwrd, $author);
    &add_buffer($_);
  }
}
&empty_buffer('latest');  # Check whether argument.

# Empty the buffer, and suffix with '###' if not latest.
sub empty_buffer {
  local ($nature) = @_;
  local ($suff, $i);
  #print STDERR "DEB: empty_buffer $nature: $_\n";
  if ($nature eq 'latest') { $suff = ''; } 
  else { $suff = ',###'; }
  for ($i=0; $i <= $#buff; $i++) { print $buff[$i], $suff, "\n"; }
  $#buff = -1;
}

# add to the buffer
sub add_buffer {
  local ($line) = @_;
  $#buff++;  $buff[$#buff] = $line;
}
