#!/usr/bin/perl
# Takes DBM files containing words and their irefs
# and produces a text file in which each line contains a word
# followed by its irefs. (irefs are indexed references to files.
# For space efficiency, numbers instead of file names are used.
# These numbers are called 'irefs' for indexed refs.)
#   Recommended usage: prirefs.pl -fF > F.irefs
# where F contains DBM files.

# Options:
# -f	-  Default  Name of dbm file existing or to be created for words and their irefs which occur in the corpus to be indexed.
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/corp2/prirefs.pl");
require ("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");


# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $Finname, $Foutname);
local ($dbmfile);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$dbmfile = 'wrdiref'; # Default  Default  Name of dbm file existing or to be created for words and their irefs which occur in the corpus to be indexed.

# Look up .prirefsrc file. Override defaults.
if (open (RC, '.prirefsrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corp2/prirefs.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'f') {
        $dbmfile = substr ($_, 2);
      }
      else { 
          print STDERR "Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corp2/prirefs.cmd');
          die;
      }
    }
    else { # 
      print STDERR "Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
     # Command line args override those of the rc file.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corp2/prirefs.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1, 1) eq  'f') {
      $dbmfile = substr ($ARGV[$i], 2);
    }
    else { 
        print STDERR "Unrecognized param: $ARGV[$i].\n";
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corp2/prirefs.cmd');
        die;
    }
  }
  else { # Argument found. Parameters end.
    $argbegpos = $i;
    last;
  }
}

# Process the arguments in the command line then call prirefs.
# 1. Can be used as a filter. OR
# 2. file args can be given instead of filters.

if ($argbegpos < 0) { # No args.
  &prirefs(STDIN, STDOUT);
}
elsif (($#ARGV - $argbegpos) == 0) { # Only one arg given. Execute the command with arg and stdout.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "Unable to open file $ARGV[$argbegpos]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = '';
  &prirefs(INFILE, STDOUT);
  close (INFILE);
}
elsif (($#ARGV - $argbegpos) == 1) { # Two args given. Execute the command with arg1 and arg2.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "Unable to open input file $ARGV[$argbegpos]\n" ;
  open (OUTFILE, ">$ARGV[$argbegpos + 1]") ||
     die "Unable to open output file $ARGV[$argbegpos + 1]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = $ARGV[$argbegpos+1];  
  &prirefs(INFILE, OUTFILE);
  close (OUTFILE);
  close (INFILE);
}
else { print STDERR "Too many arguments.\n"; }
