#! /usr/bin/perl
# Takes as its input a file name and a line number in the file containing 
# extracted sentences from the corpus. Retrieves the reference to the
# sentence. (Reference consist of corpus filename and sentence number.)

sub sent2ref {
  local ($fname, $lno) = @_;
  local ($oldlimtr, $corpfile, $sentno, $foundlno, $foundref);
  open (FIN, $fname) ||
    die "sent2ref.pl: Unable to open file $fname.\n";
  $foundlno = 0;   # Searches for line number first.
                   # On finding it, starts looking for end of sentence.
  while (<FIN>) {
    if (!($foundlno)) {  #  Line number not found yet. Search for it.
      if ($lno == $.) {
        # print STDERR "sent2ref.pl deb:: Line no. found in: $_:";
        if (/<s([0-9]+)>\s/) { # Tolerant of placement of <sn>, works
                               #  with *.xtr, *.x70 files and its variants.
          $sentno = $1;
          # print STDERR "sent2ref.pl deb: Sentence no. found in: $_:";
          # Check whether the current line itself has corpus file name.
          if (/<@(.+)@>\n/) {
            $corpfile = $1;
            $foundref = 1;
            $/ = $oldlimtr;
            last;
          }
        }
        else {
          print STDERR "unable to find sentence number in $fname at line number $.\n";
        }
        $foundlno = 1;
        $oldlimtr = $/;
        $/ = "\n\n";
      }
    }
    else {  # Line number was found earlier. Now look for corp file name.
      # print STDERR "sent2ref.pl deb: To search for corp file name  $_:";
      if (/<@(.+)@>\n/) {
        $corpfile = $1;
        $foundref = 1;
        $/ = $oldlimtr;
      }
      else {
        print STDERR "Unable to find corpus file name in $fname after line number $.\n";
      }
      last;
    }
  }
  $/ = $oldlimtr;
  if ($foundref) { return ($corpfile, $sentno); }
  else {
    print STDERR "Unable to find corpus file name in $fname after line number $.\n";
  }
}
1;
