#!/usr/bin/perl
# Searches for a given pattern in the given files (say abc*.sm), and on
# finding it writes references: filename and sentence number (or line number).
#   Example Usage: xtrref.cmd -ppat abc*.sm >pat.ref
# Options:
#   -pxyz indicates that the 'xyz' pattern is to be searched
#         (can use regular expressions of Perl for pattern). 
# cmd type many to one.

# Options: (options marked by 'x' are disabled.)
# x -dxyz indicates that 'xyz' should be used as text line end mark.
# x -dn   indicates use of new line as end marker.
#         (Default delimiter is '<.>'.)
# x -noref tells it not to put file name etc. in the extracted lines
#         (Default is to put file name. Line num is also put
#               if delimiter is new line.)
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/corp2/xtrref.pl");
require ("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $arg, $outf, $filenamelistsw);
local ($delim, $nl_limtr, $noref, $fnameaffix, $pat);
$delim = '<.>';    $noref = 0;
#$nl_limtr = $/;  # Assumes that when the prog is called, new line limtr is set. #OLD
#$/ = $delim; #OLD
$parmlastpos = -1; # Gives pos of last param in ARGV. Default setting here no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default setting here no args
$filenamelistsw = 0;  # file list not specified. default.
#print STDERR "Number of args = $#ARGV; ARGS: @ARGV\n";  # for debugging
for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++; 
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corp2/xtrref.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1,1) eq 'o') { # Specifies output file.
      $outfilename = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1,1) eq 'd') { # Delimiter specified.
      print STDERR "Switch 'd' not implemented yet. Ignoring.\n";
      #if (substr($ARGV[$i],2,1) eq 'n') { # Use new line as delimiter
      #  $delim = $/;
      #}
      #else { $delim = substr ($ARGV[$i],2); }
    }
    elsif (substr($ARGV[$i],1,5) eq 'noref') { # Do not print ref info
      $noref = 1;
    }
    elsif (substr($ARGV[$i],1,1) eq 'p') { # Pattern that is to be searched
      $pat = substr($ARGV[$i],2);
    }
    else { 
      print STDERR "xtr.pl: Unrecognized switch: $ARGV[$i]\n"; 
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corp2/xtrref.cmd');
      die;
    }
  }
  else { # Switches are over 
    if (substr($ARGV[$i],0,1) eq '@') {  # A file list found
      #print STDERR "File list recognized.\n";  # for debugging
      $filenamelistsw = 1;
      $ARGV[$i] = substr($ARGV[$i], 1);
    }
    $argbegpos = $i;
    last;
  }
}

#print STDERR "DEB params end: $parmlastpos;  args begin: $argbegpos; total: $#ARGV;\n";  # for debugging

if ($pat eq '') { die "Pattern to be searched not specified.\n"; }

# Process the arguments in the command line then call xtract.
# 1. Can be used as a filter. OR
# 2. If arg file names are given, they are treated as input files
#    and the output goes to a single file stdout.

if ($outfilename ne '') {
  open (OUT, ">$outfilename") ||
    die "Unable to open output file $outfilename.\n";
  $outf = OUT;
}
else { $outf = STDOUT; }

if ($argbegpos < 0) { # No args.
  &xtract(STDIN, $outf);
}
elsif ($filenamelistsw) {  # file with list of file names. Call with each file name.
  open (FILELIST, "$ARGV[$argbegpos]") ||
    die "Unable to open file with file name list $ARGV[$argbegpos]\n" ;
  #$savelimtr = $/;  #OLD
  #$/ = $nl_limtr;   #OLD
  while (<FILELIST>) { # Read file names from file list.
    #$/ = $savelimtr;  #OLD
    chop;
    if (!(open (INFILE, $_))) {
      print STDERR "Unable to open file $_:\n" ;
      #$/ = $nl_limtr; #OLD
      next;
    }
    #print STDERR "Processing file: $_\n";  # for debugging
    if (!($noref)) {  # Put reference info.
      $fnameaffix = "<\@$_\@>";
    }
    &xtract (INFILE, $outf);
    close (INFILE);
    #$/ = $nl_limtr;  #OLD
  }
  close (FILELIST);
}
else { # Call the program with each arg. 
  for ($arg = $argbegpos; $arg <= $#ARGV; $arg++) {
    if (!(open (INFILE, "$ARGV[$arg]"))) {
      print STDERR "Unable to open file $ARGV[$arg]. Continuing.\n" ;
      next;
    }
    if (!($noref)) {  # Put reference info.
      $fnameaffix = "<\@$ARGV[$arg]\@>" ;
    }
    &xtract (INFILE, $outf);
    close (INFILE);
  }
}

