     HELP FILE FOR THE PERL PROGRAMMER FOR COMMAND LINE ARGS

This is a help file for the Perl programmer who wants to write
a program with command line interface using the existing programs
for command line processing. (The existing command line interface 
programs are in tools\src\cmd*.hlp files.) Here, are the steps.

Suppose you have to write a program called do-something.

1. You must write do-something as a Perl subroutine with two
arguments: the input file handle, and the output file handle.
You should place your subroutine in a separate file
called, say do-somthing.pl.  All the command line options typed when 
the command is given by the user will be provided as values of global variables
of your choosing. Therefore, you can assume suitable variable names
and their values for command line options, while writing the subroutine.

2. Decide what type of command you are trying to create: one file to one,
many files to one, or many files to many. (For example, many to one means that 
if many file names are given as input, the output produced by your program
is sent to a single output file, and so on for many to many etc.) For
more details see Appendix A.) Accordingly, choose the appropriate file out of
tools/src/cmd11.hlp, cmdm1.hlp and cmdmm.hlp, and make a copy into
do-something.cmd.

3. 

APPENDIX A:  Types of commands

1. One file to one: From one input file to one output file. 
The following
are accepted where INFILE and OUTFILE are any files for input and
output respectively to the command DS (do-something)
that you are defining:

DS  < INFILE    > OUTFILE      # DS can be used as a filter.
DS  INFILE  OUTFILE
DS INFILE  > OUTFILE

2. Many files to one: From many input files to one output file.
The following are accepted where INFILE and OUTFILE etc. are 
any files for input and output files to the command DS:

DS < INFILE  >OUTFILE          # DS can be used as filter.
DS INFILE > OUTFILE
DS IN1 IN2 ... INn > OUTFILE   # DS can be used with many file args
       # including with '*' characters. (DOS does not expand '*' chars.)
DS -oOUTFILE IN1 IN2 ... INn   #    -- ditto. --
DS @FILENAMES > OUTFILE       # Input file names are in the file called 
       # FILENAMES. Useful whenever filenames have been obtained by
       # manual editing. Permits tighter control over file args.
       # (Also useful in DOS wherever '*' is not expanded.)

3. Many files to many: From many input files to many output files.
The following are accepted where INFILE and OUTFILE etc. are 
any files for input and output files to the command DS:

DS < INFILE  >OUTFILE          # DS can be used as filter.
DS IN1 IN2 ... INn             # DS can be used with many file args
       # including with '*' characters. (DOS does not expand '*' chars.)
DS -s'.SUFF' IN1 IN2 ... INn   # For each input file, say INi, its
       # output is written into an output file named as INi.SUFF.
       # If no output file is specified, it uses '.sm' as suffix.
       # (For DOS which permits only one '.' in file names,
       # IN1, IN2, etc. must not have file extensions.)
DS @FILENAMES       # Input file names are in the file called 
       # FILENAMES. Useful whenever filenames have been obtained by
       # manual editing. Permits tighter control over file args.
       # (Also useful in DOS wherever '*' is not expanded.)
       # Output file names are generated by suffixes, as above.

APPENDIX B: Standard Built-in Options

1. All options for a command must be specified before giving
the file arguments or file redirection. The options are given
by putting a '-' before every option individually. (Currently, you
cannot specify several options with a single '-'.) For those options
which take a value, the value is given without putting any spaces
between the option name and the value. For example,

DS -A -Bxxx -CDEyyy -F ... filearguments

Thus, option B is given a value 'xxx', option CDE is given a value 'yyy',
whereas options A and F simply set a flag. Alternative ways to give a
value to option B are: -B'xxx' and -B"xxx", particularly if you
have spaces or special shell symbols such as '|' or ';' etc. in the value.
(Single quote and double quote have the same meaning as in Perl. Single
quoted strings are treated as they are, while in double quoted strings,
the Perl special symbols such as \n are expanded.)

2. All commands come with the following built-in option:

-h (as in DS -h):

This does not execute the command DS, instead it provides help regarding
the command DS. It prints the location of the command, and prints
the comment lines at the beginning of the file DS.cmd. (Therefore, you
should comment your DS.cmd command immediately, for yourselves and for
your users.  You should say what the command does, give an example
usage, and list the options. This is not a substitute for the man pages,
rather it provides a short help immediately even before the man pages
are prepared.)

Usually, the help option is executed and the program aborted whenever there is an error in giving
an option. (Sometimes, the command might be tolerant of wrong values 
of options, and might use a default value instead.)

3. Standard options with 'many files to one' type of commands:

 -oOUTFILE

The -o option specifies the name of the output file.

4. Standard options with 'many files to many' type of commands:

 -sSUFFIX

The -s option specifies the SUFFIX  to be used to generate the name of the output 
file.


----------------------
#!/usr/bin/perl
# Copies the lines twice from input file to output file.

# cmd assumes many to many.
require("/tools/src/etc/dupl_line.pl");
require ("/tools/src/etc/printusage.pl");


# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $arg, $suff, $argsinfile);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$argsinfile = 0;  # Default args file names are not in a file.
$suff = '.sm'; # Default suffix for output file if necessary.
for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/tools/src/corp/dupl_line.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1,1) eq 's') { # Suffix for output file names.
       $suff = substr ($ARGV[$i],2);
    }
    else { print STDERR "Parameter not recognized $ARGV[$i]. Ignoring.\n";
      &printusage ('/tools/src/corp/dupl_line.cmd');
    }
     ##elsif (substr($ARGV[$i],1,1) eq 'd') { # type of param is d, get val
     ##  $dval = substr ($ARGV[$i],2);
     ##}
     ##elsif (substr($ARGV[$i],1,1) eq 'n') { # set switch n to true.
     ##  $nswitch = 1;
     ##}
     ##else { print STDERR "Unrecognized param: $ARGV[$i]\n"; }
  }
  else { # Argument found. Parameters end.
    if (substr($ARGV[$i],0,1) eq '@') {  # A file of args found
      $argsinfile = 1;
      $ARGV[$i] = substr($ARGV[$i], 1);
    }
    $argbegpos = $i;
    last;
  }
}
#print STDERR "DEB params end: $parmlastpos;  args begin: $argbegpos; total: $#ARGV;\n";

# Process the arguments in the command line then call dupl_line.
# 1. Can be used as a filter. OR
# 2. If one or more file arguments are given, they are treated as
#    inputs files, and the output file name is given
#    by the input file name suffixed by the default suffix
#    (which can be changed by approp flag).

if ($argbegpos < 0) { # No args.
  &dupl_line(STDIN, STDOUT);
}
elsif ($argsinfile) {  # Args in file.
  open (ARGFILE, "$ARGV[$argbegpos]") ||
    die "Unable to open args file $ARGV[$argbegpos]\n" ;
  while (<ARGFILE>) { # Read args.
    chop;
    if (!(open (INFILE, $_))) {
      print STDERR "Unable to open file $_\n" ;
      next;
    }
    if (!(open (OUTFILE, ">$_$suff"))) {
      print STDERR "Unable to open out file $_$suff\n";
      close (INFILE);
      next;
    }
    &dupl_line (INFILE, OUTFILE);
    close (INFILE);
    close (OUTFILE);
  }
  close (ARGFILE);
}
else { # Call the program with each arg. 
  for ($arg = $argbegpos; $arg <= $#ARGV; $arg++) {
    if (!(open (INFILE, "$ARGV[$arg]"))) {
      print STDERR "Unable to open file $ARGV[$arg]\n" ;
      next;
    }
    if (!(open (OUTFILE, ">$ARGV[$arg]$suff"))) {
      print STDERR "Unable to open out file $ARGV[$arg]$suff\n";
      close (INFILE);
      next;
    }
    &dupl_line (INFILE, OUTFILE);
    close (INFILE);
    close (OUTFILE);
  }
}

