#!/usr/bin/perl
#%COMMENT
#%REQUIRE

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $Finname, $Foutname);
#%LOCAL
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
#%OPTIONS DEFAULTS

#%BEGIN RC FILE OPTIONS 
# Look up .DUMMYrc file. Override defaults.
if (open (RC, '.DUMMYrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('ABSNAME');
        die;
      }
#%RC ELSIF STMTS
      else { 
          print STDERR "DUMMY.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('ABSNAME');
          die;
      }
    }
    else { # 
      print STDERR "DUMMY.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}
#%END RC FILE OPTIONS 

#%BEGIN CMD LINE OPTIONS 
for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
     # Command line args override those of the rc file.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('ABSNAME');
      die;
    }
#%CMD LINE ELSIF STMTS
    else { 
        print STDERR "DUMMY.cmd: Unrecognized param: $ARGV[$i].\n";
        &printusage ('ABSNAME');
        die;
    }
  }
  else { # Argument found. Parameters end.
    $argbegpos = $i;
    last;
  }
}
#%END CMD LINE OPTIONS 

# Process the arguments in the command line then call DUMMY.
# 1. Can be used as a filter. OR
# 2. file args can be given instead of filters.

#%BEGIN CALL PROGRAM
if ($argbegpos < 0) { # No args.
  &DUMMY(STDIN, STDOUT);
}
elsif (($#ARGV - $argbegpos) == 0) { # Only one arg given. Execute the command with arg and stdout.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "DUMMY.cmd: Unable to open file $ARGV[$argbegpos]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = '';
  &DUMMY(INFILE, STDOUT);
  close (INFILE);
}
elsif (($#ARGV - $argbegpos) == 1) { # Two args given. Execute the command with arg1 and arg2.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "DUMMY.cmd: Unable to open input file $ARGV[$argbegpos]\n" ;
  open (OUTFILE, ">$ARGV[$argbegpos + 1]") ||
     die "DUMMY.cmd: Unable to open output file $ARGV[$argbegpos + 1]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = $ARGV[$argbegpos+1];  
  &DUMMY(INFILE, OUTFILE);
  close (OUTFILE);
  close (INFILE);
}
else { print STDERR "DUMMY.cmd: Too many arguments.\n"; }

