#!/usr/bin/perl
#%COMMENT

#%REQUIRE

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $arg, $outf, $argsinfile, $Finname, $Foutname);
#%LOCAL
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$argsinfile = 0;  # Default args file names are not in a file.
#%OPTIONS DEFAULTS

#%BEGIN RC FILE OPTIONS 
# Look up .DUMMYrc file. Override defaults.
if (open (RC, '.DUMMYrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('ABSNAME');
        die;
      }
#%RC ELSIF STMTS
      else { 
          print STDERR "DUMMY.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('ABSNAME');
          die;
      }
    }
    else { # 
      print STDERR "DUMMY.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}
#%END RC FILE OPTIONS 

#%BEGIN CMD LINE OPTIONS 
for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('ABSNAME');
      die;
    }
    elsif (substr($ARGV[$i],1,1) eq 'o') { # Specifies output file.
      $outfilename = substr ($ARGV[$i],2);
    }
#%CMD LINE ELSIF STMTS
    else { print STDERR "DUMMY.cmd: Unrecognized param: $ARGV[$i].\n";
      &printusage ('ABSNAME');
      die;
    }
  }
  else { # Argument found. Parameters end.
    if (substr($ARGV[$i],0,1) eq '@') {  # A file of args found
      $argsinfile = 1;
      $ARGV[$i] = substr($ARGV[$i], 1);
    }
    $argbegpos = $i;
    last;
  }
}
#%END CMD LINE OPTIONS 

#print STDERR "DUMMY.cmd: DEB params end: $parmlastpos;  args begin: $argbegpos; total: $#ARGV;\n";

#%BEGIN CALL PROGRAM
# Process the arguments in the command line then call DUMMY.
# 1. Can be used as a filter. OR
# 2. If arg file names are given, they are treated as input files
#    and the output goes to a single file stdout.

if ($outfilename ne '') {
  open (OUT, ">$outfilename") ||
    die "DUMMY.cmd: Unable to open output file $outfilename.\n";
  $outf = OUT;
  $Foutname = $outfilename;
}
else { $outf = STDOUT; }

if ($argbegpos < 0) { # No args.
  &DUMMY(STDIN, $outf);
}
elsif ($argsinfile) {  # Argument filenames in file.
  open (ARGFILE, "$ARGV[$argbegpos]") ||
    die "DUMMY.cmd: Unable to open args file $ARGV[$argbegpos]\n" ;
  while (<ARGFILE>) { # Read args.
    chop;
    if (!(open (INFILE, $_))) {
      print STDERR "DUMMY.cmd: Unable to open file $_\n" ;
      next;
    }
    $Finname = $_;
    &DUMMY (INFILE, $outf);
    close (INFILE);
  }
  close (ARGFILE);
}
else { # Call the program with each arg. 
  for ($arg = $argbegpos; $arg <= $#ARGV; $arg++) {
    if (!(open (INFILE, "$ARGV[$arg]"))) {
      print STDERR "DUMMY.cmd: Unable to open file $ARGV[$arg]. Continuing.\n" ;
      next;
    }
    $Finname = $ARGV[$arg];
    &DUMMY (INFILE, $outf);
    close (INFILE);
  }
}
close ($outf);
  
