#!/usr/bin/perl
# Re-formats a given text record file by writing field names along with
# their values, and putting each record in a new line.
# Useful for seeing structured data such as morph output, lwg output, etc.
#   Example usage: fmtrec.cmd  -fFIELDS -d';'  -r'@'   infile  outfile
# Options:
# -f	Specifies a file name containing the field names. Each field on a new 
#	line. (File called, say, FIELDS).  In case, this switch is not used, it
#	looks for the file name in fmtrec.rc file in the current directory.
# -d	Specifies field delimeter in the input file. Default: ','.
# -r	Specifies record delimeter in the input file.  Default: \n.
# -D	Specifies field delimeter for the output file. Default: \n.
# -R	Specifies record delimeter for the output file.  Default: \n\n.
# -=	Says that the output delimters should be the same as for the 
#	input. (Overrides -D and -R options.)
# -sf	Strip off field names contained in the input file.
# -nor	Do not use record delimiters while reading. Count field 
#	delimiters to determine end of record. (As a default, no record
#	delimiter is used in the output either.)
# -yesR	Turn on the record delimiter in the output, even when the -nor
#	option is on.(Otherwise, -nor option turns off the output record
#	delimiter.)
# -new	Specifies that a new record is to be input, print field names.
# (Note: Record delimiter should NEVER be a substring of the field delimter.
# More generally, a field delimiter should not occur in a field value,
# and a record delimiter should not occur anywhere in the record value,
# including inside a field delimiter.)
# These optons can also be set in fmtrec.rc file in the current directory.
# Command line parameters always override those in the rc file.
# A sample fmtrec.rc file.
#   # This is a comment in the rc file. '-' should begin on col 1.
#   -frecflds.1
#   -d;
#   -r\n
#   -D|
#   -R@@
# cmd assumes 1 to 1.
# $Id: fmtrec.cmd,v 1.1 1996/05/11 14:26:57 sangal Exp sangal $

require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/fmtrec.pl");
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/fmtrecsf.pl");
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/fmtnewrec.pl");
require ("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");


# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $attr, $val);
local ($fldnamefile, $dlimtr, $rlimtr, $dlimtrout, $rlimtrout, $sameinout, $rmfldnames, $noreclimtr, $yesreclimout, $newrec);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
# $fldnamefile -  name of the file containing field names.
# $dlimtr -  delimiter for field values in an input record.
# $rlimtr -  delimiter for input records.
# $dlimtrout -  delimiter for output of field values in a record.
# $rlimtrout -  delimiter for output of records.
# $sameinout -  Flag for Delimiters for input and output should be the same.
# $rmfldnames -  Flag to remove field names from the input file.
# $noreclimtr -  Flag to say there is no input record delimiter.
# $yesreclimout -  Flag to say output record delimiter should be used, even when $noreclimtr is set. Default is no, which is consulted only with -nor option.
# $newrec -  Flag to say that a new record is to be input.

# Look up .dupl_linerc file. Override defaults.
if (open (RC, '.dupl_linerc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/fmtrec.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'f') {
        $fldnamefile = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  'd') {
        $dlimtr = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  'r') {
        $rlimtr = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  'D') {
        $dlimtrout = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  'R') {
        $rlimtrout = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  '=') {
        $sameinout = 1;
      }
      elsif (substr($_,1, 1) eq  'sf') {
        $rmfldnames = 1;
      }
      elsif (substr($_,1, 1) eq  'nor') {
        $noreclimtr = 1;
      }
      elsif (substr($_,1, 1) eq  'yesR') {
        $yesreclimout = 1;
      }
      elsif (substr($_,1, 1) eq  'new') {
        $newrec = 1;
      }
      else { 
          print STDERR "Unrecognized param in rc file: $_. Ignoring.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/fmtrec.cmd');
      }
    }
    else { # 
      print STDERR "Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
     # Command line args override those of the rc file.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/fmtrec.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1, 1) eq  'f') {
      $fldnamefile = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'd') {
      $dlimtr = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'r') {
      $rlimtr = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'D') {
      $dlimtrout = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'R') {
      $rlimtrout = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  '=') {
      $sameinout = 1;
    }
    elsif (substr($ARGV[$i],1, 1) eq  'sf') {
      $rmfldnames = 1;
    }
    elsif (substr($ARGV[$i],1, 1) eq  'nor') {
      $noreclimtr = 1;
    }
    elsif (substr($ARGV[$i],1, 1) eq  'yesR') {
      $yesreclimout = 1;
    }
    elsif (substr($ARGV[$i],1, 1) eq  'new') {
      $newrec = 1;
    }
    else { 
        print STDERR "Unrecognized param: $ARGV[$i]. Ignoring.\n";
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/fmtrec.cmd');
    }
  }
  else { # Argument found. Parameters end.
    $argbegpos = $i;
    last;
  }
}

# Process the arguments in the command line then call fmtrec.
# 1. Can be used as a filter. OR
# 2. file args can be given instead of filters.

if ($argbegpos < 0) { # No args.
  &dupl_line(STDIN, STDOUT);
}
elsif (($#ARGV - $argbegpos) == 0) { # Only one arg given. Execute the command with arg and stdout.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "Unable to open file $ARGV[$argbegpos]\n" ;
  &dupl_line(INFILE, STDOUT);
  close (INFILE);
}
elsif (($#ARGV - $argbegpos) == 1) { # Two args given. Execute the command with arg1 and arg2.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "Unable to open input file $ARGV[$argbegpos]\n" ;
  open (OUTFILE, ">$ARGV[$argbegpos + 1]") ||
     die "Unable to open output file $ARGV[$argbegpos + 1]\n" ;
  &dupl_line(INFILE, OUTFILE);
  close (OUTFILE);
  close (INFILE);
}
else { print STDERR "Too many arguments.\n"; }