# Given two sentences a and b in $snta and $sntb, tries to align the words 
# in the respective sentences. Produces an alignment structure 
#   (@sta, @enda, @stb, @endb, @score)  No. of entries = $algncount
# Example sentences are INA and INB below.
#   INA:<1> sone kA mAlA Kelawe hue pITa kA Upara A kara girA huA varRa 
#                            kA baccA pApU ko SAMwa Age KIMca liyA .
#   INB:    sone kI mAlA Kelawe hue pITa ke Upara A kara gire hue eka varRa 
#                            ke bacce pApU ko SAMwA  ne Age KIMca liyA .
# Align structure below: First col shows: 0th word of INA (<1>) does not
#   match any word in INB (-1). Second coloumn shows: the 1st word 
#   of INA (sone) matches 0th word of INB (sone).
# STA:  0 1 2 3 4 5 6 7 8 9 10 11 12 -1 13 14 15 16 17 18 -1 19 20 21 22
# ENDA: 0 1 2 3 4 5 6 7 8 9 10 11 12 -1 13 14 15 16 17 18 -1 19 20 21 22
# STB: -1 0 1 2 3 4 5 6 7 8 9  10 11 12 13 14 15 16 17 18 19 20 21 22 23
# ENDB:-1 0 1 2 3 4 5 6 7 8 9  10 11 12 13 14 15 16 17 18 19 20 21 22 23
# SCORE:79 100 95 100 100 100 100 95 100 100 100 97.5 96.7 79 100 95 98.1 100 100 80 79 100 100 100 100
# Compact output of the above:
# <1>[^-] sone kA[^kI] mAlA Kelawe hue pITa kA[^ke] Upara A kara girA[^gire] huA[^hue] [^+eka] varRa kA[^ke] baccA[^bacce] pApU ko SAMwa[^SAMwA] [^+ne] Age KIMca liyA . 

# Match the given two sentences in @bufa and @bufb and align the
# words in the alignment structure.
sub matchsnt {
 local ($ia, $ib, $lena, $lenb);            # Input related vars.
 local ($type, $val, $score, $score1, $tmp);         # Temp variables.

 $algncount = 0;
 $lena = @bufa;
 $lenb = @bufb;
 if ($deb0) { print "MATCHSNT:LENA:$lena:LENB:$lenb:\nBUFA:@bufa:\nBUFB:@bufb:\n"; }
 $ia = 0;  $ib = 0;
 WORD: while (($ia < $lena) && ($ib < $lenb)) {
   if ($deb0 || $deb1) { print "MATCHSNT: In while loop:BUFAelem  :$bufa[$ia]:  BUFBelem   :$bufb[$ib]:\n"; }

   if ($deb1) { print "case 1\n";} # Case 1
   $score = &matchwrd ($bufa[$ia], $bufb[$ib]);
   $score1 = $score;
   if ($score >= $good) {
      &addalgn ( $ia, $ia, $ib, $ib, $score); 
           # Add an entry to alignment vectors 
           # i.e. add an entry to the alignement structure.
      $ia++;  $ib++;
      next WORD;
   }
 
   if ($deb1) { print "case 2\n";} # Case 2
   for ($offset=1; $offset <= 2; $offset++) {
     $score = &matchwrd ($bufa [$ia], $bufb [$ib + $offset]);
     if ($score >= $good) {
       for ($tmp=0; $tmp < $offset; $tmp++) {
          &addalgn ( -1, -1, $ib+$tmp, $ib+$tmp, $score - 21);  
            # Add no match entry (Something was inserted in sentence)
       }
       &addalgn ( $ia, $ia, $ib+$offset, $ib+$offset, $score); 
            # Add an entry to alignment vectors 
            # i.e. add an entry to the alignement structure.
       $ia++ ;  $ib = $ib + $offset + 1;
       next WORD;
     }
   } # End for loop

   if ($deb1) { print "case 3\n";} # Case 3
   for ($offset=1; $offset <= 2; $offset++) {
     $score = &matchwrd ($bufa [$ia + $offset], $bufb [$ib]);
     if ($score >= $good) {
       for ($tmp=0; $tmp < $offset; $tmp++) {
          &addalgn ( $ia+$tmp, $ia+$tmp, -1, -1, $score - 21);  
            # Add no match entry (Something was deleted from sentence)
       }
       &addalgn ( $ia+$offset, $ia+$offset, $ib, $ib, $score); 
            # Add an entry to alignment vectors 
            # i.e. add an entry to the alignement structure.
       $ia = $ia + $offset + 1;  $ib++;
       next WORD;
     }
   } # End for loop

   if ($deb1) { print "case 4\n";} # Case 4
   &addalgn ( $ia, $ia, $ib, $ib, $score1 );
   $ia++; $ib++;
 } # End while loop

 # Handle remaining elements if any in the sentences.
 if ($ia < $lena) { # words in sentence a remain
   for ( ; $ia < $lena; $ia++) { &addalgn ($ia, $ia, -1, -1, 79); }
 }
 if ($ib < $lenb) { # words in sentence b remain
   for ( ; $ib < $lenb; $ib++) { &addalgn ( -1, -1, $ib, $ib, 79); }
 }
} # End sub matchsnt 

# Takes 5 numbers as args representing one new alignment entry. 
# Adds them to the alignment structure.
sub addalgn {
 if ($deb1) { print "ADDALGN:@_:\n"; }
 ($sta[$algncount], $enda[$algncount], $stb[$algncount], $endb[$algncount], $score[$algncount]) = @_;
 $algncount++;
}

# Match the two argument words.
sub matchwrd {
  local ($wa, $wb) = @_;
  local ($la, $lb, $i, $points, $lmin, $lmax);
  if ($deb0) { print "MATCHWRD:ARGS:@_:\n"; }
  if ($deb0) { print "WRDA:$wa:\nWRDB:$wb:\n"; }
  if ($wa eq $wb) { 100; }
  else {
    $la = length ($wa);
    $lb = length ($wb);
    if ($la <= $lb) { $lmin = $la; $lmax = $lb;} 
               else { $lmin = $lb; $lmax = $la;}
    if ($deb0) { print "lmax = $lmax, lmin = $lmin\n";}
    for ($i = 0; $i < $lmin; $i++) { # Check for match char by char.
      if (substr ($wa, $i, 1) eq substr ($wb, $i, 1)) { $points++; } 
    }
    if ($points != $lmin) { # Score is not maximum possible.
      # Check whether mismatch because of language considerations.
      if (&lang_ok ($wa, $wb)) { $points = $points + .9; }
    }
    elsif ($lmin != $lmax) { # Score was max possible, but lengths of
                             # words was different.
      if (&lang_ok ($wa, $wb)) { $points = $points + .9; }
    }
    $points * 100 / $lmax;  # Return match value.
  }
} # End sub matchwrd


# Checks whether two different words are similar based on
# language knowledge. Implemented simple gender test only.
# Ex.: ladakA is same as ladake.
sub lang_ok {
  local ($wa, $wb) = @_;
  if (($wa =~ /[AeI]$/) && ($wb =~ /[AeI]$/)) { 1; }
}
1;

