#!/usr/bin/perl
# Takes the punjabi-hindi dictionary and tries to identify
# entries that have a same or similar Hindi word for the punjabi
# word. Similar words and entries are written to specific named
# files, while the different entries go to stdout.
#   Example usage: tdass.pl < IN  >OUT
# (Creates a number of files with remaining words.)

$vowels = 'MHaAiIuUqeEoO';
$vowels_short = 'aiuqeo';
$vowels_long =  'AIUQEO';
$conson = 'kKgGfcCjJFtTdDNwWxXnpPbBmyrlvSRsh';
$conson_alpa = 'kgcjtdwxpb';
$conson_mahA = 'KGCJTDWXPB';

$filecr = 0;   # If 1, create output files whenever different patterns match.
if ($filecr == 1) {
  open (EQL, ">eql.dd") || die "Unable to open file eql.dd\n";
  open (RI, ">ri.dd") || die "Unable to open file ri.dd\n";
  open (NUKT, ">nukt.dd") || die "Unable to open file nukt.dd\n";
  open (NN, ">Nn.dd") || die "Unable to open file Nn.dd\n";
  open (KR, ">kR.dd") || die "Unable to open file kR.dd\n";
  open (GIA, ">giA.dd") || die "Unable to open file giA.dd\n";
  open (AA, ">Aa.dd") || die "Unable to open file Aa.dd\n";
  open (CCLUST, ">cclust.dd") || die "Unable to open file cclust.dd\n";
  open (TS, ">tS.dd") || die "Unable to open file tS.dd\n";
  open (VOWELS, ">vowels.dd") || die "Unable to open file vowels.dd\n";
}
while(<>) {
  $origline = $_;     s/\"//g;
  s/z/M/g;   # sure
  s/R/S/g;   # sure
  @fields = split(/,/);
  $fout = STDOUT;
  if (1 == &match()) { $fout = EQL; goto PR; }
  $fields[0] =~ s/KZ/K/g;  # Ignore nukta in punjabi word
  $fields[0] =~ s/gZ/g/g;
  $fields[0] =~ s/jZ/j/g;
  $fields[0] =~ s/PZ/P/g;
  if (1 == &match()) { $fout = NUKT; goto PR; }
  $fields[3] =~ s/([$conson])q/$1ari/og;   # Convert 'q' in hindi
  if (1 == &match()) { $fout = RI; goto PR; }
  $fields[0] =~ s/N/n/g;
  if (1 == &match()) { $fout = NN; goto PR; }
  $fields[3] =~ s/kR/K/g;
  if (1 == &match()) { $fout = KR; goto PR; }
  $fields[3] =~ s/jFA/giA/g;
  if (1 == &match()) { $fout = GIA; goto PR; }
  $fields[3] =~ s/A/a/g;
  if (1 == &match()) { $fout = AA; goto PR; }
  $fields[3] =~ s/i/a/g;
  if (1 == &match()) { $fout = AA; goto PR; }
  $fields[3] =~ s/([$conson])([$conson])/$1a$2/og;
     #print STDERR "DEBUG: $fields[3] FROM $origline";
  if (1 == &match()) { $fout = CCLUST; goto PR; }
     #$hindiword =~ s/yA/iA/g; # 

  $fields[3] =~ s/tt/tT/g;
  $fields[3] =~ s/Sy/SS/g;
  if (1 == &match()) { $fout = TS; goto PR;};

  $fields[3] =~ tr/[aiuqeo]/[AIUQEO]/;
  #$punjnew = $fields[0];    # Comment it (Not needed if last step) VV
  $fields[0] =~ tr/[aiuqeo]/[AIUQEO]/;
  if (1 == &match()) { #  $fields[0] = $punjnew;  #VV
    $fout = VOWELS; goto PR;
  }
  #$fields[0] = $punjnew;  #VV

  PR:  print $fout $origline;
}

# $fields[0] - punjabi word (after some substitutions)
# $fields[3] - equivalent hindi word(s) (after some substitutions)
# Returns 1 if the punjabi word matches with one of the hindi words.
sub match {
  local ($i);
  if (!($fields[3] =~ /\//)) {  # Single Hindi word.
    $subfields[0] = $fields[3];   $#subfields = 0;  
  }
  else { @subfields = split(/\//, $fields[3]); }

  for ($i=0;  $i <= $#subfields;  $i++) {
    if ($fields[0] eq $subfields[$i]) { return (1); }
  }
  return (0);
}
