#! /usr/bin/perl
# Given two sentences a and b in $snta and $sntb, tries to align the words 
# in the respective sentences. Produces an alignment structure 
#   (@sta, @enda, @stb, @endb, @score)  No. of entries = $algncount
# Example sentences are in file handles $fin1 and $fin2 below.
#   $fin1:<1> sone kA mAlA Kelawe hue pITa kA Upara A kara girA huA varRa 
#                            kA baccA pApU ko SAMwa Age KIMca liyA .
#   $fin2:    sone kI mAlA Kelawe hue pITa ke Upara A kara gire hue eka varRa 
#                            ke bacce pApU ko SAMwA  ne Age KIMca liyA .
# Align structure below: First col shows: 0th word of $fin1 (<1>) does not
#   match any word in $fin2 (-1). Second coloumn shows: the 1st word 
#   of $fin1 (sone) matches 0th word of $fin2 (sone).
# STA:  0 1 2 3 4 5 6 7 8 9 10 11 12 -1 13 14 15 16 17 18 -1 19 20 21 22
# ENDA: 0 1 2 3 4 5 6 7 8 9 10 11 12 -1 13 14 15 16 17 18 -1 19 20 21 22
# STB: -1 0 1 2 3 4 5 6 7 8 9  10 11 12 13 14 15 16 17 18 19 20 21 22 23
# ENDB:-1 0 1 2 3 4 5 6 7 8 9  10 11 12 13 14 15 16 17 18 19 20 21 22 23
# SCORE:79 100 95 100 100 100 100 95 100 100 100 97.5 96.7 79 100 95 98.1 100 100 80 79 100 100 100 100
# Compact output of the above:
# <1>[^-] sone kA[^kI] mAlA Kelawe hue pITa kA[^ke] Upara A kara girA[^gire] huA[^hue] [^+eka] varRa kA[^ke] baccA[^bacce] pApU ko SAMwa[^SAMwA] [^+ne] Age KIMca liyA . 

require ("/home/bharathi/PNJ1/usr3/sangal/tools/src/anu/matchsnt.pl");  # Matching routines

sub align {
  local ($fin1, $fin2, $fout) = @_;
  $good = 75;
  $deb0 = 0;  $deb1 = 0;    $debspl = 0;
  $deb9 = 0;
  
  while ( $snta = <$fin1>) {
    $sntb = <$fin2>;
        # old: if ($snta =~ /^\s+/) { $snta = $';}# Remove opening space.
        # old: if ($sntb =~ /^\s+/) { $sntb = $';}
        # old: if ($snta =~ /\s+$/) { $snta = $`;}# Remove trailin space.
        # old: if ($sntb =~ /\s+$/) { $sntb = $`;}
        # old: @bufb = split (/[_\s]+/, $sntb);
    @bufa = &splitter ($snta, "_");
    @bufb = &splitter ($sntb, "_");
        if ($deb0) {
          print "$fin1:$snta:\n$fin2:$sntb:\n";
          print "BUFA:@bufa:\nBUFB:@bufb:\n";
          $tmplena = @bufa;
          $tmplenb = @bufb;
          print "MAIN:LENA :$tmplena: LENB :$tmplenb:\nBUFA:@bufa:\nBUFB:@bufb:\n";
        }
        if ($deb9 && (! $deb0)) {
          print "$fin1:$snta:\n$fin2:$sntb:\n";
        }
    &matchsnt ();
        if ($deb9) { 
          for ($j=0; $j < $algncount; $j++) { printf "%2d ", $sta[$j];} print "\n";
          for ($j=0; $j < $algncount; $j++) { printf "%2d ", $enda[$j];} print "\n";
          for ($j=0; $j < $algncount; $j++) { printf "%2d ", $stb[$j];} print "\n";
          for ($j=0; $j < $algncount; $j++) { printf "%2d ", $endb[$j];} print "\n";
          for ($j=0; $j < $algncount; $j++) { printf "%3d ", $score[$j];} print "\n";
             # print "@sta\n@enda\n@stb\n@endb\n@score\n"; 
        }
    &printalgn ();
    print "\n";
  }
}  # End sub align


# Split sentences into words using white space and possibly '_'.
# depending on whether 2nd arg is '_' or null.
# Ignores words inside [...] for comparision purpose.
sub splitter {
  local ($_, $parm) = @_;
  local (@buf, $i);
  $i = 0;
  if (/^[\s]+/) { $_ = $';}   # Remove opening white space.
  if (/\s+$/) { $_ = $`;}   # Remove trailing white space.
  while ($_ ne "") {
    if ($debspl) { print "SPLITTER :$_:\n"; }
    if (/(\[|_|[\s]+)/) { # word found.
        # Ignoring $parm for the time being in above.
      $buf[$i] = $`;
      $_ = $';
      if ($& eq "[") { # Ignore spaces etc inside [...].
        $buf[$i] .= '[';
        if (/\]/) { $buf[$i] .= $`."]";  $_ = $'; }
        else { print "splitter: Unmatched opening bracket '['\n"; }
      }
      $i++; 
    }
    else { # last word.
      if ($debspl) { print "Last word :$_:\n"; }
      $buf[$i] = $_;  $i++;  $_ = "";  last;
    }
  }
  @buf;
}

# Prints the aligned words in snta and sntb together with the
# following logic: If for an alignement entry the following are
# the values of sta, stb, score then: (currently enda, endb not used)
# ---------------4 cases--------------		  where
# sta	m	m	m	-1		m,n != -1
# stb	n	n	-1	n
# score	100	<100		
#
# print	a	a[^b]	a[^-]	[^+b]		a=bufa[m], b=bufb[n]
# means a=b,  a repl b,  del a,	insert b
# ------------------------------------
sub printalgn {
  local ($cc, $i, $chars);   # $cc stands for alignment entry.
  for ($cc=0; $cc < $algncount; $cc++) {
    # Print words of snta
    if ($sta[$cc] != -1) {
      print "$bufa[$sta[$cc]]";
      for ($i=$sta[$cc]+1; $i <= $enda[$cc]; $i++) {
        print "_$bufa[$i]";
      }
    }
    if ($stb[$cc] == -1) { print "[^-] "; next; } # case 3 (in table above)
    if ($score[$cc] == 100) { print " "; next; }    # case 1
    # Print words of sntb, if score less than 100 (or snta not printed).
    if ($sta[$cc] == -1) { $chars = "^+"; } else { $chars = "^"; }
    print "\[$chars$bufb[$stb[$cc]]";
    for ($i = $stb[$cc]+1; $i <= $endb[$cc]; $i++) {
      print "_$bufb[$i]";
    }
    print "\] ";
  }
}

