#!/usr/bin/perl
#%COMMENT
# Takes the punjabi-hindi dictionary and tries to identify
# entries that have a same or similar Hindi word for the punjabi
# word. Similar words and entries are written to specific named
# files, while the different entries go to stdout.
#   Example usage: tdass.pl < IN  >OUT
# (Creates a number of files with remaining words.)
#%CMD TYPE M1

$filecr = 1;   # Create output files whenever different patterns match. (default: single output file) #OPT -c sw1
$vowels = 'MHaAiIuUqeEoO';
$vowels_short = 'aiuqeo';
$vowels_long =  'AIUQEO';
$conson = 'kKgGfcCjJFtTdDNwWxXnpPbBmyrlvSRsh';
$conson_alpa = 'kgcjtdwxpb';
$conson_mahA = 'KGCJTDWXPB';

if ($filecr) {
  open (EQL, ">eql.dd") || die "Unable to open out file eql.dd\n";
  open (RI, ">ri.dd") || die "Unable to open out file ri.dd\n";
  open (NUKT, ">nukt.dd") || die "Unable to open out file nukt.dd\n";
  open (NN, ">Nn.dd") || die "Unable to open out file Nn.dd\n";
  open (KR, ">kR.dd") || die "Unable to open out file kR.dd\n";
  open (GIA, ">giA.dd") || die "Unable to open out file giA.dd\n";
  open (IA, ">iA.dd") || die "Unable to open out file iA.dd\n";
  open (CCLUST, ">cclust.dd") || die "Unable to open out file cclust.dd\n";
  open (TS, ">tS.dd") || die "Unable to open out file tS.dd\n";
  open (VOWELS, ">vowels.dd") || die "Unable to open out file vowels.dd\n";
}
else {
  open (M_ALL, ">m_all.dd") || die "Unable to open out file m_all.dd\n";
}

while(<>) {
  $origline = $_;     s/\"//g;
  s/z/M/g;   # sure
  s/R/S/g;   # sure
  @fields = split(/,/);
  $fout = STDOUT;
  $matched = 0;
    if (&match()) { $fout = EQL; $matched = 1;  goto PR; }
  $fields[0] =~ s/KZ/K/g;  # Ignore nukta in punjabi word
  $fields[3] =~ s/KZ/K/g;  # Ignore nukta in Hindi word
  $fields[0] =~ s/gZ/g/g;
  $fields[0] =~ s/jZ/j/g;
  $fields[0] =~ s/PZ/P/g;
    if (&match()) { $fout = NUKT; $matched = 1;  goto PR; }
  $fields[3] =~ s/([$conson])q/$1ari/og;   # Convert 'q' in hindi
    if (&match()) { $fout = RI; $matched = 1;  goto PR; }
  $fields[0] =~ s/N/n/g;
  $fields[3] =~ s/N/n/g;
    if (&match()) { $fout = NN; $matched = 1;  goto PR; }
  $fields[3] =~ s/kS/K/g;  # Using kS because earlier R changed to S.
    if (&match()) { $fout = KR; $matched = 1;  goto PR; }
  $fields[3] =~ s/jFA/giA/g;
    if (&match()) { $fout = GIA; $matched = 1;  goto PR; }
  $fields[3] =~ s/tt/tT/g;
  $fields[3] =~ s/Sy/SS/g;
    if (&match()) { $fout = TS; $matched = 1;  goto PR; }
  $fields[3] =~ s/([$conson])([$conson])/$1a$2/og;
     #print STDERR "DEBUG: $fields[3] FROM $origline";
    if (&match()) { $fout = CCLUST; $matched = 1;  goto PR; }

  $fields[3] =~ s/yA/iA/g;
    if (&match()) { $fout = IA; $matched = 1;  goto PR; }

  $fields[3] =~ tr/[aiuqeo]/[AIUQEO]/;
  $fields[0] =~ tr/[aiuqeo]/[AIUQEO]/;
    if (&match()) { $fout = VOWELS; $matched = 1;  goto PR;}
  $fields[3] =~ s/I/A/g;
  $fields[0] =~ s/I/A/g;
    if (&match()) { $fout = IA; $matched = 1;  goto PR; }

  PR: if (!($filecr)) { 
    if ($matched) { $fout = M_ALL; }
    #else { print $fout "DEBUG: $fields[0], $fields[3],\n"; }
  }
  #print $fout "DEBUG: $fields[0], $fields[3],\n";
  print $fout $origline;
}

# $fields[0] - punjabi word (after some substitutions)
# $fields[3] - equivalent hindi word(s) (after some substitutions)
# Returns 1 if the punjabi word matches with one of the hindi words.
sub match {
  local ($i);
  if (!($fields[3] =~ /\//)) {  # Single Hindi word.
    $subfields[0] = $fields[3];   $#subfields = 0;  
  }
  else { @subfields = split(/\//, $fields[3]); }

  for ($i=0;  $i <= $#subfields;  $i++) {
    if ($fields[0] eq $subfields[$i]) { return (1); }
  }
  return (0);
}
