#!/usr/bin/perl
# Takes the punjabi-hindi dictionary and tries to identify
# entries that are dissimilar Hindi word for the punjabi
# word.
#   Example usage: dictdiff_pnj.pl < IN  >OUT
# Also creates a file called 'rem' with remaining words.

# Options:
# -p	-  Maximum permissible diff in consonents. 

require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $arg, $outf, $argsinfile, $Finname, $Foutname);
local ($permit_diff);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$argsinfile = 0;  # Default args file names are not in a file.
$permit_diff = 2; # Default  Maximum permissible diff in consonents. 

# Look up .dictdiff_pnjrc file. Override defaults.
if (open (RC, '.dictdiff_pnjrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anu/dictdiff_pnj.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'p') {
        $permit_diff = substr ($_, 2);
      }
      else { 
          print STDERR "dictdiff_pnj.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anu/dictdiff_pnj.cmd');
          die;
      }
    }
    else { # 
      print STDERR "dictdiff_pnj.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anu/dictdiff_pnj.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1,1) eq 'o') { # Specifies output file.
      $outfilename = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'p') {
      $permit_diff = substr ($ARGV[$i], 2);
    }
    else { print STDERR "dictdiff_pnj.cmd: Unrecognized param: $ARGV[$i].\n";
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anu/dictdiff_pnj.cmd');
      die;
    }
  }
  else { # Argument found. Parameters end.
    if (substr($ARGV[$i],0,1) eq '@') {  # A file of args found
      $argsinfile = 1;
      $ARGV[$i] = substr($ARGV[$i], 1);
    }
    $argbegpos = $i;
    last;
  }
}

#print STDERR "dictdiff_pnj.cmd: DEB params end: $parmlastpos;  args begin: $argbegpos; total: $#ARGV;\n";

# Process the arguments in the command line then call dictdiff_pnj.
# 1. Can be used as a filter. OR
# 2. If arg file names are given, they are treated as input files
#    and the output goes to a single file stdout.

if ($outfilename ne '') {
  open (OUT, ">$outfilename") ||
    die "dictdiff_pnj.cmd: Unable to open output file $outfilename.\n";
  $outf = OUT;
  $Foutname = $outfilename;
}
else { $outf = STDOUT; }

if ($argbegpos < 0) { # No args.
  &dictdiff_pnj(STDIN, $outf);
}
elsif ($argsinfile) {  # Argument filenames in file.
  open (ARGFILE, "$ARGV[$argbegpos]") ||
    die "dictdiff_pnj.cmd: Unable to open args file $ARGV[$argbegpos]\n" ;
  while (<ARGFILE>) { # Read args.
    chop;
    if (!(open (INFILE, $_))) {
      print STDERR "dictdiff_pnj.cmd: Unable to open file $_\n" ;
      next;
    }
    $Finname = $_;
    &dictdiff_pnj (INFILE, $outf);
    close (INFILE);
  }
  close (ARGFILE);
}
else { # Call the program with each arg. 
  for ($arg = $argbegpos; $arg <= $#ARGV; $arg++) {
    if (!(open (INFILE, "$ARGV[$arg]"))) {
      print STDERR "dictdiff_pnj.cmd: Unable to open file $ARGV[$arg]. Continuing.\n" ;
      next;
    }
    $Finname = $ARGV[$arg];
    &dictdiff_pnj (INFILE, $outf);
    close (INFILE);
  }
}
close ($outf);
 
#------------------------------------------------------

sub dictdiff_pnj {
local ($W_fin, $W_fout) = @_;
select($W_fout);
#%COMMENT
# Takes the punjabi-hindi dictionary and tries to identify
# entries that are dissimilar Hindi word for the punjabi
# word.
#   Example usage: dictdiff_pnj.pl < IN  >OUT
# Also creates a file called 'rem' with remaining words.

#%CMD TYPE M1

$vowels = 'MHaAiIuUqeEoO';
$vowels_short = 'aiuqeo';
$vowels_long =  'AIUQEO';
$conson = 'kKgGfcCjJFtTdDNwWxXnpPbBmyrlvSRsh';
$conson_alpa = 'kgcjtdwxpb';
$conson_mahA = 'KGCJTDWXPB';
#open (REM, ">rem.dict") || die "Unable to open output file rem.dict\n";

LINE: while($a = <STDIN>) {
  $b = $a;
  @fields = split(/,/, $a);
  $fields[0] =~ s/[$vowels]//og;
  $fields[3] =~ s/[$vowels]//og;
  if ($fields[3] =~ /\//) { @subfields = split(/\//, $fields[3]); }
  else { $subfields[0] = $fields[3]; }
  $lenpunj = length ($fields[0]);
  # If the punjabi word is found to be equal or similar to any of
  #  the hindi equivalents, go to the 'next LINE' of the dictionary.
  for($i=0;  $i <= $#subfields;  $i++){
    if($fields[0] eq $subfields[$i]) { 
      #print STDERR "DEBUG: Identical: $b";
      next LINE;}
    $lenhindi = length ($subfields[$i]);
    if ($lenpunj < $lenhindi) { 
      $lenmin = $lenpunj;    $lendiff = $lenhindi - $lenpunj; }
    else { $lenmin = $lenhindi;   $lendiff = $lenpunj - $lenhindi; }
    if ($lendiff <= $permit_diff) {
      $match_cnt = 0;
      for ($j=0;  $j <= $lenmin;  $j++) {
        if (substr ($fields[0], $j, 1) eq substr($subfields[$i], $j, 1)) {
          $match_cnt++;
        } 
      }
      if (($lenmin - $match_cnt + $lendiff) <= $permit_diff)  { 
        #print STDERR "DEBUG: Similar: $b";
        #print STDERR "i=$i, $subfields[$i], match=$match_cnt, \n";
        next LINE;
      }
    }
  }
  print $b;
}
#close REM;

}
1;
