#!/usr/bin/perl
# Takes the punjabi-hindi dictionary and tries to identify
# entries that have a same or similar Hindi word for the punjabi
# word. Similar words and entries are written to specific named
# files, while the different entries go to stdout.
#   Example usage: tdass.pl < IN  >OUT
# (Creates a number of files with remaining words.)

$vowels = 'MHaAiIuUqeEoO';
$conson = 'kKgGfcCjJFtTdDNwWxXnpPbBmyrlvSRsh';
$conson_alpa = 'kgcjtdwxpb';
$conson_mahA = 'KGCJTDWXPB';

open (EQL, ">eq.d") || die "Unable to open file eq.d\n";
open (RI, ">ri.d") || die "Unable to open file ri.d\n";
open (NUKT, ">nuk.d") || die "Unable to open file nuk.d\n";
open (NN, ">Nn.d") || die "Unable to open file Nn.d\n";
open (KR, ">kR.d") || die "Unable to open file kR.d\n";
open (GIA, ">giA.d") || die "Unable to open file giA.d\n";
open (AA, ">Aa.d") || die "Unable to open file Aa.d\n";
open (CCLUST, ">cclust.d") || die "Unable to open file cclust.d\n";
open (TS, ">tS.d") || die "Unable to open file tS.d\n";

while(<>) {
  $origline = $_;     s/\"//g;
  s/z/M/g;   # sure
  s/R/S/g;
  @fields = split(/,/);
  if (!($fields[3] =~ /\//)) {  # Single Hindi word.
    $subfields[0] = $fields[3];   $#subfields = 0;  
  }
  else { @subfields = split(/\//, $fields[3]); }

  $fout = STDOUT;
  for ($i=0;  $i <= $#subfields;  $i++) {
    if ($fields[0] eq $subfields[$i]) { $fout = EQL;  goto PR; }
  }
  $fields[0] =~ s/KZ/K/g;  # Ignore nukta
  $fields[0] =~ s/gZ/g/g;
  $fields[0] =~ s/jZ/j/g;
  $fields[0] =~ s/PZ/P/g;
  for ($i=0;  $i <= $#subfields;  $i++) {
    $hindiword = $subfields[$i];

    if ($hindiword eq $fields[0]) { $fout = NUKT;  last; }

    $hindiword =~ s/([$conson])q/$1ari/og;   # Convert 'q' suitably.   # Less sure
    if ($hindiword eq $fields[0]) { $fout = RI;  last; }

    $hindiword =~ s/n/N/g;
    if ($hindiword eq $fields[0]) { $fout = NN;  last; }
    $hindiword =~ s/kR/K/g;  # Further trials
    if ($hindiword eq $fields[0]) { $fout = KR;  last; }
    $hindiword =~ s/jFA/giA/g;  # Further trials
    if ($hindiword eq $fields[0]) { $fout = GIA;  last; }

    $hindiword =~ s/A/a/g;   # Even less sure
    if ($hindiword eq $fields[0]) { $fout = AA;  last; }
    $hindiword =~ s/i/a/g;
    if ($hindiword eq $fields[0]) { $fout = AA;  last; }

    $hindiword =~ s/([^,\- MHaAiIuUeEoO])([^,\- MHaAiIuUeEoO])/$1a$2/og; # Do after Z removal
    if ($hindiword eq $fields[0]) { $fout = CCLUST;  last; }
    #$hindiword =~ s/yA/iA/g; # 

    $hindiword =~ s/tt/tT/g;
    $hindiword =~ s/Sy/SS/g;
    if ($hindiword eq $fields[0]) { $fout = TS;  last; }
  }
  PR:  print $fout $origline;
}

