#!/usr/bin/perl
#%COMMENT
# Given a file of word forms, the likely roots underlying the word
# forms, and a file of possible paradigms filenames; 
# guesses which root and paradigm give the best result.
# Example usage: 
#   pardgm_guess.cmd  -rbanXu.pre  -ppardgm.nam  -wbanXu.wrd  > banXu.par
# (Actually, usually it is given a file of prefixes rather than
# a file of roots. So it makes a guess by assuming that the root is
# one extra vowel character at the end.)
require ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anu/pardgm_guessaux.pl');
#%CMD TYPE 11
$rootfile = 'NONE';  # File name containing likely roots. #OPT -r
$pardgm_fnames = 'NONE';  # File name containing names of paradigm files (del add strings). #OPT -p
$wrdfile = 'NONE';  # File name containing word forms. #OPT -w
$exact_match = 0;  # If flag set, does exact match with pardgms. #OPT -e sw1

@vowel = (a,A,i,I,u,U,e,E,o,O,M,H,z);
$vowel = 'aAiIuUeEoOMHz';
open (ROOTFILE, "$rootfile") ||
  die "pardgm_guess.pl: Unable to open file $rootfile: $!\n";
$i = -1;
while (<ROOTFILE>) {  # Read all the roots with their counts.
  chop;  $i++;
  ($cnt, $rt) = split (/\s+/);
  #print STDERR  "DEBUG: cnt=$cnt, root=$rt,\n";
  $root[$i] = $rt;  $count[$i] = $cnt;
  if ($exact_match) { # Exact match is desired with paradigms.
    $rt =~ /.$/;  $lastchar = $&;
    if (!($vowel =~  /$lastchar/)) { # Last char is not a vowel.
      foreach $vwl (@vowel) { # Add a different vowel and gen roots.
        $i++;   $root[$i] = $rt . $vwl;   $count[$i] = $cnt;
      }
    }
  }
}
open (PARDGM_FNAMES, "$pardgm_fnames") ||
  die "pardgm_guess.pl: Unable to open file $pardgm_fnames: $!\n";
$i = -1;
while (<PARDGM_FNAMES>) {  # Read all the pardgm file names.
  chop;   $i++;   $pardgm_filename[$i] = $_;
}
$/ = 'abcABC^@& #*';
for ($i=0;  $i <= $#pardgm_filename;  $i++)  { # Now read the paradigms
  open (PARDGM, "$pardgm_filename[$i]") ||
    die "pardgm_guess.pl: Unable to open file $pardgm_filename[$i]: $!\n";
  $pardgm[$i] = <PARDGM>;
  close (PARDGM);
}
if (!($exact_match)) {  # Prepare to do approx match
  for ($i=0;  $i <= $#pardgm_filename;  $i++)  { # Now read the root paradigms
     open (PARDGM, "$pardgm_filename[$i].r") ||
       print STDERR "pardgm_guess.pl: Unable to open file $pardgm_filename[$i]: $!. Continue.\n";
    $pardgm_r[$i] = <PARDGM>;
    close (PARDGM);
  }
}
$/ = "\xa";

$best_prefx = 0;
for ($r=0;  $r <= $#root;  $r++) { # Now try the roots one by one.
   #print STDERR 
   #  "DEBUG: /home/bharathi/PNJ1/usr3/sangal/tools/src/anu/pardgm_deladd.cmd -ptor",
   #  " -r$root[$r]   < $wrdfile > $wrdfile.add.tmp", "\n";
  system ("/home/bharathi/PNJ1/usr3/sangal/tools/src/anu/pardgm_deladd.cmd -r$root[$r] < $wrdfile > $wrdfile.add.tmp");
   #system ("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/names2cmds.cmd  -f'^'  -l'\$'  < $wrdfile.add.tmp > $wrdfile.pat.tmp");
   #system ("cp $wrdfile.add.tmp  $wrdfile.pat.tmp");
  open (PATS, "$wrdfile.add.tmp") ||
    print STDERR "pardgm_guess.pl: Unable to open pat file $wrdfile.add.tmp: $!\n";
  $pat_num = -1;
  while (<PATS>) { 
    chop;    $pat_num++;   
    s/\+/\\\+/;   $pat_str[$pat_num] = $_; 
    if (!($exact_match)) {  # Prepare to do approx match
      # Build additional del, add patterns if the root 
      # had one vowel more than the prefix.
      ($del, $add) = split (/,/, $pat_str[$pat_num]);
      $frstchar = substr ($add, 2, 1);  # first char after '\+'.
      if ($vowel =~  /$frstchar/) { # first char is a vowel in add string.
        $add = '\+' . substr ($add, 3);  # Remove the first char.
        # Keep 'del' string unchanged.
      }
      else { $del .= '.'; }  # 'Add' string unchanged.
      $pat_str_r[$pat_num] = $del . $add;
    }
  }
  close (PATS);
  $best_pc = 0;
  for ($pc=0;  $pc <= $#pardgm_filename;  $pc++) { # $pc means pardgm no.
    $score[$pc] = &score_str_pardgm ($pc);
    print STDERR "   DEBUG: Score for root $root[$r] pardgm $pc $pardgm_filename[$pc] = $score[$pc]\n";
    if ($score[$pc] > $score[$best_pc]) { $best_pc = $pc; }

    if (!($exact_match)) {  # Prepare to do approx match
      $rt =  $root[$r];  $rt =~ /.$/;  $lastchar = $&;
      if (!($vowel =~  /$lastchar/)) { # Last char is not a vowel.
        $score_r[$pc] = &score_str_pardgm_r ($pc);  # Do approx match
        print STDERR "   DEBUG: Score_r for root $root[$r] pardgm $pc $pardgm_filename[$pc].r = $score_r[$pc]\n";
        if ($score_r[$pc] > $score_r[$best_pc]) { $best_pc_r = $pc; }
      }
    }
       #print STDERR 
       # "DEBUG: grep  -f$wrdfile.pat.tmp  $pardgm_filename[$pc] | wc", "\n";
       #$res =  `grep  -f$wrdfile.pat.tmp  $pardgm_filename[$pc] | wc`;
       #print STDERR "DEBUG: Result of wc = $res";
       #if ($res =~ /^\s+/) { $res = $'; }
       #($score[$pc], $tmp) = split (/\s+/, $res, 2);
  }
  $score_prefx[$r] = $score[$best_pc];
  $score_prefx_for_pc[$r] = $best_pc;
  if (!($exact_match)) {  # results of approx match
    $score_root[$r] = $score_r[$best_pc_r];
    $score_root_for_pc[$r] = $best_pc_r;
  }
  print STDERR 
    "--DEBUG: For prefix $root[$r]",
    " paradigm $pardgm_filename[$score_prefx_for_pc[$r]] ",
    "highest score  = $score_prefx[$r]\n";
  if ($score_prefx[$r] > $score_prefx[$best_prefx]) { # if cur is better
    $best_prefx = $r;
  }
  if (!($exact_match)) {  # results of approx match
    print STDERR 
      "--DEBUG: For root $root[$r]",
      " paradigm $pardgm_filename[$score_root_for_pc[$r]] ",
      "highest score  = $score_root[$r]\n";
    if ($score_root[$r] > $score_root[$best_root]) { # if cur is better
      $best_root = $r;
    }
  }
}

#print STDERR "DEBUG: Best root no.= $best_prefx, ",
#   "pardgm no.= $score_root_for_pc[$best_prefx], ",
#   "score= $score_root[$best_prefx],\n";
print STDERR "Best with: prefx= $root[$best_prefx], ", 
    "pardgm= $pardgm_filename[$score_prefx_for_pc[$best_prefx]],",
    "\n";
if ($exact_match) {  # results of exact match
  print STDERR "Best with: root= $root[$best_root], ", 
    "pardgm= $pardgm_filename[$score_root_for_pc[$best_root]],",
    "\n";
}
if ($score_root[$best_root] > $score_root[$best_prefx]) {
  print "Final with: root= $root[$best_root], ", 
    "pardgm= $pardgm_filename[$score_root_for_pc[$best_root]],",
    "\n";
}
else {
  print "Final with: prefx= $root[$best_prefx], ", 
    "pardgm= $pardgm_filename[$score_prefx_for_pc[$best_prefx]],",
    "\n";
}
  
