#!/usr/bin/perl
# Takes an input file consisting of a histogram of prefixes
# computed for a set of words sorted on frequencies.
# Prunes the histograms by eliminating those entries, where
# longer prefixes are available (for the same count).
# Does it as long as prune_factor is not satisfied. (See sub cutoff
# in this file for more details.)
#   prefx_prune.pl  < TEST.histo
# 

# Options:
# -l	-  Factor (default .2) by which, if count less, prefixes are cutoff. OR say -lnone. 
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $Finname, $Foutname);
local ($prune_factor);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$prune_factor = .2; # Default  Factor (default .2) by which, if count less, prefixes are cutoff. OR say -lnone. 

# Look up .prefx_prunerc file. Override defaults.
if (open (RC, '.prefx_prunerc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anu/prefx_prune.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'l') {
        $prune_factor = substr ($_, 2);
      }
      else { 
          print STDERR "prefx_prune.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anu/prefx_prune.cmd');
          die;
      }
    }
    else { # 
      print STDERR "prefx_prune.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
     # Command line args override those of the rc file.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anu/prefx_prune.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1, 1) eq  'l') {
      $prune_factor = substr ($ARGV[$i], 2);
    }
    else { 
        print STDERR "prefx_prune.cmd: Unrecognized param: $ARGV[$i].\n";
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anu/prefx_prune.cmd');
        die;
    }
  }
  else { # Argument found. Parameters end.
    $argbegpos = $i;
    last;
  }
}

# Process the arguments in the command line then call prefx_prune.
# 1. Can be used as a filter. OR
# 2. file args can be given instead of filters.

if ($argbegpos < 0) { # No args.
  &prefx_prune(STDIN, STDOUT);
}
elsif (($#ARGV - $argbegpos) == 0) { # Only one arg given. Execute the command with arg and stdout.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "prefx_prune.cmd: Unable to open file $ARGV[$argbegpos]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = '';
  &prefx_prune(INFILE, STDOUT);
  close (INFILE);
}
elsif (($#ARGV - $argbegpos) == 1) { # Two args given. Execute the command with arg1 and arg2.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "prefx_prune.cmd: Unable to open input file $ARGV[$argbegpos]\n" ;
  open (OUTFILE, ">$ARGV[$argbegpos + 1]") ||
     die "prefx_prune.cmd: Unable to open output file $ARGV[$argbegpos + 1]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = $ARGV[$argbegpos+1];  
  &prefx_prune(INFILE, OUTFILE);
  close (OUTFILE);
  close (INFILE);
}
else { print STDERR "prefx_prune.cmd: Too many arguments.\n"; }
#------------------------------------------------------

sub prefx_prune {
local ($W_fin, $W_fout) = @_;
select($W_fout);
#%COMMENT
# Takes an input file consisting of a histogram of prefixes
# computed for a set of words sorted on frequencies.
# Prunes the histograms by eliminating those entries, where
# longer prefixes are available (for the same count).
# Does it as long as prune_factor is not satisfied. (See sub cutoff
# in this file for more details.)
#   prefx_prune.pl  < TEST.histo
# 

#%CMD TYPE 11

if ($prune_factor eq 'none') { }
elsif ($prune_factor >= .5) {
  print STDERR "prefx_prune.pl: prune factor must not be greater than .5. Assuming .45\n";
  $prune_factor = .45;
}
elsif ($prune_factor <= 0) {
  print STDERR "prefx_prune.pl: prune factor must not be less than 0. Assuming .05\n";
  $prune_factor = .05;
}

$first = 1;
while (<$W_fin>) {
  chop;
  if ($first) { 
    $first = 0;   ($prev_cnt, $prev_prefix) = split (/\s+/);
    # print STDERR  "DEBUG: cnt=$prev_cnt, prefix=$prev_prefix,\n";
  }
  else { # Other than the first time the loop is entered.
    ($cnt, $prefix) = split (/\s+/);
    #print STDERR  "DEBUG: cnt=$cnt, prefix=$prefix,\n";
    if ($cnt eq $prev_cnt) {
      if ($prev_prefix =~ /^$prefix/) { # Longer prefix found earlier.
        #print STDERR "DEBUG: Dropping: $cnt $prefix\n";
        next;
      }
    } 
    # A different prefix has appeared. Output the prev one.
    # Check whether to continue any further or to cut off
    #   (based on prune factor).
    if ($prune_factor ne 'none') { if (&cutoff ()) { last; } }
    print "$prev_cnt $prev_prefix\n";
    $prev_cnt = $cnt;   $prev_prefix = $prefix;
  } 
}
print "$prev_cnt $prev_prefix\n";

# cutoff looks at the freq count and prefix length for both
# current and previous (prev_) values, and decides whether to cutoff,
# based on the value of $prune_factor.
# Inputs: $cnt, $prefix, $prev_cnt, $prev_prefix
sub cutoff {
  local ($prev_lenprefix, $lenprefix, $eff_factor);
  $prev_lenprefix = length ($prev_prefix);
  $lenprefix = length ($prefix);
  if ($lenprefix = $prev_lenprefix + 1) {
    $eff_factor = (1 - $prune_factor);
    #print STDERR "DEBUG: effective factor=$eff_factor,\n";
    if ( $cnt > ($prev_cnt * $eff_factor) ) { return (0); }
    else { return (1); }
  }
  elsif ($lenprefix = $prev_lenprefix + 2) {
    $eff_factor = (1 - $prune_factor) * 1.5;
    #print STDERR "DEBUG: effective factor=$eff_factor,\n";
    if ( $cnt > ($prev_cnt * $eff_factor) ) { return (0); }
    else { return (1); }
  }
  else {
    $eff_factor = (1 - $prune_factor) * 2;
    #print STDERR "DEBUG: effective factor=$eff_factor,\n";
    if ( $cnt > ($prev_cnt * $eff_factor) ) { return (0); }
    else { return (1); }
  }
}
1;
}
1;
