#!/usr/bin/perl
#%COMMENT
# Takes an input file consisting of a histogram of prefixes
# computed for a set of words sorted on frequencies.
# Prunes the histograms by eliminating those entries, where
# longer prefixes are available (for the same count).
# Does it as long as prune_factor is not satisfied. (See sub cutoff
# in this file for more details.)
#   prefx_prune.pl  < TEST.histo
# 

#%CMD TYPE 11
$prune_factor = .2; # Factor (default .2) by which, if count less, prefixes are cutoff. OR say -lnone. #OPT -l

if ($prune_factor eq 'none') { }
elsif ($prune_factor >= .5) {
  print STDERR "prefx_prune.pl: prune factor must not be greater than .5. Assuming .45\n";
  $prune_factor = .45;
}
elsif ($prune_factor <= 0) {
  print STDERR "prefx_prune.pl: prune factor must not be less than 0. Assuming .05\n";
  $prune_factor = .05;
}

$first = 1;
while (<>) {
  chop;
  if ($first) { 
    $first = 0;   ($prev_cnt, $prev_prefix) = split (/\s+/);
    # print STDERR  "DEBUG: cnt=$prev_cnt, prefix=$prev_prefix,\n";
  }
  else { # Other than the first time the loop is entered.
    ($cnt, $prefix) = split (/\s+/);
    #print STDERR  "DEBUG: cnt=$cnt, prefix=$prefix,\n";
    if ($cnt eq $prev_cnt) {
      if ($prev_prefix =~ /^$prefix/) { # Longer prefix found earlier.
        #print STDERR "DEBUG: Dropping: $cnt $prefix\n";
        next;
      }
    } 
    # A different prefix has appeared. Output the prev one.
    # Check whether to continue any further or to cut off
    #   (based on prune factor).
    if ($prune_factor ne 'none') { if (&cutoff ()) { last; } }
    print "$prev_cnt $prev_prefix\n";
    $prev_cnt = $cnt;   $prev_prefix = $prefix;
  } 
}
print "$prev_cnt $prev_prefix\n";

# cutoff looks at the freq count and prefix length for both
# current and previous (prev_) values, and decides whether to cutoff,
# based on the value of $prune_factor.
# Inputs: $cnt, $prefix, $prev_cnt, $prev_prefix
sub cutoff {
  local ($prev_lenprefix, $lenprefix, $eff_factor);
  $prev_lenprefix = length ($prev_prefix);
  $lenprefix = length ($prefix);
  if ($lenprefix = $prev_lenprefix + 1) {
    $eff_factor = (1 - $prune_factor);
    #print STDERR "DEBUG: effective factor=$eff_factor,\n";
    if ( $cnt > ($prev_cnt * $eff_factor) ) { return (0); }
    else { return (1); }
  }
  elsif ($lenprefix = $prev_lenprefix + 2) {
    $eff_factor = (1 - $prune_factor) * 1.5;
    #print STDERR "DEBUG: effective factor=$eff_factor,\n";
    if ( $cnt > ($prev_cnt * $eff_factor) ) { return (0); }
    else { return (1); }
  }
  else {
    $eff_factor = (1 - $prune_factor) * 2;
    #print STDERR "DEBUG: effective factor=$eff_factor,\n";
    if ( $cnt > ($prev_cnt * $eff_factor) ) { return (0); }
    else { return (1); }
  }
}
1;
