#!/usr/bin/perl
# Input file is a list of blocks sorted on attribute, each line containg
# a word form and an attribute. Name of the stem-attribute is set in $stem_attr.
# Computes strings to be deleted and added to the stem to get each line.
#  Example usage: bl_deladd.pl < IN   > OUT

# Options:
# -s	-  Name of the attribute for which the word is the stem. 

require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $arg, $outf, $argsinfile, $Finname, $Foutname);
local ($stem_attr);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$argsinfile = 0;  # Default args file names are not in a file.
$stem_attr = 'f1'; # Default  Name of the attribute for which the word is the stem. 

# Look up .bl_deladdrc file. Override defaults.
if (open (RC, '.bl_deladdrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anumorph/bl_deladd.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  's') {
        $stem_attr = substr ($_, 2);
      }
      else { 
          print STDERR "bl_deladd.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anumorph/bl_deladd.cmd');
          die;
      }
    }
    else { # 
      print STDERR "bl_deladd.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anumorph/bl_deladd.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1,1) eq 'o') { # Specifies output file.
      $outfilename = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  's') {
      $stem_attr = substr ($ARGV[$i], 2);
    }
    else { print STDERR "bl_deladd.cmd: Unrecognized param: $ARGV[$i].\n";
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anumorph/bl_deladd.cmd');
      die;
    }
  }
  else { # Argument found. Parameters end.
    if (substr($ARGV[$i],0,1) eq '@') {  # A file of args found
      $argsinfile = 1;
      $ARGV[$i] = substr($ARGV[$i], 1);
    }
    $argbegpos = $i;
    last;
  }
}

#print STDERR "bl_deladd.cmd: DEB params end: $parmlastpos;  args begin: $argbegpos; total: $#ARGV;\n";

# Process the arguments in the command line then call bl_deladd.
# 1. Can be used as a filter. OR
# 2. If arg file names are given, they are treated as input files
#    and the output goes to a single file stdout.

if ($outfilename ne '') {
  open (OUT, ">$outfilename") ||
    die "bl_deladd.cmd: Unable to open output file $outfilename.\n";
  $outf = OUT;
  $Foutname = $outfilename;
}
else { $outf = STDOUT; }

if ($argbegpos < 0) { # No args.
  &bl_deladd(STDIN, $outf);
}
elsif ($argsinfile) {  # Argument filenames in file.
  open (ARGFILE, "$ARGV[$argbegpos]") ||
    die "bl_deladd.cmd: Unable to open args file $ARGV[$argbegpos]\n" ;
  while (<ARGFILE>) { # Read args.
    chop;
    if (!(open (INFILE, $_))) {
      print STDERR "bl_deladd.cmd: Unable to open file $_\n" ;
      next;
    }
    $Finname = $_;
    &bl_deladd (INFILE, $outf);
    close (INFILE);
  }
  close (ARGFILE);
}
else { # Call the program with each arg. 
  for ($arg = $argbegpos; $arg <= $#ARGV; $arg++) {
    if (!(open (INFILE, "$ARGV[$arg]"))) {
      print STDERR "bl_deladd.cmd: Unable to open file $ARGV[$arg]. Continuing.\n" ;
      next;
    }
    $Finname = $ARGV[$arg];
    &bl_deladd (INFILE, $outf);
    close (INFILE);
  }
}
close ($outf);
 
#------------------------------------------------------

sub bl_deladd {
local ($W_fin, $W_fout) = @_;
select($W_fout);
#%COMMENT
# Input file is a list of blocks sorted on attribute, each line containg
# a word form and an attribute. Name of the stem-attribute is set in $stem_attr.
# Computes strings to be deleted and added to the stem to get each line.
#  Example usage: bl_deladd.pl < IN   > OUT

#%CMD TYPE M1
#require ("/sangal/tools/anumorph/pdgm_d_s.pl"); # For home machine DOS
require ("/home/bharathi/PNJ1/usr3/sangal/tools/src/anumorph/pdgm_deladd_sub.pl");

if ($stem_attr eq '') { print STDERR 'Stem attribute not given.',
             " Will be chosen arbitrarily for each block.\n";
}
$end_blk = "---\n";
$/ = $end_blk;
$len_sep = length ($/);
while (<$W_fin>) {
  for ($i=1; $i <= $len_sep;  $i++) { chop; }  chop;
  $#rows = -1;  $#w = -1; $#attr = -1;
  @rows = split (/\n/);
    #print "DEB: //";  foreach $r (@rows) { print $r, '//'; }  print "\n";
  $stem = '';  $stem_found = 0;   $j = -1;
  foreach $r (@rows) {  # Split and find the stem (from the specified attr)
    $j++;
    ($w[$j], $attr[$j]) = split (/\s+/, $r);
    if ($attr[$j] eq $stem_attr) {
      if ($stem ne '') { print STDERR 'Two entries for the same',
        " attr $attr[$j] in a block: $w[$j], $stem\n";
      }
      else { $stem = $w[$j]; }
      $stem_found = 1;
    }
  }
  if (!($stem_found)) { 
    print STDERR 'Stem attribute not found for block. ',
           "Choosing first entry as stem.\n";
    $stem = $w[0];
  }
  #print STDERR "DEB: stem = $stem\n";

  # stem has been chosen. Now compute deladd strings.
  print $stem, "\n";
  for ($i=0; $i <= $#w;  $i++) {
    ($del, $add) = &find_deladd ($stem, $w[$i]);
    $line = '-' . $del . ',+' . $add;
    print $attr[$i], "\t", $line, "\n";
  }
  print $end_blk;
}
}
1;
