#!/usr/bin/perl
# Given an input file of words forms with attributes,
# grossly identifies blocks of related words, using (language 
# independent criteria) and puts block terminators.
# Two criteria are used while deciding about blocks.
# 1. Each block contains only "similar" word forms.
#    Similarity rating is based on length of prefix etc.
# 2. No attribute should be repeated within a block. 
#    (CURRENTLY NOT BEING USED)
#    (This is a bad heuristic if there are spelling variations of a
#    word with the same attribute, but can be still used to reduce
#    mistaken block groupings, otherwise a block has many unrelated 
#    words.)
#
#  Example usage: bl_find.pl  < IN   > OUT

$end_blk = "---\n";
$cur_pre = 'zzzzzzzzz';  $first = 1;
while (<>) {
  chop;
  ($w, $attr) = split (/\s+/);
  $new_pre = &find_pre ($w, $cur_pre);
  $score = &eval_score ($cur_pre, $new_pre, $w);
  if ($score <= 5) { # block ends.
    $cur_pre = $w;
    if ($first) { $first = 0; }  # First time do not print end_blk
    else { print $end_blk; }
  }
  else { $cur_pre = $new_pre; }
  print $_, "\n";
}
print $end_blk;   # Print at the end.

# Gives the score (0-9) by matching old prefix and proposed new one
# (9 means good match).
sub eval_score {
  local ($old_pre, $new_pre, $w) = @_;
  local ($l_new, $l_old);
  $l_new = length ($new_pre);
  $l_old = length ($old_pre);
  if (($l_new/$l_old) < .6) { return (1); }
  elsif ($l_new <= 2) { return (4); }
  else { return(9); }
}
# Given a new word and a current prefix, compute the new largest prefix.
sub find_pre {
  local ($w, $pre) = @_;
  $prop_pre = $pre;  # Propose the prefix same as before.
  while ($prop_pre ne '') {
    if ($w =~ /^$prop_pre/) { return ($prop_pre); }
    chop ($prop_pre);
  }
  return '';
}
