#!/usr/bin/perl
# Take the given input file of blocks (each block containing similar 
# words and their attributes) and 
# break them into finer blocks, using language knowledge.
# The earlier blocks were defined using only prefix similarity.
# Each line in input file is: word, TAB, attribute
#   Example usage: bl_refine.pl < a.blk  > b.blk
#   Example usage: bl_refine.cmd -lsim_noun.pl < a.blk  > b.blk
# In the above example, there should be a language file called 
# sim_noun.pl containing a perl sub-program called sim_word.
# This program should return 1 whenever given stem and word,
# the word is likely to be derived from the stem.
#
# Options:
# -l	-  Name of the language file for word similarity. 
# -s	-  (Ignored) Name of the attribute for which the word is the stem. 
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $Finname, $Foutname);
local ($lang_file, $stem_attr);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$lang_file = '/home/bharathi/PNJ1/usr3/sangal/eng_mo/sim_noun.pl'; # Default  Name of the language file for word similarity. 
$stem_attr = 'f1'; # Default  (Ignored) Name of the attribute for which the word is the stem. 

# Look up .bl_refinerc file. Override defaults.
if (open (RC, '.bl_refinerc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anumorph/bl_refine.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'l') {
        $lang_file = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  's') {
        $stem_attr = substr ($_, 2);
      }
      else { 
          print STDERR "bl_refine.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anumorph/bl_refine.cmd');
          die;
      }
    }
    else { # 
      print STDERR "bl_refine.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
     # Command line args override those of the rc file.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anumorph/bl_refine.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1, 1) eq  'l') {
      $lang_file = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  's') {
      $stem_attr = substr ($ARGV[$i], 2);
    }
    else { 
        print STDERR "bl_refine.cmd: Unrecognized param: $ARGV[$i].\n";
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anumorph/bl_refine.cmd');
        die;
    }
  }
  else { # Argument found. Parameters end.
    $argbegpos = $i;
    last;
  }
}

# Process the arguments in the command line then call bl_refine.
# 1. Can be used as a filter. OR
# 2. file args can be given instead of filters.

if ($argbegpos < 0) { # No args.
  &bl_refine(STDIN, STDOUT);
}
elsif (($#ARGV - $argbegpos) == 0) { # Only one arg given. Execute the command with arg and stdout.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "bl_refine.cmd: Unable to open file $ARGV[$argbegpos]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = '';
  &bl_refine(INFILE, STDOUT);
  close (INFILE);
}
elsif (($#ARGV - $argbegpos) == 1) { # Two args given. Execute the command with arg1 and arg2.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "bl_refine.cmd: Unable to open input file $ARGV[$argbegpos]\n" ;
  open (OUTFILE, ">$ARGV[$argbegpos + 1]") ||
     die "bl_refine.cmd: Unable to open output file $ARGV[$argbegpos + 1]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = $ARGV[$argbegpos+1];  
  &bl_refine(INFILE, OUTFILE);
  close (OUTFILE);
  close (INFILE);
}
else { print STDERR "bl_refine.cmd: Too many arguments.\n"; }
#------------------------------------------------------

sub bl_refine {
local ($W_fin, $W_fout) = @_;
select($W_fout);
#%COMMENT
# Take the given input file of blocks (each block containing similar 
# words and their attributes) and 
# break them into finer blocks, using language knowledge.
# The earlier blocks were defined using only prefix similarity.
# Each line in input file is: word, TAB, attribute
#   Example usage: bl_refine.pl < a.blk  > b.blk
#   Example usage: bl_refine.cmd -lsim_noun.pl < a.blk  > b.blk
# In the above example, there should be a language file called 
# sim_noun.pl containing a perl sub-program called sim_word.
# This program should return 1 whenever given stem and word,
# the word is likely to be derived from the stem.
#
#%CMD TYPE 11

require ("$lang_file");

# Variable $stem_attr is not being used.
if ($stem_attr eq '') { print STDERR 'Stem attribute not given.',
             " Will be chosen arbitrarily for each block.\n";
}

$end_blk2 = "---\n";   # Keeping end_blk2 same as end_blk
$end_blk = "---\n";
$/ = $end_blk;
$len_sep = length ($/);
while (<$W_fin>) { # Read a block at a time.
  if (/$end_blk$/) { # Remove the block end marker
    for ($i=1; $i <= $len_sep;  $i++) { chop; }  chop;
  }
  else { print STDERR "Warn: End of block not found: $_\n"; }
  $#rows = -1;  $#w = -1; $#attr = -1; $#mem = -1; $#suff = -1;
  $j = -1;
  @rows = split (/\n/);
    #print STDERR "DEB: //";  foreach $r (@rows) { print STDERR $r, '//'; }  print STDERR "\n";
  foreach $r (@rows) {  # Load each word and its attribute.
    $j++;
    ($w[$j], $attr[$j]) = split (/\s+/, $r);
  }
    #print STDERR "DEB: //";  foreach $w (@w) { print STDERR $w, '//'; }  print STDERR "\n";
  # Membership in block is stored after computation as follows:
  # $mem[$i] 
  #   = 0      means $ith elem (i.e. $w[$i]) is not in any block.
  #   = (N+1)  $ith elem is in block whose stem is in N. (0 <= N <= $#w)
  #   = -1     was in block but has been printed (used later).
  # $suff[$i]  Stores the suffix for the ith elem (i.e., $w[$i]th word in block)
  #
  for ($i=0; $i <= $#w;  $i++) { # Look for similarity among words.
    if (!($mem[$i])) { # i-th not already marked to be in a block.
      # Assume $i-th elem to be stem 
      for ($j=0; $j <= $#w;  $j++) {
        if ($i == $j) { next; }
        if (!($mem[$j])) { # j-th elem not already marked.
          $res = &sim_word ($w[$i], $w[$j]); # Compare words (1st as stem).
          # If the words are similar, set them to index of stem.
          if ($res) { 
            $mem[$j] = $i + 1;  $mem[$i] = $i + 1;  $suff[$i] = 'STEM';
            $suff[$j] = $res;
            #print STDERR "DEB: Found similar: $i $w[$i]\t$j $w[$j]\tSuff: $suff[$j]\n";
          }
        }
      }
    }
  }
  #for ($i=0; $i <= $#w; $i++) { print STDERR "DEBUG mem word attr suff: $mem[$i] $w[$i]\t$attr[$i]\t$suff[$i]\n"; }

  for ($i=0; $i <= $#w;  $i++) { # Print the new blocks.
    if ($mem[$i] == 0) {  # Not in any block. (i.e., A block by itself.)
      print "$w[$i] $attr[$i]_ALONE\n$end_blk2";
    }
    elsif ($mem[$i] eq -1) { }  # Already printed.
    else {  # print all the related words in this block.
      $found1 = 0;  # Flag to show that at least one stem/elem found
      for ($j=0;  $j <= $#w;  $j++) {   #CHECK whether: $j=$i is better
        if ($mem[$j] == ($i + 1)) { 
          print "$w[$j] $attr[$j]_$suff[$j]\n";
          #if ($mem[$j] == ($j + 1)) { print "_STEM\n"; } # This is stem
          #else { print "\n"; }
          $mem[$j] = -1;  # Mark as printed.
          $found1 = 1;  # Set flag that at least one stem/elem found.
        }
      }
      if ($found1) { print $end_blk2; }
    }
  }
}
}
1;
