#!/usr/bin/perl
#%COMMENT
# Take the given input file of blocks (each block containing similar 
# words and their attributes) and 
# break them into finer blocks, using language knowledge.
# The earlier blocks were defined using only prefix similarity.
# Each line in input file is: word, TAB, attribute
#   Example usage: bl_refine.pl < a.blk  > b.blk
#   Example usage: bl_refine.cmd -lsim_noun.pl < a.blk  > b.blk
# In the above example, there should be a language file called 
# sim_noun.pl containing a perl sub-program called sim_word.
# This program should return 1 whenever given stem and word,
# the word is likely to be derived from the stem.
#
#%CMD TYPE 11
$lang_file = '/home/bharathi/PNJ1/usr3/sangal/eng_mo/sim_noun.pl'; # Name of the language file for word similarity. #OPT -l

require ("$lang_file");

# Variable $stem_attr is not being used.
$stem_attr = 'f1';  # (Ignored) Name of the attribute for which the word is the stem. #OPT -s
if ($stem_attr eq '') { print STDERR 'Stem attribute not given.',
             " Will be chosen arbitrarily for each block.\n";
}

$end_blk2 = "---\n";   # Keeping end_blk2 same as end_blk
$end_blk = "---\n";
$/ = $end_blk;
$len_sep = length ($/);
while (<>) { # Read a block at a time.
  if (/$end_blk$/) { # Remove the block end marker
    for ($i=1; $i <= $len_sep;  $i++) { chop; }  chop;
  }
  else { print STDERR "Warn: End of block not found: $_\n"; }
  $#rows = -1;  $#w = -1; $#attr = -1; $#mem = -1; $#suff = -1;
  $j = -1;
  @rows = split (/\n/);
    #print STDERR "DEB: //";  foreach $r (@rows) { print STDERR $r, '//'; }  print STDERR "\n";
  foreach $r (@rows) {  # Load each word and its attribute.
    $j++;
    ($w[$j], $attr[$j]) = split (/\s+/, $r);
  }
    #print STDERR "DEB: //";  foreach $w (@w) { print STDERR $w, '//'; }  print STDERR "\n";
  # Membership in block is stored after computation as follows:
  # $mem[$i] 
  #   = 0      means $ith elem (i.e. $w[$i]) is not in any block.
  #   = (N+1)  $ith elem is in block whose stem is in N. (0 <= N <= $#w)
  #   = -1     was in block but has been printed (used later).
  # $suff[$i]  Stores the suffix for the ith elem (i.e., $w[$i]th word in block)
  #
  for ($i=0; $i <= $#w;  $i++) { # Look for similarity among words.
    if (!($mem[$i])) { # i-th not already marked to be in a block.
      # Assume $i-th elem to be stem 
      for ($j=0; $j <= $#w;  $j++) {
        if ($i == $j) { next; }
        if (!($mem[$j])) { # j-th elem not already marked.
          $res = &sim_word ($w[$i], $w[$j]); # Compare words (1st as stem).
          # If the words are similar, set them to index of stem.
          if ($res) { 
            $mem[$j] = $i + 1;  $mem[$i] = $i + 1;  $suff[$i] = 'STEM';
            $suff[$j] = $res;
            #print STDERR "DEB: Found similar: $i $w[$i]\t$j $w[$j]\tSuff: $suff[$j]\n";
          }
        }
      }
    }
  }
  #for ($i=0; $i <= $#w; $i++) { print STDERR "DEBUG mem word attr suff: $mem[$i] $w[$i]\t$attr[$i]\t$suff[$i]\n"; }

  for ($i=0; $i <= $#w;  $i++) { # Print the new blocks.
    if ($mem[$i] == 0) {  # Not in any block. (i.e., A block by itself.)
      print "$w[$i] $attr[$i]_ALONE\n$end_blk2";
    }
    elsif ($mem[$i] eq -1) { }  # Already printed.
    else {  # print all the related words in this block.
      $found1 = 0;  # Flag to show that at least one stem/elem found
      for ($j=0;  $j <= $#w;  $j++) {   #CHECK whether: $j=$i is better
        if ($mem[$j] == ($i + 1)) { 
          print "$w[$j] $attr[$j]_$suff[$j]\n";
          #if ($mem[$j] == ($j + 1)) { print "_STEM\n"; } # This is stem
          #else { print "\n"; }
          $mem[$j] = -1;  # Mark as printed.
          $found1 = 1;  # Set flag that at least one stem/elem found.
        }
      }
      if ($found1) { print $end_blk2; }
    }
  }
}
