#!/usr/bin/perl
# Given a file of word forms, the likely roots underlying the word
# forms, and a file of possible paradigm; guess whether
# the paradigm is satisfied by the word forms.
# Example usage: 
#   pardgm_guess.cmd  -rbanXu.pre  -pCele.add  -wbanXu.wrd  > banXu.par

# Options:
# -r	-  File name containing likely roots. 
# -p	-  File name containing a paradigm (del add strings). 
# -w	-  File name containing word forms. 
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $Finname, $Foutname);
local ($rootfile, $pardgmfile, $wrdfile);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$rootfile = 'NONE'; # Default  File name containing likely roots. 
$pardgmfile = 'NONE'; # Default  File name containing a paradigm (del add strings). 
$wrdfile = 'NONE'; # Default  File name containing word forms. 

# Look up .pardgm_guessrc file. Override defaults.
if (open (RC, '.pardgm_guessrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anu/pardgm_guess.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'r') {
        $rootfile = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  'p') {
        $pardgmfile = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  'w') {
        $wrdfile = substr ($_, 2);
      }
      else { 
          print STDERR "pardgm_guess.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anu/pardgm_guess.cmd');
          die;
      }
    }
    else { # 
      print STDERR "pardgm_guess.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
     # Command line args override those of the rc file.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anu/pardgm_guess.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1, 1) eq  'r') {
      $rootfile = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'p') {
      $pardgmfile = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'w') {
      $wrdfile = substr ($ARGV[$i], 2);
    }
    else { 
        print STDERR "pardgm_guess.cmd: Unrecognized param: $ARGV[$i].\n";
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anu/pardgm_guess.cmd');
        die;
    }
  }
  else { # Argument found. Parameters end.
    $argbegpos = $i;
    last;
  }
}

# Process the arguments in the command line then call pardgm_guess.
# 1. Can be used as a filter. OR
# 2. file args can be given instead of filters.

if ($argbegpos < 0) { # No args.
  &pardgm_guess(STDIN, STDOUT);
}
elsif (($#ARGV - $argbegpos) == 0) { # Only one arg given. Execute the command with arg and stdout.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "pardgm_guess.cmd: Unable to open file $ARGV[$argbegpos]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = '';
  &pardgm_guess(INFILE, STDOUT);
  close (INFILE);
}
elsif (($#ARGV - $argbegpos) == 1) { # Two args given. Execute the command with arg1 and arg2.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "pardgm_guess.cmd: Unable to open input file $ARGV[$argbegpos]\n" ;
  open (OUTFILE, ">$ARGV[$argbegpos + 1]") ||
     die "pardgm_guess.cmd: Unable to open output file $ARGV[$argbegpos + 1]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = $ARGV[$argbegpos+1];  
  &pardgm_guess(INFILE, OUTFILE);
  close (OUTFILE);
  close (INFILE);
}
else { print STDERR "pardgm_guess.cmd: Too many arguments.\n"; }
#------------------------------------------------------

sub pardgm_guess {
local ($W_fin, $W_fout) = @_;
select($W_fout);
#%COMMENT
# Given a file of word forms, the likely roots underlying the word
# forms, and a file of possible paradigm; guess whether
# the paradigm is satisfied by the word forms.
# Example usage: 
#   pardgm_guess.cmd  -rbanXu.pre  -pCele.add  -wbanXu.wrd  > banXu.par

#%CMD TYPE 11


open (ROOTFILE, "$rootfile") ||
  die "pardgm_guess.pl: Unable to open file $rootfile: $!\n";
$i = -1;
while (<ROOTFILE>) {  # Read all the roots with their counts.
  chop;  $i++;
  ($cnt, $root) = split (/\s+/);
  #print STDERR  "DEBUG: cnt=$cnt, root=$root,\n";
  $root[$i] = $root;  $count[$i] = $cnt;
}

$maxscore = $maxscore_of_root = -1;
for ($r=0;  $r <= $#root;  $r++) { # Now try the roots one by one.
  system ("pardgm_deladd.cmd  -r$root[$r]  < $wrdfile > $wrdfile.p.tmp");
  open (W_PARDGMFILE, "$wrdfile.p.tmp") ||
      die "pardgm_guess.pl: Unable to open file $wrdfile.p.tmp: $!\n";
  while (<W_PARDGMFILE>) {
    chop;
    #print STDERR "DEBUG: grep '^$_\$' $pardgmfile >> $wrdfile.tmp", "\n";
    $res = `grep '^$_\$' $pardgmfile`;
    #print STDERR "DEBUG: $res";
    if ($res ne '') { $score[$r]++; }  # Found. Add to score.
  }
  if ($score[$r] > $maxscore) { 
    $maxscore = $score[$r];     $maxscore_of_root = $r;
  }
}
print STDERR "DEBUG: root no.=$maxscore_of_root,  score=$maxscore,\n";
print STDERR "Best root: $root[$maxscore_of_root]\n";

}
1;
