#!/usr/bin/perl
# Takes an input file consisting of a histogram of prefixes
# computed for a set of words sorted on frequencies.
#   Example usage:  pfx_sel.cmd  -p3  -n4  < TEST.histo
# Selects those prefixes which are 3 or more character longer,
# and which have 4 or more frequency of occurrence.
# Options:
# -p	-  Min length of a prefix (default 3).  
# -n	-  Min frequency for a prefix (default 4).  
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $Finname, $Foutname);
local ($min_len, $min_cnt);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$min_len = 3; # Default  Min length of a prefix (default 3).  
$min_cnt = 4; # Default  Min frequency for a prefix (default 4).  

# Look up .pfx_selrc file. Override defaults.
if (open (RC, '.pfx_selrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anumorph/pfx_sel.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'p') {
        $min_len = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  'n') {
        $min_cnt = substr ($_, 2);
      }
      else { 
          print STDERR "pfx_sel.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anumorph/pfx_sel.cmd');
          die;
      }
    }
    else { # 
      print STDERR "pfx_sel.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
     # Command line args override those of the rc file.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anumorph/pfx_sel.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1, 1) eq  'p') {
      $min_len = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'n') {
      $min_cnt = substr ($ARGV[$i], 2);
    }
    else { 
        print STDERR "pfx_sel.cmd: Unrecognized param: $ARGV[$i].\n";
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anumorph/pfx_sel.cmd');
        die;
    }
  }
  else { # Argument found. Parameters end.
    $argbegpos = $i;
    last;
  }
}

# Process the arguments in the command line then call pfx_sel.
# 1. Can be used as a filter. OR
# 2. file args can be given instead of filters.

if ($argbegpos < 0) { # No args.
  &pfx_sel(STDIN, STDOUT);
}
elsif (($#ARGV - $argbegpos) == 0) { # Only one arg given. Execute the command with arg and stdout.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "pfx_sel.cmd: Unable to open file $ARGV[$argbegpos]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = '';
  &pfx_sel(INFILE, STDOUT);
  close (INFILE);
}
elsif (($#ARGV - $argbegpos) == 1) { # Two args given. Execute the command with arg1 and arg2.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "pfx_sel.cmd: Unable to open input file $ARGV[$argbegpos]\n" ;
  open (OUTFILE, ">$ARGV[$argbegpos + 1]") ||
     die "pfx_sel.cmd: Unable to open output file $ARGV[$argbegpos + 1]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = $ARGV[$argbegpos+1];  
  &pfx_sel(INFILE, OUTFILE);
  close (OUTFILE);
  close (INFILE);
}
else { print STDERR "pfx_sel.cmd: Too many arguments.\n"; }
#------------------------------------------------------

sub pfx_sel {
local ($W_fin, $W_fout) = @_;
select($W_fout);
#%COMMENT
# Takes an input file consisting of a histogram of prefixes
# computed for a set of words sorted on frequencies.
#   Example usage:  pfx_sel.cmd  -p3  -n4  < TEST.histo
# Selects those prefixes which are 3 or more character longer,
# and which have 4 or more frequency of occurrence.
#%CMD TYPE 11



while (<$W_fin>) {
  chop;
  ($cnt, $prefix) = split (/\s+/);
  #print STDERR  "DEBUG: cnt=$cnt, prefix=$prefix,\n";
  $lenprefix = length ($prefix);
  if (($lenprefix >= $min_len) && ($cnt >= $min_cnt)) { print $_, "\n";}
}

}
1;
