#!/usr/bin/perl
# Takes an input paradigm file (*.p) and puts field names and values in
# front of each line. The field names are constructed using features 
# and values specified for the entries in the paradigm.
#   pdm_flds.cmd   < noun_m1.p  > noun_m1.pf
# (Categories, and feature values for the morph paradigms, 
# have been loaded using ldcatfv.pl.)
# -s $suff = '.pf';  # Suffix to be used for output file name.
# Options:
# -nog	-  If 0, use local cat and feature names. (Default 1)
# -p	-  Name of dir where paradigm cat and features are declared.
# -a	-  If set, compute del and add strings to root to obtain the wordform.

require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");
local (%pdim_arr, %pfv_arr, %catmap_arr, %featmap_arr);
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/morph/pdm_flds.pl");


# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $arg, $suff, $argsinfile, $Finname, $Finname);
local ($globfeat, $pardgmdir, $deladdpart);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$argsinfile = 0;  # Default args file names are not in a file.
$suff = '.pf'; # Default suffix for output file if necessary.
$globfeat = 1; #  If 0, use local cat and feature names. (Default 1) Default as shown.
$pardgmdir = '/home/bharathi/PNJ1/usr3/sangal/anusAraka/punjabi/morph/test'; # Default  Name of dir where paradigm cat and features are declared.
$deladdpart = 0; #  If set, compute del and add strings to root to obtain the wordform. Default as shown.

# Look up .pdm_fldsrc file. Override defaults.
if (open (RC, '.pdm_fldsrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/pdm_flds.cmd');
        die;
      }
      elsif (substr($_,1, 3) eq  'nog') {
        $globfeat = 0;
      }
      elsif (substr($_,1, 1) eq  'p') {
        $pardgmdir = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  'a') {
        $deladdpart = 1;
      }
      else { 
          print STDERR "pdm_flds.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/pdm_flds.cmd');
          die;
      }
    }
    else { # 
      print STDERR "pdm_flds.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/pdm_flds.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1,1) eq 's') { # Suffix for output file names.
       $suff = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1, 3) eq  'nog') {
      $globfeat = 0;
    }
    elsif (substr($ARGV[$i],1, 1) eq  'p') {
      $pardgmdir = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'a') {
      $deladdpart = 1;
    }
    else { print STDERR "pdm_flds.cmd: Parameter not recognized $ARGV[$i].\n";
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/pdm_flds.cmd');
      die;
    }
  }
  else { # Argument found. Parameters end.
    if (substr($ARGV[$i],0,1) eq '@') {  # A file of args found
      $argsinfile = 1;
      $ARGV[$i] = substr($ARGV[$i], 1);
    }
    $argbegpos = $i;
    last;
  }
}
#print STDERR "pdm_flds.cmd: DEB params end: $parmlastpos;  args begin: $argbegpos; total: $#ARGV;\n";

# Process the arguments in the command line then call pdm_flds.
# 1. Can be used as a filter. OR
# 2. If one or more file arguments are given, they are treated as
#    inputs files, and the output file name is given
#    by the input file name suffixed by the default suffix
#    (which can be changed by approp flag).

require("/home/bharathi/PNJ1/usr3/sangal/tools/src/morph/ldcatfv.pl");

if ($argbegpos < 0) { # No args.
  &pdm_flds(STDIN, STDOUT);
}
elsif ($argsinfile) {  # Argument filenames in file.
  open (ARGFILE, "$ARGV[$argbegpos]") ||
    die "pdm_flds.cmd: Unable to open args file $ARGV[$argbegpos]\n" ;
  while (<ARGFILE>) { # Read args.
    chop;
    if (!(open (INFILE, $_))) {
      print STDERR "pdm_flds.cmd: Unable to open file $_\n" ;
      next;
    }
    if (!(open (OUTFILE, ">$_$suff"))) {
      print STDERR "pdm_flds.cmd: Unable to open out file $_$suff\n";
      close (INFILE);
      next;
    }
    $Finname = $_;  $Foutname = $_ . $suff;
    &pdm_flds (INFILE, OUTFILE);
    close (INFILE);
    close (OUTFILE);
  }
  close (ARGFILE);
}
else { # Call the program with each arg. 
  for ($arg = $argbegpos; $arg <= $#ARGV; $arg++) {
    if (!(open (INFILE, "$ARGV[$arg]"))) {
      print STDERR "pdm_flds.cmd: Unable to open file $ARGV[$arg]\n" ;
      next;
    }
    if (!(open (OUTFILE, ">$ARGV[$arg]$suff"))) {
      print STDERR "pdm_flds.cmd: Unable to open out file $ARGV[$arg]$suff\n";
      close (INFILE);
      next;
    }
    $Finname = $ARGV[$arg];
    $Foutname = $ARGV[$arg] . $suff;  
    &pdm_flds (INFILE, OUTFILE);
    close (INFILE);
    close (OUTFILE);
  }
}

