#!/usr/bin/perl
# Takes an input file consisting of different word forms of 
# a root, one on each line. On each line, produces the delete 
# and add strings (to the root) to obtain each word form. 
#   pdgm_deladd.pl -rROOT  < TEST.wrd   >  TEST.add
# 
# Variables set from calling *.cmd routine:
#   $root - Guessed/given root 
# Options:
# -r	-  Guessed or given root for the word forms. 

require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");
require ("/home/bharathi/PNJ1/usr3/sangal/tools/src/anumorph/pdgm_deladd_sub.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $arg, $outf, $argsinfile, $Finname, $Foutname);
local ($root);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$argsinfile = 0;  # Default args file names are not in a file.
$root = 'NONE'; # Default  Guessed or given root for the word forms. 

# Look up .pdgm_deladdrc file. Override defaults.
if (open (RC, '.pdgm_deladdrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anumorph/pdgm_deladd.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'r') {
        $root = substr ($_, 2);
      }
      else { 
          print STDERR "pdgm_deladd.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anumorph/pdgm_deladd.cmd');
          die;
      }
    }
    else { # 
      print STDERR "pdgm_deladd.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anumorph/pdgm_deladd.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1,1) eq 'o') { # Specifies output file.
      $outfilename = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'r') {
      $root = substr ($ARGV[$i], 2);
    }
    else { print STDERR "pdgm_deladd.cmd: Unrecognized param: $ARGV[$i].\n";
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anumorph/pdgm_deladd.cmd');
      die;
    }
  }
  else { # Argument found. Parameters end.
    if (substr($ARGV[$i],0,1) eq '@') {  # A file of args found
      $argsinfile = 1;
      $ARGV[$i] = substr($ARGV[$i], 1);
    }
    $argbegpos = $i;
    last;
  }
}

#print STDERR "pdgm_deladd.cmd: DEB params end: $parmlastpos;  args begin: $argbegpos; total: $#ARGV;\n";

# Process the arguments in the command line then call pdgm_deladd.
# 1. Can be used as a filter. OR
# 2. If arg file names are given, they are treated as input files
#    and the output goes to a single file stdout.

if ($outfilename ne '') {
  open (OUT, ">$outfilename") ||
    die "pdgm_deladd.cmd: Unable to open output file $outfilename.\n";
  $outf = OUT;
  $Foutname = $outfilename;
}
else { $outf = STDOUT; }

if ($argbegpos < 0) { # No args.
  &pdgm_deladd(STDIN, $outf);
}
elsif ($argsinfile) {  # Argument filenames in file.
  open (ARGFILE, "$ARGV[$argbegpos]") ||
    die "pdgm_deladd.cmd: Unable to open args file $ARGV[$argbegpos]\n" ;
  while (<ARGFILE>) { # Read args.
    chop;
    if (!(open (INFILE, $_))) {
      print STDERR "pdgm_deladd.cmd: Unable to open file $_\n" ;
      next;
    }
    $Finname = $_;
    &pdgm_deladd (INFILE, $outf);
    close (INFILE);
  }
  close (ARGFILE);
}
else { # Call the program with each arg. 
  for ($arg = $argbegpos; $arg <= $#ARGV; $arg++) {
    if (!(open (INFILE, "$ARGV[$arg]"))) {
      print STDERR "pdgm_deladd.cmd: Unable to open file $ARGV[$arg]. Continuing.\n" ;
      next;
    }
    $Finname = $ARGV[$arg];
    &pdgm_deladd (INFILE, $outf);
    close (INFILE);
  }
}
close ($outf);
 
#------------------------------------------------------

sub pdgm_deladd {
local ($W_fin, $W_fout) = @_;
select($W_fout);
#%COMMENT
# Takes an input file consisting of different word forms of 
# a root, one on each line. On each line, produces the delete 
# and add strings (to the root) to obtain each word form. 
#   pdgm_deladd.pl -rROOT  < TEST.wrd   >  TEST.add
# 
# Variables set from calling *.cmd routine:
#   $root - Guessed/given root 
#%CMD TYPE M1


while (<$W_fin>) {
  chop;
  if (/\s+$/) { $_ = $`; }
  if (/^\s+/) { $_ = $'; }
    # print STDERR "DEBUG: root=$root";
  ($del, $add) = &find_deladd ($root, $_);
    # print STDERR "  del=$del, add=$add,\n";  #DEBUG
  $line = '-' . $del . ',+' . $add;
  print $line, "\n";
}
1;
}
1;
