#!/usr/bin/perl
# Takes an input paradigm file (*.p) and puts field names in front of
# each line. The field names are constructed using features and values
# specified for the entries in the paradigm.
#   pdgm_flds.pl < noun_m1.p  > noun_m1.pf
# (Assumes that categories, and feature values for the morph paradigms, 
# have been loaded using ldcatfv.pl.)
# Variables set from calling *.cmd routine:
#  $globfeat - 1 if global cat and feature names are to be used in output.
#  $pardgmdir - Name of directory which contains Ca, Ce, Fe, map_file
#  $deladdpart - 1 if deletions and additions will be done.

   #local (%pdim_arr, %pfv_arr, %catmap_arr, %featmap_arr);
   #require ('/home/bharathi/PNJ1/usr3/sangal/tools/src/anumorph/ldcatfv.pl');
sub pdgm_flds {
  local ($fin, $fout) = @_;
  local (@features, $valuestr, @cur_values, @max_values, @values);
  local ($i, $l,  $m, $pdgm_name, $mapped_name, $root, $pfeat,
         $del, $add, $wordtmp, $totvalues, @altnat, $line);
  $pdgm_name = <$fin>;   chop($pdgm_name);
  if ($globfeat) { 
    $mapped_name = $catmap_arr{$pdgm_name};
    if ($mapped_name eq '') {
      print STDERR "pdgm_flds.pl: Mapped name not found for paradigm: $pdgm_name.\n";
      $mapped_name = $pdgm_name;
    }
    print $fout $mapped_name, "\n";
  }
  else { print $fout $pdgm_name, "\n"; }

  $pfeat = $pdim_arr{$pdgm_name};
  @features = split (/\s+/, $pfeat);
  if ($#features < 0) { 
    die "pdgm_flds.pl: Features not found in file $Finname for paradigm: $pdgm_name\n";
  }
  for ($i=0;  $i <= $#features;  $i++) { # Get the pfeature value counts
    $pfeat = $features[$i];
    $valuestr = $pfv_arr{$pfeat};
    @values = split (/\s+/, $valuestr);
    if ($#values < 0) { 
      print STDERR "pdgm_flds.pl: Values not found in file $Finname for feature: $pfeat\n";
    }
    $max_values[$i] = $#values;  # For ith feature in @features, count of values.
  }
  
  @cur_values = @max_values;  # Set counters to last value of each pfeature
  $tot_values = 1;
  for ($l=0;  $l <= $#cur_values;  $l++) {
    $tot_values *= $cur_values[$l] + 1;
  }
  $tot_values--;  # Total number of entries for the current paradigm.

  # Now print features-values for each input wordform (on a line)
  while (<$fin>) {
    $root = $_;   chop $root;
    if ($root =~  /\s+$/) { $root = $`; }
    if ($root =~ /^\s+/) { $root = $'; }
    print $fout $root, "\n";
    for ($l=0;  $l <= $tot_values;  $l++) { # Read the entire paradigm for this root
      $_ = <$fin>;  chop;
      if ($deladdpart)  {
        if (!(/\//)) {  $altnat[0] = $_;  $#altnat = 0; } # Single word form
        else { @altnat = split (/\//); }
        ($del, $add) = &find_deladd ($root, $altnat[0]);
           #$wordtmp = $altnat[0]; chop $wordtmp;  # For debugging.
           #print STDERR "DEBUG: root:$root, wordform:$wordtmp, del:$del, add:$add";
        $line = '}    -' . $del . ',+' . $add ;
        for ($m=1; $m <= $#altnat;  $m++)  {
          ($del, $add) = &find_deladd ($root, $altnat[$m]);
             #$wordtmp = $altnat[$m]; chop $wordtmp;  # For debugging.
             #print STDERR "DEBUG: root:$root, wordform:$wordtmp, del:$del, add:$add";
          $line .=  ' / -' . $del . ',+' . $add ;
        }
        $line .= "\n";
      }
      else { $line = '}     ' . $_ . "\n"; }

      for ($i = $#features;  $i >= 0;  $i--) {
        $pfeat = $features[$i];
        $valuestr = $pfv_arr{$pfeat};
        @values = split (/\s+/, $valuestr);
        $j = $cur_values[$i];
        if ($globfeat) { $feature_name = $featmap_arr{$pfeat}; }
        else { $feature_name = $pfeat; }
        $line = $feature_name . '=' . $values[$j] . ',' . $line;
      }
      $line = $root . '{' . $line;
       # print "DEBUG: cur_values: ", @cur_values, "\n";
      print $fout $line;
      $res = &nextdecr;
      if (($res == 0) && ($l >= $tot_values)) {  # Counter reset and and this is not the last iteration
        #print STDERR "DEB: l = $l, root=$root\n";
        print STDERR "pdgm_flds.pl: End of a paradigm found late for root $root, in file $Finname.\n";
      }
    }  # End of for loop. Paradigm for a root over.
  }  # End of while loop.
}

# Procedure nextdecr:
# Decrements the counters for values in @cur_values from end-backwards
# If the counter cannot be decremented, return 0. Value of counter left
# fully set.
# $#features or $#cur_values - Number of features.
# $cur_values[$i] - stores the value of current counter for feature $i
sub nextdecr {
  local ($k, $zeroes_found);
  $zeroes_found = 0;
  for ($k = $#cur_values;  $k >= 0;  $k-- ) {
    if ($cur_values[$k] > 0) { $cur_values[$k]--; last; }
    else { $cur_values[$k] = $max_values[$k]; $zeroes_found++; }
  }
  if ($zeroes_found == $#cur_values) { return (0); } # Counter end reached (all 0s).
  else { return (1); }
}

1;
