#!/usr/bin/perl
# Checks and outputs the fields in a proper format from an input
#  text database (1 record per line).
# 1. Number of fields in each line, and the output format should be
# given by a file 'cheq.fmt' in the current directory.
# 2. Example cheq.fmt file is given below:
#    (Must use $fldcnt, and $flds[0] etc. separated by ','s.
#     A line can be commented by using standard perl comments.)
#
# $fldcnt = 2;
# format FLDFORMAT =
# @<<<<<<<<<< | @<<<<<<<<<<<<<<<<<<<<<<<<<<<<< | @######.##
# $flds[0],     $flds[1],                        $flds[2]
# .
#
# Options:
# -f	-  Name of format file. 

require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $arg, $outf, $argsinfile, $Finname, $Foutname);
local ($fmtfile);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$argsinfile = 0;  # Default args file names are not in a file.
$fmtfile = 'cheq.fmt'; # Default  Name of format file. 

# Look up .fld_chkrc file. Override defaults.
if (open (RC, '.fld_chkrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/appl/fld_chk.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'f') {
        $fmtfile = substr ($_, 2);
      }
      else { 
          print STDERR "fld_chk.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/appl/fld_chk.cmd');
          die;
      }
    }
    else { # 
      print STDERR "fld_chk.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/appl/fld_chk.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1,1) eq 'o') { # Specifies output file.
      $outfilename = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'f') {
      $fmtfile = substr ($ARGV[$i], 2);
    }
    else { print STDERR "fld_chk.cmd: Unrecognized param: $ARGV[$i].\n";
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/appl/fld_chk.cmd');
      die;
    }
  }
  else { # Argument found. Parameters end.
    if (substr($ARGV[$i],0,1) eq '@') {  # A file of args found
      $argsinfile = 1;
      $ARGV[$i] = substr($ARGV[$i], 1);
    }
    $argbegpos = $i;
    last;
  }
}

#print STDERR "fld_chk.cmd: DEB params end: $parmlastpos;  args begin: $argbegpos; total: $#ARGV;\n";

# Process the arguments in the command line then call fld_chk.
# 1. Can be used as a filter. OR
# 2. If arg file names are given, they are treated as input files
#    and the output goes to a single file stdout.

if ($outfilename ne '') {
  open (OUT, ">$outfilename") ||
    die "fld_chk.cmd: Unable to open output file $outfilename.\n";
  $outf = OUT;
  $Foutname = $outfilename;
}
else { $outf = STDOUT; }

if ($argbegpos < 0) { # No args.
  &fld_chk(STDIN, $outf);
}
elsif ($argsinfile) {  # Argument filenames in file.
  open (ARGFILE, "$ARGV[$argbegpos]") ||
    die "fld_chk.cmd: Unable to open args file $ARGV[$argbegpos]\n" ;
  while (<ARGFILE>) { # Read args.
    chop;
    if (!(open (INFILE, $_))) {
      print STDERR "fld_chk.cmd: Unable to open file $_\n" ;
      next;
    }
    $Finname = $_;
    &fld_chk (INFILE, $outf);
    close (INFILE);
  }
  close (ARGFILE);
}
else { # Call the program with each arg. 
  for ($arg = $argbegpos; $arg <= $#ARGV; $arg++) {
    if (!(open (INFILE, "$ARGV[$arg]"))) {
      print STDERR "fld_chk.cmd: Unable to open file $ARGV[$arg]. Continuing.\n" ;
      next;
    }
    $Finname = $ARGV[$arg];
    &fld_chk (INFILE, $outf);
    close (INFILE);
  }
}
close ($outf);
 
#------------------------------------------------------

sub fld_chk {
local ($W_fin, $W_fout) = @_;
select($W_fout);
#%COMMENT
# Checks and outputs the fields in a proper format from an input
#  text database (1 record per line).
# 1. Number of fields in each line, and the output format should be
# given by a file 'cheq.fmt' in the current directory.
# 2. Example cheq.fmt file is given below:
#    (Must use $fldcnt, and $flds[0] etc. separated by ','s.
#     A line can be commented by using standard perl comments.)
#
# $fldcnt = 2;
# format FLDFORMAT =
# @<<<<<<<<<< | @<<<<<<<<<<<<<<<<<<<<<<<<<<<<< | @######.##
# $flds[0],     $flds[1],                        $flds[2]
# .
#
#%CMD TYPE M1

require ("$fmtfile");
$~ = "FLDFORMAT";   # Format to be used with 'write'
while (<$W_fin>) {
  if (/^\s*$/)  { print; next; }  # Print but do not process the line.
  if (/^\s*#/) { print; next; }  # Print but do not process the line.
  chop;
  @flds = split (/\s*\|\s*/);
  # print STDERR "DEBUG: Fld count: $#flds, @flds\n";
  if (!($#flds == $fldcnt)) {
    print STDERR "Line $.: Invalid number of fields: $_\n";
  }
  write ;
}
}
1;
