#!/usr/bin/perl
# Takes the ledger as its input file. Produces 
# the ledger with balance filled in.
# Options:
# -d	-  Debit field number. Default 3 (counting from 0) 
# -c	-  Credit field number. Default 4 (counting from 0) 

require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $arg, $outf, $argsinfile, $Finname, $Foutname);
local ($debfld, $credfld);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$argsinfile = 0;  # Default args file names are not in a file.
$debfld = 3; # Default  Debit field number. Default 3 (counting from 0) 
$credfld = 4; # Default  Credit field number. Default 4 (counting from 0) 

# Look up .ledgbalrc file. Override defaults.
if (open (RC, '.ledgbalrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/appl/ledgbal.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'd') {
        $debfld = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  'c') {
        $credfld = substr ($_, 2);
      }
      else { 
          print STDERR "ledgbal.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/appl/ledgbal.cmd');
          die;
      }
    }
    else { # 
      print STDERR "ledgbal.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/appl/ledgbal.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1,1) eq 'o') { # Specifies output file.
      $outfilename = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'd') {
      $debfld = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'c') {
      $credfld = substr ($ARGV[$i], 2);
    }
    else { print STDERR "ledgbal.cmd: Unrecognized param: $ARGV[$i].\n";
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/appl/ledgbal.cmd');
      die;
    }
  }
  else { # Argument found. Parameters end.
    if (substr($ARGV[$i],0,1) eq '@') {  # A file of args found
      $argsinfile = 1;
      $ARGV[$i] = substr($ARGV[$i], 1);
    }
    $argbegpos = $i;
    last;
  }
}

#print STDERR "ledgbal.cmd: DEB params end: $parmlastpos;  args begin: $argbegpos; total: $#ARGV;\n";

# Process the arguments in the command line then call ledgbal.
# 1. Can be used as a filter. OR
# 2. If arg file names are given, they are treated as input files
#    and the output goes to a single file stdout.

if ($outfilename ne '') {
  open (OUT, ">$outfilename") ||
    die "ledgbal.cmd: Unable to open output file $outfilename.\n";
  $outf = OUT;
  $Foutname = $outfilename;
}
else { $outf = STDOUT; }

if ($argbegpos < 0) { # No args.
  &ledgbal(STDIN, $outf);
}
elsif ($argsinfile) {  # Argument filenames in file.
  open (ARGFILE, "$ARGV[$argbegpos]") ||
    die "ledgbal.cmd: Unable to open args file $ARGV[$argbegpos]\n" ;
  while (<ARGFILE>) { # Read args.
    chop;
    if (!(open (INFILE, $_))) {
      print STDERR "ledgbal.cmd: Unable to open file $_\n" ;
      next;
    }
    $Finname = $_;
    &ledgbal (INFILE, $outf);
    close (INFILE);
  }
  close (ARGFILE);
}
else { # Call the program with each arg. 
  for ($arg = $argbegpos; $arg <= $#ARGV; $arg++) {
    if (!(open (INFILE, "$ARGV[$arg]"))) {
      print STDERR "ledgbal.cmd: Unable to open file $ARGV[$arg]. Continuing.\n" ;
      next;
    }
    $Finname = $ARGV[$arg];
    &ledgbal (INFILE, $outf);
    close (INFILE);
  }
}
close ($outf);
 
#------------------------------------------------------

sub ledgbal {
local ($W_fin, $W_fout) = @_;
select($W_fout);
#%COMMENT
# Takes the ledger as its input file. Produces 
# the ledger with balance filled in.
#%CMD TYPE M1

$bal = 0;
while (<$W_fin>) {
  if (/^\s*$/)  { print; next; }  # Print but do not process the line.
  if (/^\s*#/) { print; next; }  # Print but do not process the line.
  chop;
  @flds = split (/\s*\|\s*/);
  $bal += $flds[$credfld] - $flds[$debfld];
  print $_;
  print STDOUT sprintf (" %9.2f\n", $bal );  # Removed ' |'

}
}
1;
