#!/usr/bin/perl
# Calculates return on a recurring deposit (yearly) of 10000, 
# compounded per year for N years.
# Options:
# -y	-  Number of years.  
# -r	-  Percent interest rate. 
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $Finname, $Foutname);
local ($n, $rate);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$n = 16; # Default  Number of years.  
$rate = 12.0; # Default  Percent interest rate. 

# Look up .licrc file. Override defaults.
if (open (RC, '.licrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/appl/lic.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'y') {
        $n = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  'r') {
        $rate = substr ($_, 2);
      }
      else { 
          print STDERR "lic.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/appl/lic.cmd');
          die;
      }
    }
    else { # 
      print STDERR "lic.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
     # Command line args override those of the rc file.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/appl/lic.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1, 1) eq  'y') {
      $n = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'r') {
      $rate = substr ($ARGV[$i], 2);
    }
    else { 
        print STDERR "lic.cmd: Unrecognized param: $ARGV[$i].\n";
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/appl/lic.cmd');
        die;
    }
  }
  else { # Argument found. Parameters end.
    $argbegpos = $i;
    last;
  }
}

# Process the arguments in the command line then call lic.
# 1. Can be used as a filter. OR
# 2. file args can be given instead of filters.

if ($argbegpos < 0) { # No args.
  &lic(STDIN, STDOUT);
}
elsif (($#ARGV - $argbegpos) == 0) { # Only one arg given. Execute the command with arg and stdout.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "lic.cmd: Unable to open file $ARGV[$argbegpos]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = '';
  &lic(INFILE, STDOUT);
  close (INFILE);
}
elsif (($#ARGV - $argbegpos) == 1) { # Two args given. Execute the command with arg1 and arg2.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "lic.cmd: Unable to open input file $ARGV[$argbegpos]\n" ;
  open (OUTFILE, ">$ARGV[$argbegpos + 1]") ||
     die "lic.cmd: Unable to open output file $ARGV[$argbegpos + 1]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = $ARGV[$argbegpos+1];  
  &lic(INFILE, OUTFILE);
  close (OUTFILE);
  close (INFILE);
}
else { print STDERR "lic.cmd: Too many arguments.\n"; }
#------------------------------------------------------

sub lic {
local ($W_fin, $W_fout) = @_;
select($W_fout);
#%COMMENT
# Calculates return on a recurring deposit (yearly) of 10000, 
# compounded per year for N years.
#%CMD TYPE 11

$princ = 10000 * .7;  # Tax savings of 30%
$comp =  10000 * .5;  # First year has higher tax saving.
for ($i=1;  $i<=$n;  $i++) {
  $int = $comp * $rate/100.0;
  $comp += $int + $princ;
}
print STDOUT sprintf ("Give amt=%7.0f,    rate=%3d  period=%3d,\n", 
       $comp, $rate, $n);
}
1;
