#!/usr/bin/perl
# Calculates breakup of interest and principal amount paid over the 
# years when equal monthly installments (EMI) are being paid.
# Principal is specified as the value of $princ variable.
# Writes short result in STDOUT. Detailed results in loan_full.tmp
#   Usage:  loan_int.pl > loan.out
# Options:
# -p	-  Principal amount (default 800000) 
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $Finname, $Foutname);
local ($princ);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$princ = 800000; # Default  Principal amount (default 800000) 

# Look up .loan_intrc file. Override defaults.
if (open (RC, '.loan_intrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/mine/house/loan_int.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'p') {
        $princ = substr ($_, 2);
      }
      else { 
          print STDERR "loan_int.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/mine/house/loan_int.cmd');
          die;
      }
    }
    else { # 
      print STDERR "loan_int.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
     # Command line args override those of the rc file.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/mine/house/loan_int.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1, 1) eq  'p') {
      $princ = substr ($ARGV[$i], 2);
    }
    else { 
        print STDERR "loan_int.cmd: Unrecognized param: $ARGV[$i].\n";
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/mine/house/loan_int.cmd');
        die;
    }
  }
  else { # Argument found. Parameters end.
    $argbegpos = $i;
    last;
  }
}

# Process the arguments in the command line then call loan_int.
# 1. Can be used as a filter. OR
# 2. file args can be given instead of filters.

if ($argbegpos < 0) { # No args.
  &loan_int(STDIN, STDOUT);
}
elsif (($#ARGV - $argbegpos) == 0) { # Only one arg given. Execute the command with arg and stdout.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "loan_int.cmd: Unable to open file $ARGV[$argbegpos]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = '';
  &loan_int(INFILE, STDOUT);
  close (INFILE);
}
elsif (($#ARGV - $argbegpos) == 1) { # Two args given. Execute the command with arg1 and arg2.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "loan_int.cmd: Unable to open input file $ARGV[$argbegpos]\n" ;
  open (OUTFILE, ">$ARGV[$argbegpos + 1]") ||
     die "loan_int.cmd: Unable to open output file $ARGV[$argbegpos + 1]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = $ARGV[$argbegpos+1];  
  &loan_int(INFILE, OUTFILE);
  close (OUTFILE);
  close (INFILE);
}
else { print STDERR "loan_int.cmd: Too many arguments.\n"; }
#------------------------------------------------------

sub loan_int {
local ($W_fin, $W_fout) = @_;
select($W_fout);
#%COMMENT
# Calculates breakup of interest and principal amount paid over the 
# years when equal monthly installments (EMI) are being paid.
# Principal is specified as the value of $princ variable.
# Writes short result in STDOUT. Detailed results in loan_full.tmp
#   Usage:  loan_int.pl > loan.out
#%CMD TYPE 11

$int = 14;
$emi = 134 * $princ / 10000;
 #$emi = 10720;  # for 8lakhs
$yrs = 15;
$months = $yrs * 12;

format FULLFORMAT =
@## @#######  @#######      @########
$i, $pay_int,   $pay_princ, $princ
.
format SHRTFORMAT =
@## @#######
$cur_yr, $cum_int
.
$~ = "SHRTFORMAT";

open (FULL, ">loan_full.tmp") || 
  die "Unable to open out file loan_full.tmp: $!\n";

print "Principal	I-rate-mo	Yrs	EMI\n";
print "$princ\t\t$int\t\t$yrs\t$emi\n";
print FULL "Principal	I-rate-mo	Yrs	EMI\n";
print FULL "$princ\t\t$int\t\t$yrs\t$emi\n";

print "\n";
print "Year\tInterest\n";
print FULL "\n";
print FULL "Month\tInterest\tPay-principal\tPrinc-rem\n";
for ($i=1; $i <= $months;  $i++) {
  $mo_int = $princ * $int / 1200;
  if ($emi <= $mo_int) {
    print STDERR "EMI is less than interest due: $emi, $mo_int\n";
    last;
  }
  $pay_int = $mo_int;
  $pay_princ = $emi - $mo_int;
   #print FULL $i, "\t", $pay_int, "\t", $pay_princ, "\n";
  $princ = $princ - $pay_princ;
  if (!($i % 12)) {  # Year over.
    $cum_int += $pay_int;   # Add the final interest for 12th month
    $cur_yr = $i / 12;
    $~ = "SHRTFORMAT";    write;
    #ERR $~ = "FULLFORMAT";   write FULL;
    print FULL "$i   $pay_int    $pay_princ    $princ\n";  # NOERR
    print FULL "------------------------------------------- ", "$cur_yr   $cum_int\n";
    $cum_int = 0;
  }
  else { 
    $cum_int += $pay_int;
    #ERR $~ = "FULLFORMAT";   write FULL;
    print FULL "$i   $pay_int    $pay_princ    $princ\n";  # NOERR
  }
}
}
1;
