#! /usr/bin/perl
# Takes as its input the output of morph analyser and checks 
# whether a word satisfies the specifications.
#   Example Usage: xtrandmo.cmd -fpatfile -cmostrucfile  <abhi.moh  >abhi.xtr
# Options:
#    -fpatfile	Pattern specification is contained in file patfile.
#               (Default file: /home/bharathi/PNJ1/usr3/sangal/tools/src/corp/patmo)
#    -csfile 	Structure of morph output is contained in sfile.
#               (Default file: /home/bharathi/PNJ1/usr3/sangal/tools/src/corp/struc)
#    -dabc	Delimiter used for sentences is 'abc'. Default: <.>.
#    -dn	Delimiter used is for sentences is new line.
#    -s.abc	Output file will be suffixed by '.abc'. Default: '.xtr'.
# cmd type many to many.

require ("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $arg, $outf, $suff, $argsinfile);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$argsinfile = 0;  # Default args file names are not in a file.
$suff = '.xtr'; # Default suffix for output file if necessary.
$delim = '<.>';
$patfile = '/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/patmo';
$mostrucfile = '/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/struc';

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/xtrandmo.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1,1) eq 's') { # Suffix for output file names.
       $suff = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1,1) eq 'd') { # Delimiter specified.
      $delim = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1,1) eq 'n') { # Use new line as delimiter
      $delim = 'n';
    }
    elsif (substr($ARGV[$i],1,1) eq 'f') { # Pattern is in file.
      $patfile = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1,1) eq 'c') { # morph struc is in file.
      $mostrucfile = substr ($ARGV[$i],2);
    }
    else { print STDERR "xtrandmo.cmd: Unrecognized parm: $ARGV[$i]. Ignoring.\n";
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/xtrandmo.cmd');
    }
  }
  else { # Argument found. Parameters end.
    if (substr($ARGV[$i],0,1) eq '@') {  # A file list found
      $argsinfile = 1;
      $ARGV[$i] = substr($ARGV[$i], 1);
    }
    $argbegpos = $i;
    last;
  }
}
#print STDERR "DEB params end: $parmlastpos;  args begin: $argbegpos; total: $#ARGV;\n";

# Process the arguments in the command line then call xtrmorph.
# 1. Can be used as a filter. OR
# 2. If one or more file arguments are given, they are treated as
#    inputs files, and the output file name is given
#    by the input file name suffixed by the default suffix
#    (which can be changed by approp flag).

require ("/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/ldstr.pl");
&load_struc ($mostrucfile);
  #require ("/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/prstr.pl");
  #print STDERR "Category structure is:\n"
  #&print_struc;

require ("/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/ldpatmo.pl");
&load_patmo($patfile);
  #require ("/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/prpatmo.pl");
  #print STDERR "Pattern spec is:\n"
  #&print_patmo;

require ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/xtrandmo.pl');
if ($outfilename ne '') {
  open (OUT, ">$outfilename") ||
    die "Unable to open output file $outfilename.\n";
  $outf = OUT;
}
else { $outf = STDOUT; }

if ($argbegpos < 0) { # No args.
  &xtrmorph(STDIN, STDOUT);
}
elsif ($argsinfile) {  # Args in file.
  open (ARGFILE, "$ARGV[$argbegpos]") ||
    die "Unable to open args file $ARGV[$argbegpos]\n" ;
  while (<ARGFILE>) { # Read args.
    chop;
    if (!(open (INFILE, $_))) {
      print STDERR "Unable to open file $_\n" ;
      next;
    }
    if (!(open (OUTFILE, ">$_$suff"))) {
      print STDERR "Unable to open out file $_$suff\n";
      close (INFILE);
      next;
    }
    &xtrmorph (INFILE, OUTFILE);
    close (INFILE);
    close (OUTFILE);
  }
  close (ARGFILE);
}
else { # Call the program with each arg. 
  for ($arg = $argbegpos; $arg <= $#ARGV; $arg++) {
    if (!(open (INFILE, "$ARGV[$arg]"))) {
      print STDERR "Unable to open file $ARGV[$arg]\n" ;
      next;
    }
    &xtrmorph (INFILE, $outf);
    close (INFILE);
  }
}

