#!/usr/bin/perl
# Takes as its input: the output of morph analyser and checks 
# whether a word satisfies the pattern specifications.
#    Usage: xtrandmo.cmd -fpatfile -smostrucfile  <abhi.moh  >abhi.xtr
# Pattern specification is contained in file patfile, structure of
# morph output is in file mostrucfile. 
# (Default pattern file is /home/bharathi/PNJ1/usr3/sangal/tools/src/corp/patmo)
# (Default morph structure file is /home/bharathi/PNJ1/usr3/sangal/tools/src/corp/struc)

# Assumes that the structure of categories as well as 
# pattern specification to tbe searched have been loaded in mem.
# (Being loaded by xtrandmo.cmd file or other files that call it.)

sub xtrmorph {
  local ($infile, $outfile) = @_;
  local ($oldfile) = select($outfile);
  local ($entry, $root, $categ, $fvalues, @vals, $wordnum, $sentnum);
  local ($oldlimtr, $res, $sentmatch, $lastwordmatch, $eachw, @words);
  $oldlimtr = $/;
  $/ = "# s";
  while (<$infile>) { # Reads in a sentence.
    if (/^[0-9]+\n/) {  # sentence num found at the beginng.
      $sentnum = '# s'.$&;
      $_ = $';
    }
    $sentmatch = 0; $lastwordmatch = 0;
    $sent = $_;    # For debugging purpose.
    @words = split (/\n/);
    #print STDERR "SENT---------:$sentnum$sent\n";
    for ($eachw=0;  $eachw < $#words;  $eachw++) { # Do for each word.
      $_ = $words[$eachw];
      #print STDERR "LINE: $_\n";
      if ($lastwordmatch) {
        $res =  &match_word ($words[$eachw], 1) ;
        if ('AVY' eq  $res) { }  # Do not reset $lastwordmatch.
        elsif ('' ne $res) {
          print STDERR "Word matched (2nd): ", $res;
          $sentmatch = 1;
          last;
        }
        else { $lastwordmatch = 0; }
      }
      else {
        $res =  &match_word ($words[$eachw], 0) ;
        if ('AVY' eq $res) { }
        elsif ('' ne $res) {
          print STDERR "Word matched (1st): ", $res;
          $lastwordmatch = 1;
        }
        else { $lastwordmatch = 0; }
      }
    }
    if ($sentmatch) { 
      print $sentnum; 
      for ($eachw=0;  $eachw < $#words;  $eachw++) { # print sentence
        print $words[$eachw], "\n";
      }
    }
  }
  select ($oldfile);
  $/ = $oldlimtr;
}

# Given a word as its argument, see whether it matches the
# pattern. (The word might have several lexical entries.)
# If $wpatnum is 0, first word spec is tried
# else second word spec is tried.
sub match_word {
  ($_, $wpatnum) = @_;
  if (/^# s[0-9]+$/) { # beginning of sent found
    print STDERR "match_word: beginning of sentence found:", $_, ". Ignoring.\n";  return;
  }

  if (/^w[0-9]+\s+/) { # Strip off word number
    $wordnum = $&;
    $_ = $';
  }
  else { print STDERR "Word number not found but continuing: $_\n"; }
  while ($_ ne '') {
    if (/{/) { # Remove the root.
      $root = $`;    $_ = $';
    }
    else { print STDERR "'{' not found. $_\n";   last; }
    if (/}/) {
      $entry = $`;
      $_ = $';  # Remaining entries '|{...} ...' if any.
      #print STDERR "Remaining entries:$_;\n";
      if ($entry =~ /(\S+)\s*/) {    # TEMPORARILY + changed to *
        $categ = $1;
        if ($categ eq 'AVY') { return('AVY'); }  # Ignore entry.
        $fvalues = $';
        @vals = split(/\s+/, $fvalues);
        #print STDERR "CAT: $categ;   FVAL: @vals;\n";
        if (&check_fval ($root, $categ, $wpatnum, @vals)) {
           #print STDERR "Entry matched. $root{$fvalues}\n";
           return("$wordnum $root{$categ $fvalues}\n");
        }
        #else { print STDERR "Entry not matched.\n"; }
        if ($_ ne '') {
          if (substr($_,0,1) eq '|') {
            $_ = substr($_,1);
          }
          else { print STDERR "'|' not found. $_\n"; }
        }
      }
      else { print STDERR "Unrecognized format for horiz morph output: $entry\n";}
    }
    else { print STDERR "'}' not found. $_\n"; }
  }
  return ('');
}

# Called with values of features of a morph output entry.
# Returns true if satisfies the pattern.
# $wpatnum tells whether to try the first word spec or the second.
sub check_fval {
  local ($root, $categ, $wpatnum, @fvalues) = @_;
    #print STDERR "In check_fval: @_\n";
  local ($wc, $i, $cfpat, $vpat, $pos, $vread, $curwordmatch );
  #for ($wc=0;  $wc < $#wordspec;  $wc++) { # Do for each word spec.
  $wc = $wpatnum;
  $curwordmatch =1;
  for ($i=$wordspec[$wc];  $i < $wordspec[$wc + 1];  $i++) { # do for each pattern in spec.
    $vpat  = $patrn[$i];  # get the pattern value.
    if ($vpat ne 'any') { # if val of pat not 'any', only then test for feature values.
      $cfpat = $catfld[$i]; # categ name, field name in pat
      $cfpat =~ /,/;
      $cpat =  $`;  $fpat = $';
      if ($fpat eq 'root') {
        if ($root ne 'any') {
          if ($root ne $vpat) { $curwordmatch = 0; last; }
        }
        if ('*' ne $cpat) {
          if ($categ ne $cpat) { $curwordmatch = 0; last; }
        }
      }
      else {
        $pos = $fldpos{$cpat, $fpat};  # position of field.
        $vread = $fvalues[$pos];
          #print STDERR "Check_fval: val-read=$vread, val-pat=$vpat, cpat=$cpat, fpat=$fpat,  pos=$pos\n";
        if ($vread ne 'any') {
          if ($vread ne $vpat) { # if not equal, current word pat fails 
            $curwordmatch = 0;
            last;
          } 
        } 
      }
    }
  }
  if ($curwordmatch) { return(1); }
  #}
  return(0); # Means all word pats over no match.
}
1;

