#!/usr/bin/perl
# Takes as its input: the output of morph analyser and checks 
# whether a word satisfies the pattern specifications.
# Usage: xtrmo.pl  <abhi.moh  >abhi.xtr
# Pattern specification is contained in patmo, struct in struc.

# Assumes that the structure of categories as well as 
# pattern specification to tbe searched have been loaded in mem.
# (Being done by xtrmo.cmd file or other files that call it.)

sub xtrmorph {
  local ($infile, $outfile) = @_;
  local ($oldfile) = select($outfile);
  local ($entry, $root, $categ, $fvalues, @vals, $wordnum );
  local ($oldlimtr, $sentmatch, $eachw, @words);
  $oldlimtr = $/;
  $/ = "# s";
  while (<$infile>) { # Reads in a sentence.
    if (/^[0-9]+\n/) {  # sentence num found at the beginng.
      $sentnum = '# s'.$&;
      $_ = $';
    }
    $sentmatch = 0;
    $sent = $_;    # For debugging purpose.
    @words = split (/\n/);
    #print STDERR "SENT---------:$sentnum$sent\n";
    for ($eachw=0;  $eachw < $#words;  $eachw++) { # Do for each word.
      $_ = $words[$eachw];
      #print STDERR "LINE: $_\n";
      if ('' ne ($res = &match_word ($words[$eachw])) ) {
        print STDERR "Word matched: ", $res;
        $sentmatch = 1;
        last;
      }
    }
    if ($sentmatch) { 
      print $sentnum; 
      for ($eachw=0;  $eachw < $#words;  $eachw++) { # Do for each word.
        print $words[$eachw], "\n";
      }
    }
  }
  select ($oldfile);
  $/ = $oldlimtr;
}

# Given a word as its argument, see whether it matches the
# pattern. (The word might have several lexical entries.)
sub match_word {
  $_ = $_[0];
  if (/^# s[0-9]+$/) { # beginning of sent found
    print $_, "\n";  next;
  }

  if (/^w[0-9]+\s+/) { # Strip off word number
    $wordnum = $&;
    $_ = $';
  }
  else { print STDERR "Word number not found but trying in: $_\n"; }
  while ($_ ne '') {
    if (/{/) { # Remove the root.
      $root = $`;    $_ = $';
    }
    else { print STDERR "'{' not found. $_\n";   last; }
    if (/}/) {
      $entry = $`;
      $_ = $';  # Remaining entries '|{...} ...' if any.
      #print STDERR "Remaining entries:$_;\n";
      if ($entry =~ /(\S+)\s*/) {    # TEMPORARILY + changed to *
        $categ = $1;
        $fvalues = $';
        @vals = split(/\s+/, $fvalues);
        #print STDERR "CAT: $categ;   FVAL: @vals;\n";
        if (&check_fval ($root, $categ, @vals)) {
           #print STDERR "Entry matched. $root{$fvalues}\n";
           return("$wordnum $root{$categ $fvalues}\n");
        }
        #else { print STDERR "Entry not matched.\n"; }
        if ($_ ne '') {
          if (substr($_,0,1) eq '|') {
            $_ = substr($_,1);
          }
          else { print STDERR "'|' not found. $_\n"; }
        }
      }
      else { print STDERR "Unrecognized format for horiz morph output: $entry\n";}
    }
    else { print STDERR "'}' not found. $_\n"; }
  }
  return ('');
}

# Called with values of features of a morph output entry.
# Returns true if satisfies the pattern.
# Uses values of variables $categ and $root above.
sub check_fval {
  local ($root, $categ, @fvalues) = @_;
    #print STDERR "In check_fval: @_\n";
  local ($wc, $i, $cfpat, $vpat, $pos, $vread, $curwordmatch );
  for ($wc=0;  $wc < $#wordspec;  $wc++) { # Do for each word spec.
    $curwordmatch =1;
    for ($i=$wordspec[$wc];  $i < $wordspec[$wc + 1];  $i++) { # do for each pattern in spec.
      $vpat  = $patrn[$i];  # get the pattern value.
      $cfpat = $catfld[$i]; # categ name, field name in pat
      $cfpat =~ /,/;
      $cpat =  $`;  $fpat = $';
      if ($fpat eq 'root') {
        if ($root ne $vpat) { $curwordmatch = 0; last; }
        if ('*' ne $cpat) {
          if ($categ ne $cpat) { $curwordmatch = 0; last; }
        }
      }
      else {
        $pos = $fldpos{$cpat, $fpat};  # position of field.
        $vread = $fvalues[$pos];
          #print STDERR "Check_fval: val-read=$vread, val-pat=$vpat, cpat=$cpat, fpat=$fpat,  pos=$pos\n";
        if ($vread ne 'any') {
          if ($vread ne $vpat) { # if not equal, current word pat fails 
            $curwordmatch = 0;
            last;
          } 
        } 
      }
    }
    if ($curwordmatch) { return(1); }
  }
  return(0); # Means all word pats over no match.
}
1;

