#!/usr/bin/perl
# Takes a file containing word and irefs, and returns the
# names of files for the irefs in sorted order after removing
# duplicates. It needs to be supplied the
# mapping file for filename-irefs (-f option).
#   Example usage: iref2name.cmd  -f/usr3/corpus/teluguorig/wi.nam < aMtis.iref  >aMtis.nam
# The above takes a file called aMtis.iref and produces aMtis.nam,
# using the mapping file: /usr3/corpus/teluguorig/wi.nam.
#
# (For space efficiency, numbers instead of file names are used.
# These numbers are called 'irefs' for indexed refs.
# where file F contains files names (each in a separate line) say
# A, ... Z. It takes files named A to Z, and for each word that occurs
# in the files, stores the word followed by irefs for files
# in which it occurs. (iref for file A is 1, for B is 2, and so on.))

# Options:
# -f	-  Name of file to be used for mapping between filenames and irefs. (Default: Telugu orig)
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/iref2name.pl");


# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $Finname, $Foutname);
local ($wifile);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$wifile = '/usr3/corpus/teluguorig/wi.nam'; # Default  Name of file to be used for mapping between filenames and irefs. (Default: Telugu orig)

# Look up .iref2namerc file. Override defaults.
if (open (RC, '.iref2namerc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/iref2name.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'f') {
        $wifile = substr ($_, 2);
      }
      else { 
          print STDERR "Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/iref2name.cmd');
          die;
      }
    }
    else { # 
      print STDERR "Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
     # Command line args override those of the rc file.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/iref2name.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1, 1) eq  'f') {
      $wifile = substr ($ARGV[$i], 2);
    }
    else { 
        print STDERR "Unrecognized param: $ARGV[$i].\n";
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/iref2name.cmd');
        die;
    }
  }
  else { # Argument found. Parameters end.
    $argbegpos = $i;
    last;
  }
}

# Process the arguments in the command line then call iref2name.
# 1. Can be used as a filter. OR
# 2. file args can be given instead of filters.

if ($argbegpos < 0) { # No args.
  &iref2name(STDIN, STDOUT);
}
elsif (($#ARGV - $argbegpos) == 0) { # Only one arg given. Execute the command with arg and stdout.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "Unable to open file $ARGV[$argbegpos]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = '';
  &iref2name(INFILE, STDOUT);
  close (INFILE);
}
elsif (($#ARGV - $argbegpos) == 1) { # Two args given. Execute the command with arg1 and arg2.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "Unable to open input file $ARGV[$argbegpos]\n" ;
  open (OUTFILE, ">$ARGV[$argbegpos + 1]") ||
     die "Unable to open output file $ARGV[$argbegpos + 1]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = $ARGV[$argbegpos+1];  
  &iref2name(INFILE, OUTFILE);
  close (OUTFILE);
  close (INFILE);
}
else { print STDERR "Too many arguments.\n"; }
