#!/usr/bin/perl
# Loads the file /usr4/corpus/corptab giving details about the corpora.
#   Example usage: put in 'require("/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/ldcorptab.pl");'
#   and then call &ldcorptab.
# This is used for accessing the corpus location and format etc.
# The result of the function call is stored in the following arrays.
#  @langtab, @origtab, @dirtab, @namestab, @scrpttab, @eostab, @witab, @abbrdirtab
# The arrays can be searched for the desired language with
# normal/clean corpus value and the result may be kept in index $found.
#
# The corptab file looks as follows:
#> # The following is used by programs to get details about corpus. '#' begins a comment.
#> # language  orig-cleaned  directory  names-corp-files              eos   word-irefs          abbrdir
#> #  |          |             |                    |  script-notat    |        |   word-freq       |
#> #  |          |             |                    |          |       |        |       |           |
#> #  V          V             V                    V          V       V        V       V           V
#> telugu  | normal    | /usr4/corpus/telugu8    | wi.nam    | i8  | '.'  | word.ndx | word.frq |/usr4/corpus/tlg8
#> hindi   | normal    | /usr4/corpus/hindi      | wi.nam    | i8  | ' ' | word.ndx | word.frq | /usr4/corpus/hnd
#> punjabi | normal    | /usr4/corpus/punjabi    | wi.nam    | i8  | ' ' | word.ndx | word.frq | /usr4/corpus/pnj


$corptabfile = '/usr4/corpus/corptab';
# &ldcorptab;  # For standalone testing.
sub ldcorptab {
  local ($c, $found, @corptabline);
  open (CORPTAB, $corptabfile) || die "Unable to open corpus table file: $corpdir.\n";
  $c = -1;
  while (<CORPTAB>) {
    chop;
    if (/^\s+/) { $_ = $'; }
    if (/^#/) { next; }  # Ignore the comment line.
    @corptabline = split (/\s*\|\s*/);
    if ($#corptabline < 6) { 
      print STDERR "ldcorptab.pl: Ignoring illegal/incomplete format in $corptabfile: $_\n";
      next;
    }
    $c++;
    ($langtab[$c], $origtab[$c], $dirtab[$c], $namestab[$c], $scrpttab[$c], $eostab[$c], $witab[$c], $wordfrqtab[$c], $abbrdirtab[$c])  = @corptabline;
  }
  close (CORPTAB);
  # for ($i=0; $i <= $#langtab;  $i++) {  # for loop for debugging
  #   print STDERR  'DEB: ', $langtab[$i], ' ', $origtab[$i], ' ', $dirtab[$i], ' ', $namestab[$i], ' ', $scrpttab[$i], ' ', $eostab[$i], ' ', $witab[$i], ' ', "\n";
  # }
}
1;
