#! /usr/bin/perl
# Takes an input file containing the specification for morph
# patterns about which words have to be selected.
# Example morph pattern specification (with \n denoting new line char):
#  { \n n,parsarg = yoVkka \n n,gender = p \n } \n 
#  { \n *,root = ivvu \n } \n 
# Creates the following data structure:
#  $wordspec[0] = 0
#       $catfld[0] = 'n,parsarg'; $patrn[0] = yoVkka;
#       $catfld[1] = 'n,gender';  $patrn[1] = p;
#  $wordspec[1] = 2
#       $catfld[2] = 'v,tam';  $patrn[1] = cond_ppl;
#  $wordspec[2] = 3
#  $#wordspec = 2
# Note that the reserved word 'root' refers to the root of the word entry.

sub load_patmo {
  local($filename) = @_[0];
  local (@specinfo, $i, $j, $foundone, $wordnum, $patnum, $oldlimtr);
  open (SPECF, $filename) ||
    die "Unable to open morph spec file $filename\n";
  $wordnum = 0;
  $patnum = 0;
  $oldlimtr = $/;
  $/ = '}';
  while (<SPECF>) {  # Read a new word spec
    if (/^\s*$/) { next; }    # blank space found not a new struc.
    if (/^\n+/) { $_ = $'; }  # Remove new lines in the beginning.
    if (/\n+$/) { $_ = $`; }  # Remove new lines at the end.
    @specinfo = split (/\n/);
    if ($specinfo[0] ne '{') {
      print STDERR "Expected '{' not found: $specinfo[0], $specinfo[1],...\n";
      next;
    }
    $wordspec[$wordnum] = $patnum;
    $foundone = 0;
    for ($i=1;   $i < $#specinfo;   $i++) {
      if ($specinfo[$i] =~ /^\s*(\S+\,\S+)\s*=\s*(\S+)\s*$/) {
        $catfld[$patnum] = $1;
        $patrn[$patnum] = $2;
        $patnum++;
        $foundone = 1;
      }
      else {
        print STDERR "Format of pattern not recognized $specinfo[$i]. Ignoring.\n";
      }
    }
    if ($foundone) { $wordnum++; }
  }
  $wordspec[$wordnum] = $patnum;
  close (SPECF);
  $/ = $oldlimtr;
}
1;
