#!/usr/bin/perl
#!/usr/bin/perl
# Searches for a given pattern and extracts it from given files (say, abc*.sm).
#   Example Usage: xtr.cmd -ppat [-dxyz] abc*.sm >pat.xtr
# Options:
#   -pxyz indicates that the 'xyz' pattern is to be searched
#         (can use regular expressions of Perl for pattern). 
#   -noref tells it not to put file name etc. in the extracted lines
#         (Default is to put file name. Line num is also put
#               if delimiter is new line.)
#   -dxyz indicates that 'xyz' should be used as text line end mark.
#   -dn   indicates use of new line as end marker.
#         (Default delimiter is '<.>'.)
# cmd type many to one.
# Options:
# -p	-  Pattern to be searched.
# -t	-  Type of pattern (1 as it is, 2 word, 3 prefix, 4 suffix)
# -d	-  Delimiter to be used for end of 'sentence' mark.
# -noref	-  References not to be put with 'sentences'.
# -c	-  Clean the corpus (0=no clean, 1=join words). Default: 0

require("/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/xtr.pl");
require ("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $arg, $outf, $argsinfile, $Finname, $Foutname);
local ($pat , $strtype, $delim, $noref, $clean);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$argsinfile = 0;  # Default args file names are not in a file.
# $pat  -  Pattern to be searched.
$strtype = 1 ; # Default  Type of pattern (1 as it is, 2 word, 3 prefix, 4 suffix)
$delim = '<.>'; # Default  Delimiter to be used for end of 'sentence' mark.
$noref = 0; #  References not to be put with 'sentences'. Default as shown.
$clean = 0; # Default  Clean the corpus (0=no clean, 1=join words). Default: 0

# Look up .xtrrc file. Override defaults.
if (open (RC, '.xtrrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/xtr.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'p') {
        $pat  = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  't') {
        $strtype = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  'd') {
        $delim = substr ($_, 2);
      }
      elsif (substr($_,1, 5) eq  'noref') {
        $noref = 1;
      }
      elsif (substr($_,1, 1) eq  'c') {
        $clean = substr ($_, 2);
      }
      else { 
          print STDERR "xtr.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/xtr.cmd');
          die;
      }
    }
    else { # 
      print STDERR "xtr.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/xtr.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1,1) eq 'o') { # Specifies output file.
      $outfilename = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'p') {
      $pat  = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  't') {
      $strtype = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'd') {
      $delim = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 5) eq  'noref') {
      $noref = 1;
    }
    elsif (substr($ARGV[$i],1, 1) eq  'c') {
      $clean = substr ($ARGV[$i], 2);
    }
    else { print STDERR "xtr.cmd: Unrecognized param: $ARGV[$i].\n";
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/xtr.cmd');
      die;
    }
  }
  else { # Argument found. Parameters end.
    if (substr($ARGV[$i],0,1) eq '@') {  # A file of args found
      $argsinfile = 1;
      $ARGV[$i] = substr($ARGV[$i], 1);
    }
    $argbegpos = $i;
    last;
  }
}

#print STDERR "xtr.cmd: DEB params end: $parmlastpos;  args begin: $argbegpos; total: $#ARGV;\n";

# Process the arguments in the command line then call xtr.
# 1. Can be used as a filter. OR
# 2. If arg file names are given, they are treated as input files
#    and the output goes to a single file stdout.

if ($outfilename ne '') {
  open (OUT, ">$outfilename") ||
    die "xtr.cmd: Unable to open output file $outfilename.\n";
  $outf = OUT;
  $Foutname = $outfilename;
}
else { $outf = STDOUT; }

if ($argbegpos < 0) { # No args.
  &xtr(STDIN, $outf);
}
elsif ($argsinfile) {  # Argument filenames in file.
  open (ARGFILE, "$ARGV[$argbegpos]") ||
    die "xtr.cmd: Unable to open args file $ARGV[$argbegpos]\n" ;
  while (<ARGFILE>) { # Read args.
    chop;
    if (!(open (INFILE, $_))) {
      print STDERR "xtr.cmd: Unable to open file $_\n" ;
      next;
    }
    $Finname = $_;
    &xtr (INFILE, $outf);
    close (INFILE);
  }
  close (ARGFILE);
}
else { # Call the program with each arg. 
  for ($arg = $argbegpos; $arg <= $#ARGV; $arg++) {
    if (!(open (INFILE, "$ARGV[$arg]"))) {
      print STDERR "xtr.cmd: Unable to open file $ARGV[$arg]. Continuing.\n" ;
      next;
    }
    $Finname = $ARGV[$arg];
    &xtr (INFILE, $outf);
    close (INFILE);
  }
}
close ($outf);
 
