#!/usr/bin/perl
# Searches for the given word in the input file,
# and aligns the lines so that the matched word occurs
# on the same coloumn in each line.
#   Example usage: algn_vert.cmd -pPAT < PAT.xtr > PAT.aln
#   Example usage: algn_vert.pl -pPAT  < PAT.xtr > PAT.aln
# Options:
# -p	-  Pattern at which the line has to be aligned.  
# -t	-  Type of pattern (1 String, 2 Word, 3 Prefix, 4 Suffix, 5 Use STEM_FILE.iref file to get patterns). (Default '1')  
# -c	-  Column number at which the pattern will be placed. Default:35 
# -no	-  Do not put spaces before the pattern. Default: yes. 
# -s	-  Suffix for default output file name. Default: '.aln'  
# -N	-  Name of the stem filename ($stem_filename.iref will be looked up.) 

require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $arg, $suff, $argsinfile, $Finname, $Finname);
local ($pat, $strtype, $col, $nospace, $suff, $stem_filename);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$argsinfile = 0;  # Default args file names are not in a file.
$suff = '.new'; # Default suffix for output file if necessary.
$pat = 'test'; # Default  Pattern at which the line has to be aligned.  
$strtype = '1'; # Default  Type of pattern (1 String, 2 Word, 3 Prefix, 4 Suffix, 5 Use STEM_FILE.iref file to get patterns). (Default '1')  
$col = 35; # Default  Column number at which the pattern will be placed. Default:35 
$nospace = 0; #  Do not put spaces before the pattern. Default: yes.  Default value as shown.
$suff = '.aln'; # Default  Suffix for default output file name. Default: '.aln'  
$stem_filename = ''; # Default  Name of the stem filename ($stem_filename.iref will be looked up.) 

# Look up .algn_vertrc file. Override defaults.
if (open (RC, '.algn_vertrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/algn_vert.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'p') {
        $pat = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  't') {
        $strtype = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  'c') {
        $col = substr ($_, 2);
      }
      elsif (substr($_,1, 2) eq  'no') {
        $nospace = 1;
      }
      elsif (substr($_,1, 1) eq  's') {
        $suff = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  'N') {
        $stem_filename = substr ($_, 2);
      }
      else { 
          print STDERR "algn_vert.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/algn_vert.cmd');
          die;
      }
    }
    else { # 
      print STDERR "algn_vert.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/algn_vert.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1,1) eq 's') { # Suffix for output file names.
       $suff = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'p') {
      $pat = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  't') {
      $strtype = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'c') {
      $col = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 2) eq  'no') {
      $nospace = 1;
    }
    elsif (substr($ARGV[$i],1, 1) eq  's') {
      $suff = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'N') {
      $stem_filename = substr ($ARGV[$i], 2);
    }
    else  { print STDERR "algn_vert.cmd: Parameter not recognized $ARGV[$i].\n";
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/algn_vert.cmd');
      die;
    }
  }
  else { # Argument found. Parameters end.
    if (substr($ARGV[$i],0,1) eq '@') {  # A file of args found
      $argsinfile = 1;
      $ARGV[$i] = substr($ARGV[$i], 1);
    }
    $argbegpos = $i;
    last;
  }
}
#print STDERR "algn_vert.cmd: DEB params end: $parmlastpos;  args begin: $argbegpos; total: $#ARGV;\n";

# Process the arguments in the command line then call algn_vert.
# 1. Can be used as a filter. OR
# 2. If one or more file arguments are given, they are treated as
#    inputs files, and the output file name is given
#    by the input file name suffixed by the default suffix
#    (which can be changed by approp flag).

if ($argbegpos < 0) { # No args.
  &algn_vert(STDIN, STDOUT);
}
elsif ($argsinfile) {  # Argument filenames in file.
  open (ARGFILE, "$ARGV[$argbegpos]") ||
    die "algn_vert.cmd: Unable to open args file $ARGV[$argbegpos]\n" ;
  while (<ARGFILE>) { # Read args.
    chop;
    if (!(open (INFILE, $_))) {
      print STDERR "algn_vert.cmd: Unable to open file $_\n" ;
      next;
    }
    if (!(open (OUTFILE, ">$_$suff"))) {
      print STDERR "algn_vert.cmd: Unable to open out file $_$suff\n";
      close (INFILE);
      next;
    }
    $Finname = $_;  $Foutname = $_ . $suff;
    &algn_vert (INFILE, OUTFILE);
    close (INFILE);
    close (OUTFILE);
  }
  close (ARGFILE);
}
else { # Call the program with each arg. 
  for ($arg = $argbegpos; $arg <= $#ARGV; $arg++) {
    if (!(open (INFILE, "$ARGV[$arg]"))) {
      print STDERR "algn_vert.cmd: Unable to open file $ARGV[$arg]\n" ;
      next;
    }
    if (!(open (OUTFILE, ">$ARGV[$arg]$suff"))) {
      print STDERR "algn_vert.cmd: Unable to open out file $ARGV[$arg]$suff\n";
      close (INFILE);
      next;
    }
    $Finname = $ARGV[$arg];
    $Foutname = $ARGV[$arg] . $suff;  
    &algn_vert (INFILE, OUTFILE);
    close (INFILE);
    close (OUTFILE);
  }
}
#------------------------------------------------------

sub algn_vert {
local ($W_fin, $W_fout) = @_;
select($W_fout);
#%COMMENT
# Searches for the given word in the input file,
# and aligns the lines so that the matched word occurs
# on the same coloumn in each line.
#   Example usage: algn_vert.cmd -pPAT < PAT.xtr > PAT.aln
#   Example usage: algn_vert.pl -pPAT  < PAT.xtr > PAT.aln
#%CMD TYPE MM
require('/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/cons_srchpat.pl');

local ($srchpat, $rightcount, $i, $spacecount);
local ($lenpat, $lenleft, $left, $right, $l, $m, $r);
$srchpat = &cons_srchpat;
if ($srchpat eq '') {
  die "algn_vert.pl: No string to align. align string = NIL.\n";
}
print STDERR "algn_vert.pl:DEBUG: align string = $srchpat,\n";

 #$lenpat = length ($pat);
 #$rightcount = 72 - $col - $lenpat - 2;
 #if ($nospace) { $rightcount += 2; }
while (<$W_fin>) {
  chop;
  s/\t/ /g;
  if (/$srchpat/o) {
    $l = $`;  $m = $&;  $r = $';
    $matchpat = $m;  $lenpat = length($matchpat);
    $rightcount = 72 - $col - $lenpat - 2;
    if ($nospace) { $rightcount += 2; }
    $lenleft = length ($l);
    if ($lenleft >= $col) {
      $left = substr ($l, $lenleft - $col, $col); $spacecount = 0;
    }
    else { $left = $l; $spacecount = $col - $lenleft; }
    for ($i=1; $i <= $spacecount;  $i++) { print ' '; }
    $right = substr ($r, 0, $rightcount);
    #if ($matchpat =~ /\s$/) { $right = ' ' . $right; } 
    #if ($matchpat =~ /^\s/) { $left .= ' '; } 
     #print STDERR "DEBUG: lenleft= $lenleft, lenpat= $lenpat, lenright= $lenright,\n";
     #print STDERR $left, '  |', $matchpat, '|', $right, "\n";
    if ($nospace) { print $left, $matchpat, $right, "\n"; }
    else { print $left, '  ', $matchpat, $right, "\n"; }
  }
}

}
1;
