#!/usr/bin/perl
# Searches for a given pattern and extracts it from given files (say, abc*.sm).
#   Example Usage: xtriref.cmd -sSTR -tTYPE  WI*.TXT > STR.iref
# Where WI*.TXT are word-irefs files for a given corpus.
# cmd type many to one.
# Options:
# -s	-  String to be searched.
# -t	-  Type of string (1 String, 2 Word, 3 Prefix, 4 Suffix). (Default '1')

require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/xtriref.pl");
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/iref2name.pl");

# DBM not being used now.
# dbmopen (%word_iref, $dbmfile, undef) ||   # Open for read only.
#   die "Unable to open DBM file: $dbmfile.\n";
# close (DBM);
# -f $dbmfile = '/usr3/corpus/telugu/wrdiref';	# Name of the database file. (Default: wrdiref)


# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $arg, $outf, $argsinfile, $Finname, $Foutname);
local ($str , $strtype);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$argsinfile = 0;  # Default args file names are not in a file.
# $str  -  String to be searched.
$strtype = '1'; # Default  Type of string (1 String, 2 Word, 3 Prefix, 4 Suffix). (Default '1')

# Look up .xtrirefrc file. Override defaults.
if (open (RC, '.xtrirefrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/xtriref.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  's') {
        $str  = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  't') {
        $strtype = substr ($_, 2);
      }
      else { 
          print STDERR "Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/xtriref.cmd');
          die;
      }
    }
    else { # 
      print STDERR "Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/xtriref.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1,1) eq 'o') { # Specifies output file.
      $outfilename = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  's') {
      $str  = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  't') {
      $strtype = substr ($ARGV[$i], 2);
    }
    else { print STDERR "Unrecognized param: $ARGV[$i].\n";
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/xtriref.cmd');
      die;
    }
  }
  else { # Argument found. Parameters end.
    if (substr($ARGV[$i],0,1) eq '@') {  # A file of args found
      $argsinfile = 1;
      $ARGV[$i] = substr($ARGV[$i], 1);
    }
    $argbegpos = $i;
    last;
  }
}

#print STDERR "DEB params end: $parmlastpos;  args begin: $argbegpos; total: $#ARGV;\n";

# Process the arguments in the command line then call xtriref.
# 1. Can be used as a filter. OR
# 2. If arg file names are given, they are treated as input files
#    and the output goes to a single file stdout.

if ($outfilename ne '') {
  open (OUT, ">$outfilename") ||
    die "Unable to open output file $outfilename.\n";
  $outf = OUT;
  $Foutname = $outfilename;
}
else { $outf = STDOUT; }

if ($argbegpos < 0) { # No args.
  &xtriref(STDIN, $outf);
}
elsif ($argsinfile) {  # Argument filenames in file.
  open (ARGFILE, "$ARGV[$argbegpos]") ||
    die "Unable to open args file $ARGV[$argbegpos]\n" ;
  while (<ARGFILE>) { # Read args.
    chop;
    if (!(open (INFILE, $_))) {
      print STDERR "Unable to open file $_\n" ;
      next;
    }
    $Finname = $_;
    &xtriref (INFILE, $outf);
    close (INFILE);
  }
  close (ARGFILE);
}
else { # Call the program with each arg. 
  for ($arg = $argbegpos; $arg <= $#ARGV; $arg++) {
    if (!(open (INFILE, "$ARGV[$arg]"))) {
      print STDERR "Unable to open file $ARGV[$arg]. Continuing.\n" ;
      next;
    }
    $Finname = $ARGV[$arg];
    &xtriref (INFILE, $outf);
    close (INFILE);
  }
}
close ($outf);
 
# &iref2name();

