#!/usr/bin/perl
# Takes a file of Hindi headers, and puts subjects.
#   Example usage: hdr2subj.pl Subj.comp < Headers.lst   > Subj_small.lst
# The input file has file number followed by the header.

# It uses two global associative arrays keeping the mapping between
# classification number and subject:
#  %classi_subj,   and   %subj_classi

# A sample HEADERS.hlp file:
# 1 <Official And Media Languages> <Mass Media> <Mishra, Vijay Kumar> <Sanjay Gandhi Jid Aur Jokhim Se Bhari Zindagi> <1980> <Jyo-Yugeen Sahitya Chetna> <Delhi><57-59> <1500> <Mishra - Sanjay Gandhi Jid Aur Jokhim> <H. S. Wadhawan> 
# 2 <Translated Material><Literature><Das, Jogesh><Prithvi ki peeda><1988><Sahitya Akademi><New Delhi><84-89><2100><Das, J-Prithvi Ki peeda-Lt-O><Abha Sood>
# 3 <Official And Media Languages><Mass Media><Anorymous><Haryana : Pehli Pari Devilal Ki><1986><DIN - Times Of India><New Delhi><15-17><2040><Anon - Haryana : Pehli Pari Devilal Ki><H.S.Wadhawan>

require ('/usr4/corpus/sbin/ldsubj.pl');

local ($fileno, @words, $majsubj, $classif);
$filename = $ARGV[0];
open (FCAT, "$filename") ||
  die "hdr2subj.pl: Unable to open subject file $filename\n";
&ldsubj (FCAT);
close FCAT;
while (<STDIN>) {
  $fullline = $_;
  if (/^([0-9]+)\s*</) { $_ = $';  $fileno = $1; }  
                    # ^^ Remove file no. from the leading part of the line.
  if (/>\s*$/) { $_ = $`; } # Remove the trailing part of the line.
  @words = split (/>\s*</);
     #print STDERR "hdr2subj.pl DEBUG: Headers has words: @words\n";
  if ($#words != 10) {
    print STDERR "hdr2subj.pl: Invalid header in line $.: $_";
  }
  ($majsubj, $minsubj, $author, $title, $year, $publ,
     $place, $pages, $numwords, $cright, $dateentry) = @words;
  $majsubj =~ tr/A-Z/a-z/;
  $minsubj =~ tr/A-Z/a-z/;
  $classif = $subj_classi{$minsubj};
  if ($classif eq '') { $classif = $subj_classi{$majsubj}; }
  if ($classif eq '') { 
    print STDERR "hdr2subj.pl: Classification number not found:$_";
  }
  print $classif, $fullline;
}
1;
