#!/usr/bin/perl
# THIS DIFFERS FROM IREFMK.CMD FILE IN THAT IT TRIES TO MAKE
# DBM FILES DIRECTLY. (THIS IS NOW SUPERSEDED.)
# Takes text files and builds an exhaustive index on each word
# indicating files in which it occurs.
# For space efficiency, numbers instead of file names are used.
# These numbers are called 'irefs' for indexed refs.
#   Recommended usage: irefmkd.cmd  @F   > F.irefs
# where file F contains files names (each in a separate line) say
# A, ... Z. It takes files named A to Z, and for each word that occurs
# in the files, stores the word followed by irefs for files
# in which it occurs. (iref for file A is 1, for B is 2, and so on.)
#   Future Example usage:  irefmkd.cmd  F1 ... Fn  > F.irefs
# Options:
# -mem	-  Maximum number of words in working memory (for efficiency only). Use 0 if using DBM.
# -f	-  Name of dbm file existing or to be created for words and their irefs which occur in the corpus to be indexed.
# -n	-  iref number for the files completed. (Default 0

require("/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/irefmkd.pl");
require ("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $arg, $outf, $argsinfile, $Finname, $Foutname);
local ($inmemmax, $dbmfile, $curiref);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$argsinfile = 0;  # Default args file names are not in a file.
$inmemmax = 0; # Default  Maximum number of words in working memory (for efficiency only). Use 0 if using DBM.
$dbmfile = 'wrdiref'; # Default  Name of dbm file existing or to be created for words and their irefs which occur in the corpus to be indexed.
$curiref = 0; # Default  iref number for the files completed. (Default 0

# Look up .irefmkdrc file. Override defaults.
if (open (RC, '.irefmkdrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/irefmkd.cmd');
        die;
      }
      elsif (substr($_,1, 3) eq  'mem') {
        $inmemmax = substr ($_, 4);
      }
      elsif (substr($_,1, 1) eq  'f') {
        $dbmfile = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  'n') {
        $curiref = substr ($_, 2);
      }
      else { 
          print STDERR "Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/irefmkd.cmd');
          die;
      }
    }
    else { # 
      print STDERR "Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/irefmkd.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1,1) eq 'o') { # Specifies output file.
      $outfilename = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1, 3) eq  'mem') {
      $inmemmax = substr ($ARGV[$i], 4);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'f') {
      $dbmfile = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'n') {
      $curiref = substr ($ARGV[$i], 2);
    }
    else { print STDERR "Unrecognized param: $ARGV[$i].\n";
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/irefmkd.cmd');
      die;
    }
  }
  else { # Argument found. Parameters end.
    if (substr($ARGV[$i],0,1) eq '@') {  # A file of args found
      $argsinfile = 1;
      $ARGV[$i] = substr($ARGV[$i], 1);
    }
    $argbegpos = $i;
    last;
  }
}

#print STDERR "DEB params end: $parmlastpos;  args begin: $argbegpos; total: $#ARGV;\n";

# Process the arguments in the command line then call irefmkd.
# 1. Can be used as a filter. OR
# 2. If arg file names are given, they are treated as input files
#    and the output goes to a single file stdout.

dbmopen (%word_iref, $dbmfile, 0644) ||
  die "Unable to open DBM file: $dbmfile.\n";

if ($outfilename ne '') {
  open (OUT, ">$outfilename") ||
    die "Unable to open output file $outfilename.\n";
  $outf = OUT;
  $Foutname = $outfilename;
}
else { $outf = STDOUT; }

if ($argbegpos < 0) { # No args.
  &irefmkd(STDIN, $outf);
  local (%w_iref);  # Insert in loop after call to &irefmkd.
}
elsif ($argsinfile) {  # Argument filenames in file.
  open (ARGFILE, "$ARGV[$argbegpos]") ||
    die "Unable to open args file $ARGV[$argbegpos]\n" ;
  while (<ARGFILE>) { # Read args.
    chop;
    if (!(open (INFILE, $_))) {
      print STDERR "Unable to open file $_\n" ;
      next;
    }
    $Finname = $_;
    &irefmkd (INFILE, $outf);
    local (%w_iref);  # Insert in loop after call to &irefmkd.
    close (INFILE);
  }
  close (ARGFILE);
}
else { # Call the program with each arg. 
  for ($arg = $argbegpos; $arg <= $#ARGV; $arg++) {
    if (!(open (INFILE, "$ARGV[$arg]"))) {
      print STDERR "Unable to open file $ARGV[$arg]. Continuing.\n" ;
      next;
    }
    $Finname = $ARGV[$arg];
    &irefmkd (INFILE, $outf);
    local (%w_iref);  # Insert in loop after call to &irefmkd.
    close (INFILE);
  }
}
close ($outf)
