# Called by irefmkd.cmd.
# THIS DIFFERS FROM IREFMK.PL FILE IN THAT IT TRIES TO MAKE
# DBM FILES DIRECTLY. (THIS IS NOW SUPERSEDED.)
# irefmkd Makes exhaustive index for every word in a file.
# Following variables are passed from irefmkd.cmd.
#  $curiref	- iref number incremented each time irefmkd is called.
#  $inmemmax	- Max no. of words to be kept in core (for efficiency only). set by the calling routine.
#  $wrdfreq	- Name of file containing words and their frequencies.
#  $dbmfile	- Name of the DBM file (corresponding to %word_iref).
# Global variables (%word_iref gets its value from DBM files):
#  %word_irefmem	- assoc array between each word and its irefs, kept in mem.
#  %word_iref	- assoc array between each word and its irefs, kept in dbm.
#  %w_iref	- Assoc array to store words found from a file. Discarded after the file is processed.

# There are two global associative arrays which store the words
# and their irefs (i.e., the references to files in which they occur): 
#   word_irefmem       word_iref
# The former keeps things in memory, while the latter keeps things
# in DBM files. (The two arrays are there to experiment with efficiency.
# $inmemmax is there to control the size of the former, and could
# be even made 0.)
# 
# The main routine is `irefmkd' which is called with a input filehandle
# and the iref for the input file (in $curiref). It adds the iref to the
# list of irefs for the words in the file in the two global arrays.

# irefmkd Makes exhaustive index for every word in the given
# input file ($fin) and given iref ($curiref).
sub irefmkd {
  local ($fin, $fout) = @_;
  local ($w, $iref, @words, $val, $newwords, $newentry);
  $curiref++;
  while (<$fin>) { # Collect all the unique words from this file.
    chop;
    if (/^\s+/) { $_ = $'; }
    &rm_punct;  # Remove punctuation marks, <sNNN>, <.> etc. that should not be indexed.
    @words = split (/ +/);
    foreach $w (@words) { 
      if ($w ne '') {
        $w_iref {$w} = $curiref;
        # print STDERR "DEBUG: Word found is: $w.\n";
      }
    }
  }

  # while ( ($w, $iref) = each %w_iref) {
  #   print STDERR "DEBUG: word in this file = $w,\tiref = $iref.\n";
  # }
  $newentry = 0;
  while ( ($w, $iref) = each %w_iref) {
    if ($word_irefmem{$w} ne '') { 
      if (!($word_irefmem{$w} =~  / $iref( |$)/)) {
        $word_irefmem{$w} .= " $iref";
        $newentry = 1;
      }
    }
    elsif (($val = $word_iref{$w}) ne '') { 
      if (!($val =~  / $iref( |$)/)) {
        $word_iref{$w} = $val . " $iref";
        $newentry = 1;
      }
    }
    else { 
      # $newwords .= " $w"; # For debugging (A1).
      # print STDERR "DEBUG: New word in this file: $val. Ignored.\n"; # if assignment in next line is not being made.
      $word_iref{$w} = " $iref";
      # print STDERR "DEBUG: New word in this file: $w. Added.\n"; # if assignment in prev line is being made.
    }
  }
  if ($curiref == 1) { $newentry = 1; } # Ignore the first file.
  if ($newentry == 0) {
    print STDERR "No new entry made for file $Finname (with iref $curiref).\n";
  }
  # if ($newwords ne '') { # This and next 3 lines for debugging (A1).
  #   print STDERR "Some new words found in file = $Finname, iref = $curiref.\n";
  #   # print STDERR $newwords, "\n";  # Use for debugging only.
  # }
  # while ( ($w, $iref) = each %word_irefmem) {
  #   print STDERR "DEBUG: word_irefmem = $w,\tiref = $iref.\n";
  # }
  # print STDERR "DEBUG: -----------------------\n";
  # while ( ($w, $iref) = each %word_iref) {
  #   print STDERR "DEBUG: word_iref = $w,\tiref = $iref.\n";
  # }
  # print STDERR "DEBUG: -----------------------\n";
}
sub rm_punct {
    #s/lY/lV/g; Only for Kannada
  s/rY/r/g;
  s/Z//g;
  s/#/$ /g;
  s/[!"'`\?\(\)\*\+\,\.\[\]\{\}\;\:]/ /g; # Not removed '-'.
  s/<s[0-9]+>/ /g;
  s/<.>/ /g;
    #s/"/ /g;
    #s/,/ /g;
    #s/\./ /g;
    #s/\?/ /g;
    #s/!/ /g;
    #s/;/ /g;
    #s/:/ /g;
    #s/\)/ /g;
    #s/\(/ /g;
    #s/'/ /g;
    #s/`/ /g;
    #s/_/ /g;
}
1;
