#To setup a new corpus for a language,
# 1. login as corpus
# 2. Define shell variables below.
# 3. Create directory corresponding to CorpDir.
#    Actually copy the corpus text files.
# 4. Create wi.nam
# 5. Put suitable entries in ~corpus/corptab particularly for Corp.
# 6. Create a file Subj.comp  (to be used for subject classification)
# 7. Modify the line containing lang_hdr after looking at the number 
#    of lines containing the header.
# Now run this script.

# Defining shell variables used in this script
CorpHome=/usr4/corpus
CorpDir=/usr4/corpus/kannada # This should be the same as in corptab
Corp=/usr4/corpus/knn        # This should be the same as in corptab
Lang=kannada

# The splitting is carried out first (to be done only if the files in the
# corpus have several texts each):
#    mkdir $CorpDir
#    cd $CorpDir
#    $CorpHome/sbin/hindisplit.cmd  @../hindiorig/wiorig.nam

cd $CorpDir

# The list of filenames is created as follows after the above:
#   ls -lR > files.all
#   /home/bharathi/PNJ1/usr3/sangal/tools/src/etc/ls2names.pl  < files.all  > wi.nam 
#   ln -s  $CorpDir/wi.nam names.all 

# The headers out of the files are pulled out as follows:
#   $CorpHome/sbin/hindi_hdr.cmd  @wi.nam > Headers.lst  2> Headers.err
#                       OR 
$CorpHome/sbin/lang_hdr.cmd -l1  @wi.nam > Headers.lst  2> Headers.err

# The word counts for files are created as follows:
$CorpHome/sbin/lang_wc.cmd  @wi.nam > wc.lst  2> wc.err

# A classified list of files along with subj is created in 
# dir Subjects as follows:
$CorpHome/sbin/subj_cr.sh  $CorpDir
# Creates a summary of data regarding each subjects.
cd $CorpDir/Subjects
$CorpHome/sbin/subj_summ.cmd -f../wc.lst  < $CorpHome/Subj.comp > ../Subj.sum 
cd $CorpDir

# word.ndx file is created as follows:
/usr/bin/time  sh $CorpHome/sbin/wisetup.sh $CorpDir    # Giving dir name

# Cleaned word-index file is created as follows:
$CorpHome/sbin/wicln.pl < word.ndx  | sort > word_cln.ndx 2> word_cln.err 

# Abbreviated hindi directory is created as follows:
# (This contains file names called 1, 2, 3 ... linked to the main language
# corpus in $CorpDir.)
#    a. First create an entry say '/usr4/corpus/hnd' as the name of the
#       abbreviated directory in ~corpus/corptab.
#    b. Then:
$CorpHome/sbin/mkabbr.cmd -l$Lang -tnormal # Creates dir for lang and file 'mkabbr.sh'
cd $Corp
mkabbr.sh
cd $CorpDir

# Word frequency file is created as follows:
#cd $CorpDir
#mkdir tmp
#cd tmp
#cp ../names.all .
#nohup /usr/bin/time $CorpHome/sbin/prp_frq_cnt.sh 
#mv freq.cnt ../word.frq
#cd ..
##rm -r tmp
$CorpHome/sbin/cum_frq.pl  word.frq  > wordfrq.cum
$CorpHome/sbin/mk_cover.sh

# Word length frequency and count file:
$CorpHome/sbin/wrdlenfrq.pl  < word.frq  > wordlen.frq
$CorpHome/sbin/corplen1.sh   < wordlen.frq  |  \
   ~sangal/tools/src/etc/ave.pl  > wordlen.ave

# Create root.ndx file after running morph.
$CorpHome/sbin/word_mo_setup.pl > do.tmp
chmod +x do.tmp
do.tmp &
