#!/usr/bin/perl
# Hindi corpus as given has several large files, each file
# containing a large number of texts.
# This splits the given files in Hindi corpus into smaller files
# each containing a different text.
#   Example usage:  hindisplit.pl BASEFILENAME.xxx.yyy 
# The above produces several files each containing a different
# text named as:
#   BASEFILENAME/1, BASEFILENAME/2, ...
# Options:
# -DUM	-  Dummy option. Not being used.

require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/hindisplit.pl");


# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $arg, $suff, $argsinfile, $Finname, $Finname);
local ($dummy );
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$argsinfile = 0;  # Default args file names are not in a file.
$suff = '.new'; # Default suffix for output file if necessary.
# $dummy  -  Dummy option. Not being used.

# Look up .hindisplitrc file. Override defaults.
if (open (RC, '.hindisplitrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/hindisplit.cmd');
        die;
      }
      elsif (substr($_,1, 3) eq  'DUM') {
        $dummy  = substr ($_, 4);
      }
      else { 
          print STDERR "hindisplit.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/hindisplit.cmd');
          die;
      }
    }
    else { # 
      print STDERR "hindisplit.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/hindisplit.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1,1) eq 's') { # Suffix for output file names.
       $suff = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1, 3) eq  'DUM') {
      $dummy  = substr ($ARGV[$i], 4);
    }
    else { print STDERR "hindisplit.cmd: Parameter not recognized $ARGV[$i].\n";
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/hindisplit.cmd');
      die;
    }
  }
  else { # Argument found. Parameters end.
    if (substr($ARGV[$i],0,1) eq '@') {  # A file of args found
      $argsinfile = 1;
      $ARGV[$i] = substr($ARGV[$i], 1);
    }
    $argbegpos = $i;
    last;
  }
}
#print STDERR "hindisplit.cmd: DEB params end: $parmlastpos;  args begin: $argbegpos; total: $#ARGV;\n";

# Process the arguments in the command line then call hindisplit.
# 1. Can be used as a filter. OR
# 2. If one or more file arguments are given, they are treated as
#    inputs files, and the output file name is given
#    by the input file name suffixed by the default suffix
#    (which can be changed by approp flag).

if ($argbegpos < 0) { # No args.
  &hindisplit(STDIN, STDOUT);
}
elsif ($argsinfile) {  # Argument filenames in file.
  open (ARGFILE, "$ARGV[$argbegpos]") ||
    die "hindisplit.cmd: Unable to open args file $ARGV[$argbegpos]\n" ;
  while (<ARGFILE>) { # Read args.
    chop;
    # print STDERR "hindisplit.cmd DEBUG: filename $_ in AT file.\n";
    if (!(open (INFILE, $_))) {
      print STDERR "hindisplit.cmd: Unable to open file $_\n" ;
      next;
    }
    if (!(open (OUTFILE, ">$_$suff"))) {
      print STDERR "hindisplit.cmd: Unable to open out file $_$suff\n";
      close (INFILE);
      next;
    }
    $Finname = $_;  $Foutname = $_ . $suff;
    &hindisplit (INFILE, OUTFILE);
    close (INFILE);
    close (OUTFILE);
  }
  close (ARGFILE);
}
else { # Call the program with each arg. 
  for ($arg = $argbegpos; $arg <= $#ARGV; $arg++) {
    if (!(open (INFILE, "$ARGV[$arg]"))) {
      print STDERR "hindisplit.cmd: Unable to open file $ARGV[$arg]\n" ;
      next;
    }
    if (!(open (OUTFILE, ">$ARGV[$arg]$suff"))) {
      print STDERR "hindisplit.cmd: Unable to open out file $ARGV[$arg]$suff\n";
      close (INFILE);
      next;
    }
    $Finname = $ARGV[$arg];
    $Foutname = $ARGV[$arg] . $suff;  
    &hindisplit (INFILE, OUTFILE);
    close (INFILE);
    close (OUTFILE);
  }
}

