#!/usr/bin/perl
# Takes text files and builds an exhaustive index on each word
# indicating files in which it occurs.
#   Example usage: irefmk.cmd  -fWW @F
# This produces three output files containing words and their irefs:
#   WW.txt 		- a text file.
#   WW.dir, WW.pag 	- the dbm files.
# (Currently making of dbm files is disabled, eg, nodbm option is always on.)
# It is recommended that the names WW and F are kept the same.
#
# For space efficiency, numbers instead of file names are used.
# These numbers are called 'irefs' for indexed refs.
# where file F contains files names (each in a separate line) say
# A, ... Z. It takes files named A to Z, and for each word that occurs
# in the files, stores the word followed by irefs for files
# in which it occurs. (iref for file A is 1, for B is 2, and so on.)
#   Possible future Example usage:  irefmk.cmd  F1 ... Fn  > F.irefs
# Options:
# -f	-  Name of index file and text for words and their irefs which occur in the corpus. (Default word.ndx)
# -nodbm -  Switch for no dbm file to be produced. (Default 0 - produce dbm)
# -n	-  iref number for the files completed. (Default 0)

require("/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/irefmk.pl");
require ("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $arg, $outf, $argsinfile, $Finname, $Foutname);
local ($wndxfile, $nodbm, $curiref);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$argsinfile = 0;  # Default args file names are not in a file.
$wndxfile = 'word'; # Default  Name of index file and text for words and their irefs which occur in the corpus. (Default word)
$nodbm = 0; #  Switch for no dbm file to be produced. (Default 0) Default as shown.
$curiref = 0; # Default  iref number for the files completed. (Default 0)

# Look up .irefmkrc file. Override defaults.
if (open (RC, '.irefmkrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/irefmk.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'f') {
        $wndxfile = substr ($_, 2);
      }
      elsif (substr($_,1, 5) eq  'nodbm') {
        $nodbm = 1;
      }
      elsif (substr($_,1, 1) eq  'n') {
        $curiref = substr ($_, 2);
      }
      else { 
          print STDERR "Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/irefmk.cmd');
          die;
      }
    }
    else { # 
      print STDERR "Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/irefmk.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1,1) eq 'o') { # Specifies output file.
      $outfilename = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'f') {
      $wndxfile = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 5) eq  'nodbm') {
      $nodbm = 1;
    }
    elsif (substr($ARGV[$i],1, 1) eq  'n') {
      $curiref = substr ($ARGV[$i], 2);
    }
    else { print STDERR "Unrecognized param: $ARGV[$i].\n";
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/irefmk.cmd');
      die;
    }
  }
  else { # Argument found. Parameters end.
    if (substr($ARGV[$i],0,1) eq '@') {  # A file of args found
      $argsinfile = 1;
      $ARGV[$i] = substr($ARGV[$i], 1);
    }
    $argbegpos = $i;
    last;
  }
}

#print STDERR "DEB params end: $parmlastpos;  args begin: $argbegpos; total: $#ARGV;\n";

# Process the arguments in the command line then call irefmk.
# 1. Can be used as a filter. OR
# 2. If arg file names are given, they are treated as input files
#    and the output goes to a single file stdout.

if ($outfilename ne '') {
  open (OUT, ">$outfilename") ||
    die "Unable to open output file $outfilename.\n";
  $outf = OUT;
  $Foutname = $outfilename;
}
else { $outf = STDOUT; }

if ($argbegpos < 0) { # No args.
  &irefmk(STDIN, $outf);
}
elsif ($argsinfile) {  # Argument filenames in file.
  open (ARGFILE, "$ARGV[$argbegpos]") ||
    die "Unable to open args file $ARGV[$argbegpos]\n" ;
  while (<ARGFILE>) { # Read args.
    chop;
    if (!(open (INFILE, $_))) {
      print STDERR "Unable to open file $_\n" ;
      next;
    }
    $Finname = $_;
    &irefmk (INFILE, $outf);
    close (INFILE);
  }
  close (ARGFILE);
}
else { # Call the program with each arg. 
  for ($arg = $argbegpos; $arg <= $#ARGV; $arg++) {
    if (!(open (INFILE, "$ARGV[$arg]"))) {
      print STDERR "Unable to open file $ARGV[$arg]. Continuing.\n" ;
      next;
    }
    $Finname = $ARGV[$arg];
    &irefmk (INFILE, $outf);
    close (INFILE);
  }
}
close ($outf);
&store_word_irefmem();
 
