# Called by irefmk.cmd.
# irefmk Makes exhaustive index in memory for every word in a file.
# Following variables are passed from irefmk.cmd.
#  $curiref	- cur iref number incremented each time irefmk is called.
#  $wndxfile	- Name of the DBM file (corresponding to %word_iref).
# 		  (Making of DBM file is currently disabled.)

# There is a global associative array which store in mem the words
# and their irefs (i.e., the references to files in which they occur): 
#   word_irefmem
# 
# The main routine is `irefmk' which is called with a input filehandle
# and the iref for the input file (in $curiref). It adds the iref to the
# list of irefs for the words in the file in the two global arrays.

# irefmk Makes exhaustive index for every word in the given
# input file handle ($fin) and given iref ($curiref).
sub irefmk {
  local ($fin, $fout) = @_;
  local ($w, $iref, @words, $val, $newentry);
  $curiref++;
  $newentry = 0;
  while (<$fin>) { # Collect all the words from the next line in the file and add to assoc array.
    chop;
    if (/^\s+/) { $_ = $'; }
    # &rm_punct;  # Remove punctuation marks, <sNNN>, <.> etc. that should not be indexed.
    @words = split (/[\",:;'`\\\/\?><\]\[}{\)\(\-_ \r\t=!\@\#%&\$\|\*\+\.\xea0-9]+/);  # Not using: ~
   #@words = split (/[\",:;'`\?><\]\[}{\)\(-_ \t=!\@\#%&\$\~\|\*\.\xea]+/);
    foreach $w (@words) { 
      if ($w ne '') {
        $val = $word_irefmem{$w};
        #print STDERR "DEBUG: Next word is|$w| with iref|$val|\n";
        if (!($val =~  / $curiref( |$)/)) { # Add only if not present
          $newentry = 1;
          $word_irefmem{$w} = "$val $curiref";
          #print STDERR "DEBUG: Added word with iref|$val $curiref|\n";
        }
      }
    }  # End foreach
  }

  if ($newentry == 0) {
    print STDERR "No new entry made for file $Finname (with iref $curiref).\n";
  }
  #while ( ($w, $iref) = each %word_irefmem) {
  #  print STDERR "DEBUG: word_irefmem = $w,\tiref = $iref.\n";
  #}
}

# Write the words in %word_irefmem to text file.
# Making of DBM file disabled.
sub store_word_irefmem {
  local ($w, $iref);
  print STDERR "Date: ";
  system ("date 1>&2");
  if (open (TXT, ">$wndxfile.ndx")) { # Write into a text file.
    while ( ($w, $iref) = each %word_irefmem) {
      print TXT $w, "  ", $iref, "\n";
    }
  }
  else {
    print STDERR "Unable to open output file $wndxfile.ndx.\n";
  }
  if ($nodbm) { 
    #print STDERR "DEBUG: skipping making dbm files.\n";
    return;
  }

  # Currently the dbm file making has been removed.
  # dbmopen (%new_word_iref, $wndxfile, 0644) ||
  #   die "Unable to open DBM file: $wndxfile.\n";
  # print STDERR "Starting to write word-irefs to dbm file on: ";
  # system ("date 1>&2");
  # while ( ($w, $iref) = each %word_irefmem) { # Write into a DBM file.
  #   if ($new_word_iref{$w} ne '') {
  #     print STDERR "irefmk.pl: Word $w already present in DBM. Adding.\n";
  #     $new_word_iref{$w} .= $iref;
  #   }
  #   else { $new_word_iref{$w} = $iref; }
  # }
  # print STDERR "Finished writing  word-irefs to dbm file on: ";
  # system ("date 1>&2");
}

sub rm_punct {
     # punctuation marks etc. are given below.
  #s/([\",:;'`\?><\]\[}{\)\(=!%&\*\.])/ \1 /g; # Not removing -,_,#,~,@,$,|,
  #s/([\",:;'`\?><\]\[}{\)\(=!@#%&\*\.])/ \1 /g; # Not removing '-', '_'.
  #s/[\",:;'`\?><\]\[}{\)\(-_=!@#%&\*\.\xea]//g;
    #s/\xea/ \1 /g;  # Remove 'KadI pAi' for Hindi and Punjabi etc.
    #s/lY/lV/g; Only for Kannada
  #s/<s[0-9]+>/ /g;
  #s/<.>/ /g;
}
1;
