#!/usr/bin/perl
# Loads into associative arrays the sizes of the files in the corpus,
# given as its input, a file (wc.lst) giving details regarding 
# size of the files in the corpus. 
#   Example usage: &ldsize ('/usr4/corpus/telugu8/wc.lst')
#
# These arrays can be used by other programs:
#    $cfile_line{$fno}	- Number of lines in file $fno.
#    $cfile_word{$fno}	- Number of words in file $fno.
#    $cfile_byte{$fno}	- Number of bytes in file $fno.
#    $cfile_fname{$fno}	- Name of file $fno.
#
# The wc.lst file contains summary regarding the corpus files. Here
# is a sample from wc.lst
# File                      File
# no.   Lines  Words  Bytes name
# 1     134    1689    9130 /usr4/corpus/hindi/flop1.txt/1
# 2     215    2673   12922 /usr4/corpus/hindi/flop1.txt/10
# 3     211    2429   11763 /usr4/corpus/hindi/flop1.txt/11
# T   @###### @####### @######## ##TOTAL-LINES-WORDS-BYTES

# For stand alone testing remove the comments from the next 2 lines.
#$corpuswc = '/usr4/corpus/telugu8/wc.lst';
#&ldsize ($corpuswc);

sub ldsize {
  local ($corpwc) = @_;
  local ($fno, $lines, $words, $bytes, $fname);
  open (WC, "$corpwc") ||
    die "Unable to open file $corpwc: $!\n";
  while (<WC>) {
    chop;
    ($fno, $lines, $words, $bytes, $fname) = split (/\s+/);
    $cfile_line{$fno} = $lines;
    $cfile_word{$fno} = $words;
    $cfile_byte{$fno} = $bytes;
    $cfile_fname{$fno} = $fname;
  }
  close (WC);
  # For debugging
  #for ($i=0;  $i <= $fno;  $i++) {
  #  print STDERR $cfile_word{$i}, '  ', $cfile_byte{$i}, ' ', $cfile_fname{$i}, "\n";
  #} 
}  
1;
