#!/usr/bin/perl
#Input : a file of frequency of occurrence f and phrase in each line
#          sorted from high to low f.
#Output : frequency of frequency of occurrence f (when f is less than 
#          or equal to given limit)
#  Example usage:  low_frq2.pl  < phrase.frq  > phrase.lcov
# (Same as low_frq.pl except allows phrases instead of only words.)
# Options:
# -l	-  Frequency of occurrence less than or equal to given limit is considered low. (Default: 20) 
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $Finname, $Foutname);
local ($low);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$low = 20; # Default  Frequency of occurrence less than or equal to given limit is considered low. (Default: 20) 

# Look up .low_frq2rc file. Override defaults.
if (open (RC, '.low_frq2rc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/low_frq2.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'l') {
        $low = substr ($_, 2);
      }
      else { 
          print STDERR "low_frq2.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/low_frq2.cmd');
          die;
      }
    }
    else { # 
      print STDERR "low_frq2.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
     # Command line args override those of the rc file.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/low_frq2.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1, 1) eq  'l') {
      $low = substr ($ARGV[$i], 2);
    }
    else { 
        print STDERR "low_frq2.cmd: Unrecognized param: $ARGV[$i].\n";
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/low_frq2.cmd');
        die;
    }
  }
  else { # Argument found. Parameters end.
    $argbegpos = $i;
    last;
  }
}

# Process the arguments in the command line then call low_frq2.
# 1. Can be used as a filter. OR
# 2. file args can be given instead of filters.

if ($argbegpos < 0) { # No args.
  &low_frq2(STDIN, STDOUT);
}
elsif (($#ARGV - $argbegpos) == 0) { # Only one arg given. Execute the command with arg and stdout.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "low_frq2.cmd: Unable to open file $ARGV[$argbegpos]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = '';
  &low_frq2(INFILE, STDOUT);
  close (INFILE);
}
elsif (($#ARGV - $argbegpos) == 1) { # Two args given. Execute the command with arg1 and arg2.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "low_frq2.cmd: Unable to open input file $ARGV[$argbegpos]\n" ;
  open (OUTFILE, ">$ARGV[$argbegpos + 1]") ||
     die "low_frq2.cmd: Unable to open output file $ARGV[$argbegpos + 1]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = $ARGV[$argbegpos+1];  
  &low_frq2(INFILE, OUTFILE);
  close (OUTFILE);
  close (INFILE);
}
else { print STDERR "low_frq2.cmd: Too many arguments.\n"; }
#------------------------------------------------------

sub low_frq2 {
local ($W_fin, $W_fout) = @_;
select($W_fout);
#%COMMENT
#Input : a file of frequency of occurrence f and phrase in each line
#          sorted from high to low f.
#Output : frequency of frequency of occurrence f (when f is less than 
#          or equal to given limit)
#  Example usage:  low_frq2.pl  < phrase.frq  > phrase.lcov
# (Same as low_frq.pl except allows phrases instead of only words.)
#%CMD TYPE 11

$tot_cnt=0;  # Total count of all the words (corpus size).
$tot_dist_cnt=0;  # Number of distinct words.
$cur_cnt=0;  # Count of words with the current frequency (<= $low)
$prev_frq = $low + 1;  # Set it some value higher than $low.
while(<$W_fin>) {
  chop;
  @wrd_lst = split(/\s+/, $_, 2);
  ($frq, $word) = @wrd_lst;
  $tot_cnt += $frq;
  $tot_dist_cnt++;
  if ($frq > $low) { $cur_cnt++; }
  else {  # A low frequency found.
    if ($prev_frq == $frq) {
      $cur_cnt++;
    }
    else { # A new freq found.
      if ($prev_frq  > $low) { # first time a low freq found
        print $cur_cnt, " >$low\n";
        $prev_frq = $frq;  $cur_cnt = 1;
      }
      else {
        print $cur_cnt, " $prev_frq\n";
        $prev_frq = $frq;  $cur_cnt = 1;
      }
    }
  }
} 
# Now empty the last entry.
print $cur_cnt, " $frq\n";
print "$tot_dist_cnt Distinct_words ($tot_cnt Corpus_size)\n";
  #print "$tot_cnt $tot_dist_cnt   Total words and distinct words\n";
}
1;
