#!/usr/bin/perl
# Makes directory with abbreviated names for the corpus file.
# It will use numbers as file names, same as that used in word index.
# Name of the new directory is taken from corptab under abbreviated dir.
#   Example usage: mkabbr.cmd  -ltelugu -tnormal
# Will create a directory with abbreviated names for telugu original
# corpus. Gets details about the corpora from ~corpus/corptab file.
# Options:
# -l	-  Name of the language for whose corpus an abbreviated directory is to be created.
# -t	-  Type of corpus: original, normal, cleaned etc.

require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/mkabbr.pl");


# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $arg, $outf, $argsinfile, $Finname, $Foutname);
local ($lang, $orig);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$argsinfile = 0;  # Default args file names are not in a file.
# $lang -  Name of the language for whose corpus an abbreviated directory is to be created.
$orig = 'normal'; # Default  Type of corpus: original, normal, cleaned etc.

# Look up .mkabbrrc file. Override defaults.
if (open (RC, '.mkabbrrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/mkabbr.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'l') {
        $lang = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  't') {
        $orig = substr ($_, 2);
      }
      else { 
          print STDERR "mkabbr.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/mkabbr.cmd');
          die;
      }
    }
    else { # 
      print STDERR "mkabbr.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/mkabbr.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1,1) eq 'o') { # Specifies output file.
      $outfilename = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'l') {
      $lang = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  't') {
      $orig = substr ($ARGV[$i], 2);
    }
    else { print STDERR "mkabbr.cmd: Unrecognized param: $ARGV[$i].\n";
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/mkabbr.cmd');
      die;
    }
  }
  else { # Argument found. Parameters end.
    if (substr($ARGV[$i],0,1) eq '@') {  # A file of args found
      $argsinfile = 1;
      $ARGV[$i] = substr($ARGV[$i], 1);
    }
    $argbegpos = $i;
    last;
  }
}

#print STDERR "mkabbr.cmd: DEB params end: $parmlastpos;  args begin: $argbegpos; total: $#ARGV;\n";

# Process the arguments in the command line then call mkabbr.
# 1. Can be used as a filter. OR
# 2. If arg file names are given, they are treated as input files
#    and the output goes to a single file stdout.

if ($outfilename ne '') {
  open (OUT, ">$outfilename") ||
    die "mkabbr.cmd: Unable to open output file $outfilename.\n";
  $outf = OUT;
  $Foutname = $outfilename;
}
else { $outf = STDOUT; }

if ($argbegpos < 0) { # No args.
  &mkabbr(STDIN, $outf);
}
elsif ($argsinfile) {  # Argument filenames in file.
  open (ARGFILE, "$ARGV[$argbegpos]") ||
    die "mkabbr.cmd: Unable to open args file $ARGV[$argbegpos]\n" ;
  while (<ARGFILE>) { # Read args.
    chop;
    if (!(open (INFILE, $_))) {
      print STDERR "mkabbr.cmd: Unable to open file $_\n" ;
      next;
    }
    $Finname = $_;
    &mkabbr (INFILE, $outf);
    close (INFILE);
  }
  close (ARGFILE);
}
else { # Call the program with each arg. 
  for ($arg = $argbegpos; $arg <= $#ARGV; $arg++) {
    if (!(open (INFILE, "$ARGV[$arg]"))) {
      print STDERR "mkabbr.cmd: Unable to open file $ARGV[$arg]. Continuing.\n" ;
      next;
    }
    $Finname = $ARGV[$arg];
    &mkabbr (INFILE, $outf);
    close (INFILE);
  }
}
close ($outf);
 
