#!/usr/bin/perl
# Makes shell files for computing ngrams of the given
# input file containing names of files (usually subject wise).
#   Example usage:  ngram_mk_files.sh -s5 < ASTRO.NAM   > ASTRO.ng3
# The above generates sequences of words of length 5.
# Options:
# -d	-  Directory in which files reside (default .). 
# -s	-  Size of ngram (default 3). 

require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $arg, $outf, $argsinfile, $Finname, $Foutname);
local ($corp_dir, $ngram_size);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$argsinfile = 0;  # Default args file names are not in a file.
$corp_dir = '.'; # Default  Directory in which files reside (default .). 
$ngram_size = 3; # Default  Size of ngram (default 3). 

# Look up .ngram_mk_filesrc file. Override defaults.
if (open (RC, '.ngram_mk_filesrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/ngram_mk_files.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'd') {
        $corp_dir = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  's') {
        $ngram_size = substr ($_, 2);
      }
      else { 
          print STDERR "ngram_mk_files.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/ngram_mk_files.cmd');
          die;
      }
    }
    else { # 
      print STDERR "ngram_mk_files.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/ngram_mk_files.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1,1) eq 'o') { # Specifies output file.
      $outfilename = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'd') {
      $corp_dir = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  's') {
      $ngram_size = substr ($ARGV[$i], 2);
    }
    else { print STDERR "ngram_mk_files.cmd: Unrecognized param: $ARGV[$i].\n";
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/ngram_mk_files.cmd');
      die;
    }
  }
  else { # Argument found. Parameters end.
    if (substr($ARGV[$i],0,1) eq '@') {  # A file of args found
      $argsinfile = 1;
      $ARGV[$i] = substr($ARGV[$i], 1);
    }
    $argbegpos = $i;
    last;
  }
}

#print STDERR "ngram_mk_files.cmd: DEB params end: $parmlastpos;  args begin: $argbegpos; total: $#ARGV;\n";

# Process the arguments in the command line then call ngram_mk_files.
# 1. Can be used as a filter. OR
# 2. If arg file names are given, they are treated as input files
#    and the output goes to a single file stdout.

if ($outfilename ne '') {
  open (OUT, ">$outfilename") ||
    die "ngram_mk_files.cmd: Unable to open output file $outfilename.\n";
  $outf = OUT;
  $Foutname = $outfilename;
}
else { $outf = STDOUT; }

if ($argbegpos < 0) { # No args.
  &ngram_mk_files(STDIN, $outf);
}
elsif ($argsinfile) {  # Argument filenames in file.
  open (ARGFILE, "$ARGV[$argbegpos]") ||
    die "ngram_mk_files.cmd: Unable to open args file $ARGV[$argbegpos]\n" ;
  while (<ARGFILE>) { # Read args.
    chop;
    if (!(open (INFILE, $_))) {
      print STDERR "ngram_mk_files.cmd: Unable to open file $_\n" ;
      next;
    }
    $Finname = $_;
    &ngram_mk_files (INFILE, $outf);
    close (INFILE);
  }
  close (ARGFILE);
}
else { # Call the program with each arg. 
  for ($arg = $argbegpos; $arg <= $#ARGV; $arg++) {
    if (!(open (INFILE, "$ARGV[$arg]"))) {
      print STDERR "ngram_mk_files.cmd: Unable to open file $ARGV[$arg]. Continuing.\n" ;
      next;
    }
    $Finname = $ARGV[$arg];
    &ngram_mk_files (INFILE, $outf);
    close (INFILE);
  }
}
close ($outf);
 
#------------------------------------------------------

sub ngram_mk_files {
local ($W_fin, $W_fout) = @_;
select($W_fout);
#%COMMENT
# Makes shell files for computing ngrams of the given
# input file containing names of files (usually subject wise).
#   Example usage:  ngram_mk_files.sh -s5 < ASTRO.NAM   > ASTRO.ng3
# The above generates sequences of words of length 5.
#%CMD TYPE M1


while (<$W_fin>) {
  chop;
  if (/^$/) { next; }
  if (/^\s*#/) { next; }
  if (/ /) { $filename = $corp_dir . '/' . $`; } 
  else     { $filename = $corp_dir . '/' . $_; }
  system ("/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/ngram_seq_mk.sh $ngram_size < $filename > ngr_seq.tmp;");
}
system ("/home/bharathi/PNJ1/usr3/sangal/tools/src/corp/frq_cnt.sh < ngr_seq.tmp;");
}
1;
