#!/usr/bin/perl
# Takes a file containing text and makes a file of all the
# word sequences of length s (default 3).
# Assumes that punctuation marks have been separated in the input file,
# and the end of sentence markers have been inserted.
#   Example usage: ngram_seq_mk.pl  -s5  < TEXTFILE  > FRQFILE
# In the above, word sequences of length 5 or less are generated.
# Options:
# -s	-  Length of word sequences which have to be retrieved. 

require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $arg, $outf, $argsinfile, $Finname, $Foutname);
local ($seq_len);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$argsinfile = 0;  # Default args file names are not in a file.
$seq_len = 3; # Default  Length of word sequences which have to be retrieved. 

# Look up .ngram_seq_mkrc file. Override defaults.
if (open (RC, '.ngram_seq_mkrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/ngram_seq_mk.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  's') {
        $seq_len = substr ($_, 2);
      }
      else { 
          print STDERR "ngram_seq_mk.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/ngram_seq_mk.cmd');
          die;
      }
    }
    else { # 
      print STDERR "ngram_seq_mk.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/ngram_seq_mk.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1,1) eq 'o') { # Specifies output file.
      $outfilename = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  's') {
      $seq_len = substr ($ARGV[$i], 2);
    }
    else { print STDERR "ngram_seq_mk.cmd: Unrecognized param: $ARGV[$i].\n";
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/ngram_seq_mk.cmd');
      die;
    }
  }
  else { # Argument found. Parameters end.
    if (substr($ARGV[$i],0,1) eq '@') {  # A file of args found
      $argsinfile = 1;
      $ARGV[$i] = substr($ARGV[$i], 1);
    }
    $argbegpos = $i;
    last;
  }
}

#print STDERR "ngram_seq_mk.cmd: DEB params end: $parmlastpos;  args begin: $argbegpos; total: $#ARGV;\n";

# Process the arguments in the command line then call ngram_seq_mk.
# 1. Can be used as a filter. OR
# 2. If arg file names are given, they are treated as input files
#    and the output goes to a single file stdout.

if ($outfilename ne '') {
  open (OUT, ">$outfilename") ||
    die "ngram_seq_mk.cmd: Unable to open output file $outfilename.\n";
  $outf = OUT;
  $Foutname = $outfilename;
}
else { $outf = STDOUT; }

if ($argbegpos < 0) { # No args.
  &ngram_seq_mk(STDIN, $outf);
}
elsif ($argsinfile) {  # Argument filenames in file.
  open (ARGFILE, "$ARGV[$argbegpos]") ||
    die "ngram_seq_mk.cmd: Unable to open args file $ARGV[$argbegpos]\n" ;
  while (<ARGFILE>) { # Read args.
    chop;
    if (!(open (INFILE, $_))) {
      print STDERR "ngram_seq_mk.cmd: Unable to open file $_\n" ;
      next;
    }
    $Finname = $_;
    &ngram_seq_mk (INFILE, $outf);
    close (INFILE);
  }
  close (ARGFILE);
}
else { # Call the program with each arg. 
  for ($arg = $argbegpos; $arg <= $#ARGV; $arg++) {
    if (!(open (INFILE, "$ARGV[$arg]"))) {
      print STDERR "ngram_seq_mk.cmd: Unable to open file $ARGV[$arg]. Continuing.\n" ;
      next;
    }
    $Finname = $ARGV[$arg];
    &ngram_seq_mk (INFILE, $outf);
    close (INFILE);
  }
}
close ($outf);
 
#------------------------------------------------------

sub ngram_seq_mk {
local ($W_fin, $W_fout) = @_;
select($W_fout);
#%COMMENT
# Takes a file containing text and makes a file of all the
# word sequences of length s (default 3).
# Assumes that punctuation marks have been separated in the input file,
# and the end of sentence markers have been inserted.
#   Example usage: ngram_seq_mk.pl  -s5  < TEXTFILE  > FRQFILE
# In the above, word sequences of length 5 or less are generated.
#%CMD TYPE M1


$/ = '<.>';
while (<$W_fin>) {
  chop; chop; chop;
  if (/^\s+/) { $_ = $'; }
  if (/\s+$/) { $_ = $`; }
  @words = split (/[\",:;\'`\?><\]\[}{\)\(=!@#%&\*\.\xea\s]+/);
     #NOT USED:-> '[",:;\'`\?\]\[}{\)\(=!@#%&\*\s]';

  #for ($i=0;  $i <= $#words;  $i++)  { #Sequences of length 1
  #  #print "DEB: word $i is  = '$words[$i]'\n";
  #  print $words[$i], "\n";
  #}
  #for ($i=0;  $i <= $#words - 1;  $i++)  { #Sequences of length 2
  #  for ($j=0;  $j <= $seq_len - 3;  $j++)  {
  #    print $words[$i+$j], ' ';
  #  }
  #  print $words[$i + $seq_len - 2],  "\n";
  #}
  for ($i=0;  $i <= ($#words - ($seq_len - 1));  $i++)  { #Sequences of length 3
    #print "DEB: word $i is  = '$words[$i]'\n";
    for ($j=0;  $j <= $seq_len - 2;  $j++)  {
      print $words[$i+$j], ' ';
    }
    print $words[$i + $seq_len - 1],  "\n";
  }
}
}
1;
