#!/usr/bin/perl
# Takes a file of Hindi headers, and puts classification numbers.
#   Example usage: hdr2classi.pl Classi.lst < HEADERS.hlp   > Classi.hlp
# The input file has file number followed by the header.

# It uses two global associative arrays keeping the mapping between
# classification numbers and categories:
#  %classi_cat,   and   %cat_classi

# A sample HEADERS.hlp file:
# 1 <Official And Media Languages> <Mass Media> <Mishra, Vijay Kumar> <Sanjay Gandhi Jid Aur Jokhim Se Bhari Zindagi> <1980> <Jyo-Yugeen Sahitya Chetna> <Delhi><57-59> <1500> <Mishra - Sanjay Gandhi Jid Aur Jokhim> <H. S. Wadhawan> 
# 2 <Translated Material><Literature><Das, Jogesh><Prithvi ki peeda><1988><Sahitya Akademi><New Delhi><84-89><2100><Das, J-Prithvi Ki peeda-Lt-O><Abha Sood>
# 3 <Official And Media Languages><Mass Media><Anorymous><Haryana : Pehli Pari Devilal Ki><1986><DIN - Times Of India><New Delhi><15-17><2040><Anon - Haryana : Pehli Pari Devilal Ki><H.S.Wadhawan>

require ('/usr4/corpus/sbin/ldcatclassi.pl');

local ($fileno, @words, $majcat, $classif);
$filename = $ARGV[0];
open (FCAT, "$filename") ||
  die "hdr2classi.pl: Unable to open categ file $filename\n";
&ldcatclassi (FCAT);
close FCAT;
while (<STDIN>) {
  $fullline = $_;
  if (/^([0-9]+)\s*</) { $_ = $';  $fileno = $1; }  
                    # ^^ Remove file no. from the leading part of the line.
  if (/>\s*$/) { $_ = $`; } # Remove the trailing part of the line.
  @words = split (/>\s*</);
     #print STDERR "hdr2classi.pl DEBUG: Headers has words: @words\n";
  if ($#words != 10) {
    print STDERR "hdr2classi.pl: Invalid header in line $.: $_";
  }
  ($majcat, $mincat, $author, $title, $year, $publ,
     $place, $pages, $numwords, $cright, $dateentry) = @words;
  $majcat =~ tr/A-Z/a-z/;
  $mincat =~ tr/A-Z/a-z/;
  $classif = $cat_classi{$mincat};
  if ($classif eq '') { $classif = $cat_classi{$majcat}; }
  if ($classif eq '') { 
    print STDERR "hdr2classi.pl: Classification number not found:$_";
  }
  print $classif, $fullline;
}
    
