#!/usr/bin/perl
# Takes as its input, a file containing classified nos. and subjects.
# Looks up a file for each subject in the current directory and 
# determines which corpus files pertain to that subject.
# For each of the corpus files detemines its size, and
# writes in stdout a summary of sizes for each subject.
#   Example usage:   cd  /usr4/corpus/telugu8/Subjects
#               subj_summ.cmd  -f../wc.lst  < Subj.comp  > Subj.sum
# Example Subj.comp:
#      I    Aesthetics
#      I.a  Literature
#      I.a.6   Children
#      I.a.7   Biographies/Autobiographies
#      I.a.9   Letters/Diaries/Speeches
# In the above, for each of the subjects listed such as:
# aesthetics, children, biographies, autobiographies an entry
# will be created.
# Options:
# -f	-  Name of file containing word count etc. (output of wc) for all corpus files. 

require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $arg, $outf, $argsinfile, $Finname, $Foutname);
local ($wcfile);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$argsinfile = 0;  # Default args file names are not in a file.
$wcfile = '../wc.lst'; # Default  Name of file containing word count etc. (output of wc) for all corpus files. 

# Look up .subj_summrc file. Override defaults.
if (open (RC, '.subj_summrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/subj_summ.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'f') {
        $wcfile = substr ($_, 2);
      }
      else { 
          print STDERR "subj_summ.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/subj_summ.cmd');
          die;
      }
    }
    else { # 
      print STDERR "subj_summ.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/subj_summ.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1,1) eq 'o') { # Specifies output file.
      $outfilename = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'f') {
      $wcfile = substr ($ARGV[$i], 2);
    }
    else { print STDERR "subj_summ.cmd: Unrecognized param: $ARGV[$i].\n";
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/subj_summ.cmd');
      die;
    }
  }
  else { # Argument found. Parameters end.
    if (substr($ARGV[$i],0,1) eq '@') {  # A file of args found
      $argsinfile = 1;
      $ARGV[$i] = substr($ARGV[$i], 1);
    }
    $argbegpos = $i;
    last;
  }
}

#print STDERR "subj_summ.cmd: DEB params end: $parmlastpos;  args begin: $argbegpos; total: $#ARGV;\n";

# Process the arguments in the command line then call subj_summ.
# 1. Can be used as a filter. OR
# 2. If arg file names are given, they are treated as input files
#    and the output goes to a single file stdout.

if ($outfilename ne '') {
  open (OUT, ">$outfilename") ||
    die "subj_summ.cmd: Unable to open output file $outfilename.\n";
  $outf = OUT;
  $Foutname = $outfilename;
}
else { $outf = STDOUT; }

if ($argbegpos < 0) { # No args.
  &subj_summ(STDIN, $outf);
}
elsif ($argsinfile) {  # Argument filenames in file.
  open (ARGFILE, "$ARGV[$argbegpos]") ||
    die "subj_summ.cmd: Unable to open args file $ARGV[$argbegpos]\n" ;
  while (<ARGFILE>) { # Read args.
    chop;
    if (!(open (INFILE, $_))) {
      print STDERR "subj_summ.cmd: Unable to open file $_\n" ;
      next;
    }
    $Finname = $_;
    &subj_summ (INFILE, $outf);
    close (INFILE);
  }
  close (ARGFILE);
}
else { # Call the program with each arg. 
  for ($arg = $argbegpos; $arg <= $#ARGV; $arg++) {
    if (!(open (INFILE, "$ARGV[$arg]"))) {
      print STDERR "subj_summ.cmd: Unable to open file $ARGV[$arg]. Continuing.\n" ;
      next;
    }
    $Finname = $ARGV[$arg];
    &subj_summ (INFILE, $outf);
    close (INFILE);
  }
}
close ($outf);
 
#------------------------------------------------------

sub subj_summ {
local ($W_fin, $W_fout) = @_;
select($W_fout);
#%COMMENT
# Takes as its input, a file containing classified nos. and subjects.
# Looks up a file for each subject in the current directory and 
# determines which corpus files pertain to that subject.
# For each of the corpus files detemines its size, and
# writes in stdout a summary of sizes for each subject.
#   Example usage:   cd  /usr4/corpus/telugu8/Subjects
#               subj_summ.cmd  -f../wc.lst  < Subj.comp  > Subj.sum
# Example Subj.comp:
#      I    Aesthetics
#      I.a  Literature
#      I.a.6   Children
#      I.a.7   Biographies/Autobiographies
#      I.a.9   Letters/Diaries/Speeches
# In the above, for each of the subjects listed such as:
# aesthetics, children, biographies, autobiographies an entry
# will be created.
#%CMD TYPE M1


# Format for output summary
format SUMMFORMAT =
@<<<<<<< @>>>>>>>>>>>>>> (@#%) @######## @####### @###### @####
$cno,    $f,             $percent_word, $sum_byte, $sum_word, $sum_line, $sum_file
.
$~ = "SUMMFORMAT";   # Format to be used with 'write'

# Format to be used if total words not available. No percent value.
format SUMM2FORMAT =
@<<<<<<< @>>>>>>>>>>>>>> (  %) @######## @####### @###### @####
$cno,    $f,                   $sum_byte, $sum_word, $sum_line, $sum_file
.

require ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/ldsize.pl');
&ldsize($wcfile);
$tot_word = $cfile_word {'T'};# Default total words in corpus.
if ($tot_word == 0) { print STDERR 
   "subj_summ.pl: Unable to get total words in corpus from wc.lst. Continuing.\n";
   $~ = "SUMM2FORMAT";   # Format to be used with 'write'
}

system ('echo "#Classification Subject  word%     Bytes    Words   Lines Files"');
while (<$W_fin>) {
  if (/^\s*$/) { next; }  # Ignore blank line
  if (/^\s*#/) { next; }  # Ignore comment line
  chop;
  ($cno, $subjs) = split (/\s+/, $_, 2);
  if ($subjs eq '') {   # Only cno is given, ignore line.
    print STDERR "Invalid format. Ignoring line: $_.\n";
    next;
  }
  if ($subjs =~ /\//) { @fields = split (/\//, $subjs); }
  else { $fields[0] = $subjs; $#fields = 0; }
  foreach $f (@fields) {
    $filename = $f;
    $filename =~ tr/A-Z/a-z/;
    $filename =~ s/[^a-z0-9_\-]//g;  # Delete all non alpha-num-dash
    #print STDERR "DEBUG: Subj field=$f, Filename=$filename\n";
    $sum_file = $sum_line = $sum_word = $sum_byte = 0;
    if (-e "$filename.nam") { # Check if file exists.
      if (!(open (INFILE, "$filename.nam") )) {
        print STDERR "Unable to open file $filename: $!\n";
      }
      else {
        while (<INFILE>) { # Read file no. in each line.
          if (/\s/) { $corp_fno = $`; }
          else { print STDERR "Invalid line in $filename.nam: $_"; }
          $sum_line += $cfile_line{$corp_fno};
          $sum_word += $cfile_word{$corp_fno};
          $sum_byte += $cfile_byte{$corp_fno};
          $sum_file++;
        }
      }
    }
    if ($tot_word != 0) {
      $percent_word = (100 * $sum_word / $tot_word) + .5; # Round off.
    }
    else { $percent_word = ' '};
    write;
  }
}
}
1;
