#!/usr/bin/perl
# Takes as its input, a file containing classification nos. and 
# subjects. Looks up the file Headers.lst (which contains file numbers
# and their header which contain subject), and constructs separate files
# for each of the subjects. The new files are named after
# the subjects.
# Sends the pair (subject, file name) to stdout for each subject.
#   Example usage:  subj_cr.cmd  -fHEADERSFILE  < Subj.comp  > Subj_files.lst
# In the example above, each line of file Subj_files.lst has pair (filename, subj).
# Example Subj.comp:
#      I    Aesthetics
#      I.a  Literature
#      I.a.6   Children
#      I.a.7   Biographies/Autobiographies
#      I.a.9   Letters/Diaries/Speeches
# In the above, for each of the subjects listed such as:
# aesthetics, children, biographies, autobiographies a file
# will be created.
# Options:
# -f	-  Name of file containing headers for corpus files. 

require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $arg, $outf, $argsinfile, $Finname, $Foutname);
local ($headersfile);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$argsinfile = 0;  # Default args file names are not in a file.
$headersfile = 'Headers.lst'; # Default  Name of file containing headers for corpus files. 

# Look up .subj_crrc file. Override defaults.
if (open (RC, '.subj_crrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/subj_cr.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'f') {
        $headersfile = substr ($_, 2);
      }
      else { 
          print STDERR "subj_cr.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/subj_cr.cmd');
          die;
      }
    }
    else { # 
      print STDERR "subj_cr.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/subj_cr.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1,1) eq 'o') { # Specifies output file.
      $outfilename = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'f') {
      $headersfile = substr ($ARGV[$i], 2);
    }
    else { print STDERR "subj_cr.cmd: Unrecognized param: $ARGV[$i].\n";
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/subj_cr.cmd');
      die;
    }
  }
  else { # Argument found. Parameters end.
    if (substr($ARGV[$i],0,1) eq '@') {  # A file of args found
      $argsinfile = 1;
      $ARGV[$i] = substr($ARGV[$i], 1);
    }
    $argbegpos = $i;
    last;
  }
}

#print STDERR "subj_cr.cmd: DEB params end: $parmlastpos;  args begin: $argbegpos; total: $#ARGV;\n";

# Process the arguments in the command line then call subj_cr.
# 1. Can be used as a filter. OR
# 2. If arg file names are given, they are treated as input files
#    and the output goes to a single file stdout.

if ($outfilename ne '') {
  open (OUT, ">$outfilename") ||
    die "subj_cr.cmd: Unable to open output file $outfilename.\n";
  $outf = OUT;
  $Foutname = $outfilename;
}
else { $outf = STDOUT; }

if ($argbegpos < 0) { # No args.
  &subj_cr(STDIN, $outf);
}
elsif ($argsinfile) {  # Argument filenames in file.
  open (ARGFILE, "$ARGV[$argbegpos]") ||
    die "subj_cr.cmd: Unable to open args file $ARGV[$argbegpos]\n" ;
  while (<ARGFILE>) { # Read args.
    chop;
    if (!(open (INFILE, $_))) {
      print STDERR "subj_cr.cmd: Unable to open file $_\n" ;
      next;
    }
    $Finname = $_;
    &subj_cr (INFILE, $outf);
    close (INFILE);
  }
  close (ARGFILE);
}
else { # Call the program with each arg. 
  for ($arg = $argbegpos; $arg <= $#ARGV; $arg++) {
    if (!(open (INFILE, "$ARGV[$arg]"))) {
      print STDERR "subj_cr.cmd: Unable to open file $ARGV[$arg]. Continuing.\n" ;
      next;
    }
    $Finname = $ARGV[$arg];
    &subj_cr (INFILE, $outf);
    close (INFILE);
  }
}
close ($outf);
 
#------------------------------------------------------

sub subj_cr {
local ($W_fin, $W_fout) = @_;
select($W_fout);
#%COMMENT
# Takes as its input, a file containing classification nos. and 
# subjects. Looks up the file Headers.lst (which contains file numbers
# and their header which contain subject), and constructs separate files
# for each of the subjects. The new files are named after
# the subjects.
# Sends the pair (subject, file name) to stdout for each subject.
#   Example usage:  subj_cr.cmd  -fHEADERSFILE  < Subj.comp  > Subj_files.lst
# In the example above, each line of file Subj_files.lst has pair (filename, subj).
# Example Subj.comp:
#      I    Aesthetics
#      I.a  Literature
#      I.a.6   Children
#      I.a.7   Biographies/Autobiographies
#      I.a.9   Letters/Diaries/Speeches
# In the above, for each of the subjects listed such as:
# aesthetics, children, biographies, autobiographies a file
# will be created.
#%CMD TYPE M1

while (<$W_fin>) {
  if (/^\s*$/) { next; }  # Ignore blank line
  if (/^\s*#/) { next; }  # Ignore comment line
  chop;
  ($cno, $subjs) = split (/\s+/, $_, 2);
  if ($subjs eq '') {   # Only cno is given, ignore line.
    print STDERR "Invalid format. Ignoring line: $_.\n";
    next;
  }
  if ($subjs =~ /\s*\/\s*/) { @fields = split (/\s*\/\s*/, $subjs); }
  else { $fields[0] = $subjs;  $#fields = 0; } # Only 1 subj field
  foreach $f (@fields) {
    $filename = $f;
    $filename =~ tr/A-Z/a-z/;
    $filename =~ s/[^a-z0-9_\-]//g;  # Delete all non alpha-num-dash
    #print STDERR "DEBUG: Field=$f, Filename=$filename\n";
    $res = system ("grep -i '$f' $headersfile > $filename.nam");
    if ($res == 0) {  # Print only if grep yielded a value.
      print $filename, '.nam   ', $f, "\n";  # Write filename and field
    }
    else { system ("rm $filename.nam"); }  # Delete empty file.
  }
}
}
1;
