#!/usr/bin/perl
#%COMMENT
# Takes as its input, a file containing classification nos. and 
# subjects. Looks up the file Headers.lst (which contains file numbers
# and their header which contain subject), and constructs separate files
# for each of the subjects. The new files are named after
# the subjects.
# Sends the pair (subject, file name) to stdout for each subject.
#   Example usage:  subj_cr.cmd  -fHEADERSFILE  < Subj.comp  > Subj_files.lst
# In the example above, each line of file Subj_files.lst has pair (filename, subj).
# Example Subj.comp:
#      I    Aesthetics
#      I.a  Literature
#      I.a.6   Children
#      I.a.7   Biographies/Autobiographies
#      I.a.9   Letters/Diaries/Speeches
# In the above, for each of the subjects listed such as:
# aesthetics, children, biographies, autobiographies a file
# will be created.
#%CMD TYPE M1

$headersfile = 'Headers.lst';  # Name of file containing headers for corpus files. #OPT -f
while (<>) {
  if (/^\s*$/) { next; }  # Ignore blank line
  if (/^\s*#/) { next; }  # Ignore comment line
  chop;
  ($cno, $subjs) = split (/\s+/, $_, 2);
  if ($subjs eq '') {   # Only cno is given, ignore line.
    print STDERR "Invalid format. Ignoring line: $_.\n";
    next;
  }
  if ($subjs =~ /\s*\/\s*/) { @fields = split (/\s*\/\s*/, $subjs); }
  else { $fields[0] = $subjs;  $#fields = 0; } # Only 1 subj field
  foreach $f (@fields) {
    $filename = $f;
    $filename =~ tr/A-Z/a-z/;
    $filename =~ s/[^a-z0-9_\-]//g;  # Delete all non alpha-num-dash
    #print STDERR "DEBUG: Field=$f, Filename=$filename\n";
    $res = system ("grep -i '$f' $headersfile > $filename.nam");
    if ($res == 0) {  # Print only if grep yielded a value.
      print $filename, '.nam   ', $f, "\n";  # Write filename and field
    }
    else { system ("rm $filename.nam"); }  # Delete empty file.
  }
}
