#!/usr/bin/perl
#%COMMENT
# Takes as its input, a file containing classification nos. and subjects.
# Looks up a file for each subject in the current directory and 
# determines which corpus files pertain to that subject.
# For each of the corpus files detemines its size, and
# writes in stdout a summary of sizes for each subject.
#   Example usage:   cd  /usr4/corpus/telugu8/Subjects
#               subj_summ.cmd  -f../wc.lst  < Subj.comp  > Subj.sum
# Example Subj.comp:
#      I    Aesthetics
#      I.a  Literature
#      I.a.6   Children
#      I.a.7   Biographies/Autobiographies
#      I.a.9   Letters/Diaries/Speeches
# In the above, for each of the subjects listed such as:
# aesthetics, children, biographies, autobiographies an entry
# will be created.
#%CMD TYPE M1


# Format for output summary
format SUMMFORMAT =
@<<<<<<< @>>>>>>>>>>>>>> (@#%) @######## @####### @###### @####
$cno,    $f,             $percent_word, $sum_byte, $sum_word, $sum_line, $sum_file
.
$~ = "SUMMFORMAT";   # Format to be used with 'write'

# Format to be used if total words not available. No percent value.
format SUMM2FORMAT =
@<<<<<<< @>>>>>>>>>>>>>> (  %) @######## @####### @###### @####
$cno,    $f,                   $sum_byte, $sum_word, $sum_line, $sum_file
.

$wcfile = '../wc.lst';  # Name of file containing word count etc. (output of wc) for all corpus files. #OPT -f
require ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/ldsize.pl');
&ldsize($wcfile);
$tot_word = $cfile_word {'T'};# Default total words in corpus.
if ($tot_word == 0) { print STDERR "subj_summ.pl: Unable to get total words in corpus from wc.lst. Continuing.\n";
   $~ = "SUMM2FORMAT";   # Format to be used with 'write'
}

system ('echo "#Classification Subject  word%     Bytes    Words   Lines Files"');
while (<>) {
  if (/^\s*$/) { next; }  # Ignore blank line
  if (/^\s*#/) { next; }  # Ignore comment line
  chop;
  ($cno, $subjs) = split (/\s+/, $_, 2);
  if ($subjs eq '') {   # Only cno is given, ignore line.
    print STDERR "Invalid format. Ignoring line: $_.\n";
    next;
  }
  if ($subjs =~ /\//) { @fields = split (/\//, $subjs); }
  else { $fields[0] = $subjs; $#fields = 0; }
  foreach $f (@fields) {
    $filename = $f;
    $filename =~ tr/A-Z/a-z/;
    $filename =~ s/[^a-z0-9_\-]//g;  # Delete all non alpha-num-dash
    #print STDERR "DEBUG: Subj field=$f, Filename=$filename\n";
    $sum_file = $sum_line = $sum_word = $sum_byte = 0;
    if (-e "$filename.nam") { # Check if file exists.
      if (!(open (INFILE, "$filename.nam") )) {
        print STDERR "Unable to open file $filename: $!\n";
      }
      else {
        while (<INFILE>) { # Read file no. in each line.
          if (/\s/) { $corp_fno = $`; }
          else { print STDERR "Invalid line in $filename.nam: $_"; }
          $sum_line += $cfile_line{$corp_fno};
          $sum_word += $cfile_word{$corp_fno};
          $sum_byte += $cfile_byte{$corp_fno};
          $sum_file++;
        }
      }
    }
    if ($tot_word != 0) {
      $percent_word = (100 * $sum_word / $tot_word) + .5; # Round off.
    }
    else { $percent_word = ' '};
    write;
  }
}
