#!/usr/bin/perl
# Cleans a file of words where each word is in a new line. 
# (The word may be followed by other information after white space
#  in the new line, but that is not affected.)
# Cleaning involves removing punctuation marks etc from words.
# The cleaned word is followed by the original uncleaned word
# if a substring is removed (other than leading or trailing substrings).
#   Example usage: wicln.pl < word_raw.ndx   > word.ndx
#
# The punctuation marks etc are:
#   " , : ; ' ` ? > < ] [ } { ) ( = ! % & * .
# In particular, the following are NOT used as punctuation marks
# and are not included in the above list:
#   ~ - _ # @
#
# TO IMPLEMENT IN FUTURE: remove a line with only [0-9\-]+ in the word.

local ($word, $rest, $mid, $old_word);
while (<>) {
  $mid = 0;
  if (/\s+/) {  $word = $`; $rest = $';  $old_word = $word;  }
  else { # invalid format. Print the line anyway.
    print STDERR "wicln.pl: Word not found: $_";  print;  next;
  }
  $word =~ s/^[\",:;'`\?><\]\[}{\)\(=!%&\*\.]+//; # Remove leading punct marks.
  $word =~ s/[\",:;'`\?><\]\[}{\)\(=!%&\*\.]+$//g; # Remove trailing punct marks.
  if ($word eq '') { # Word had only punct marks.
    #print STDERR "wicln.pl DEBUG: Removing word consisting of punct marks only: $_";
    next;
  }
  if ($word =~ /[\",:;'`\?><\]\[}{\)\(=!%&\*\.]/) { # Found at least one in middle.
    $mid = 1;
    $word =~ s/[\",:;'`\?><\]\[}{\)\(=!%&\*\.]//g; # Remove remaining punct marks
  }
  if ($mid == 0)  { print $word, '  ', $rest; }
  else { print $word, ' <-- ', $old_word, '  ', $rest; }

  #s/^([\",;'`\?><\]\[}{\)\(=!@#%&\*\.\$])//; # Remove punct marks before the beginning of the word.
  #s/([\",;'`\?><\]\[}{\)\(=!@#%&\*\.\$])\s/\s/; # Remove punct marks after the end of the word.
  #if (/^-+/) {  $_ = $'; }  # Remove '-' if more than one in the beginning.
}

