#!/usr/bin/perl
# Takes an input file containing on each line: 
#  an input word, and its morph analysis (with root and features); 
# and inverts the file to produce:
#  root, words with that root
#   Example usage: word_mo.pl < IN  > MO.ndx

# Here is an example of an input file
#   unna    uMdu{v *ina* any }/
#   lexu    lexu{Avy }/lexu{n *avy_0* }/gala{v *a* 3_non_pu_e }/
#   mana    mana{n *avy_0* }/manu{v *an* any }/
#   nuMdi   nuMdi{Avy }/nuMdi{n *avy_0* }/
#   aMtAru  aMtu{v *A* 23_ba }/anu{v *wA* 23_ba }/
# The output is:
#   uMdu unna
#   lexu lexu
#   lexu lexu
#   gala lexu
#   mana mana
#   manu mana
#   nuMdi nuMdi
#   nuMdi nuMdi
#   aMtu aMtAru
#   anu aMtAru
# Options:

require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $arg, $outf, $argsinfile, $Finname, $Foutname);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$argsinfile = 0;  # Default args file names are not in a file.

# Look up .word_morc file. Override defaults.
if (open (RC, '.word_morc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/word_mo.cmd');
        die;
      }
      else { 
          print STDERR "word_mo.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/word_mo.cmd');
          die;
      }
    }
    else { # 
      print STDERR "word_mo.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/word_mo.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1,1) eq 'o') { # Specifies output file.
      $outfilename = substr ($ARGV[$i],2);
    }
    else { print STDERR "word_mo.cmd: Unrecognized param: $ARGV[$i].\n";
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/corpsetup/word_mo.cmd');
      die;
    }
  }
  else { # Argument found. Parameters end.
    if (substr($ARGV[$i],0,1) eq '@') {  # A file of args found
      $argsinfile = 1;
      $ARGV[$i] = substr($ARGV[$i], 1);
    }
    $argbegpos = $i;
    last;
  }
}

#print STDERR "word_mo.cmd: DEB params end: $parmlastpos;  args begin: $argbegpos; total: $#ARGV;\n";

# Process the arguments in the command line then call word_mo.
# 1. Can be used as a filter. OR
# 2. If arg file names are given, they are treated as input files
#    and the output goes to a single file stdout.

if ($outfilename ne '') {
  open (OUT, ">$outfilename") ||
    die "word_mo.cmd: Unable to open output file $outfilename.\n";
  $outf = OUT;
  $Foutname = $outfilename;
}
else { $outf = STDOUT; }

if ($argbegpos < 0) { # No args.
  &word_mo(STDIN, $outf);
}
elsif ($argsinfile) {  # Argument filenames in file.
  open (ARGFILE, "$ARGV[$argbegpos]") ||
    die "word_mo.cmd: Unable to open args file $ARGV[$argbegpos]\n" ;
  while (<ARGFILE>) { # Read args.
    chop;
    if (!(open (INFILE, $_))) {
      print STDERR "word_mo.cmd: Unable to open file $_\n" ;
      next;
    }
    $Finname = $_;
    &word_mo (INFILE, $outf);
    close (INFILE);
  }
  close (ARGFILE);
}
else { # Call the program with each arg. 
  for ($arg = $argbegpos; $arg <= $#ARGV; $arg++) {
    if (!(open (INFILE, "$ARGV[$arg]"))) {
      print STDERR "word_mo.cmd: Unable to open file $ARGV[$arg]. Continuing.\n" ;
      next;
    }
    $Finname = $ARGV[$arg];
    &word_mo (INFILE, $outf);
    close (INFILE);
  }
}
close ($outf);
 
#------------------------------------------------------

sub word_mo {
local ($W_fin, $W_fout) = @_;
select($W_fout);
#%COMMENT
# Takes an input file containing on each line: 
#  an input word, and its morph analysis (with root and features); 
# and inverts the file to produce:
#  root, words with that root
#   Example usage: word_mo.pl < IN  > MO.ndx

# Here is an example of an input file
#   unna    uMdu{v *ina* any }/
#   lexu    lexu{Avy }/lexu{n *avy_0* }/gala{v *a* 3_non_pu_e }/
#   mana    mana{n *avy_0* }/manu{v *an* any }/
#   nuMdi   nuMdi{Avy }/nuMdi{n *avy_0* }/
#   aMtAru  aMtu{v *A* 23_ba }/anu{v *wA* 23_ba }/
# The output is:
#   uMdu unna
#   lexu lexu
#   lexu lexu
#   gala lexu
#   mana mana
#   manu mana
#   nuMdi nuMdi
#   nuMdi nuMdi
#   aMtu aMtAru
#   anu aMtAru
#%CMD TYPE M1

while (<$W_fin>) {
  chop;
  if (/\s+/) {
    $word = $`;  $mo_output = $'; 
    @mo_out = split (/\//, $mo_output);
    if ($#mo_out < 0) { 
      print STDERR "DEB: No morph analysis for: $word\n";
    }
    else {
       #for ($i=0;  $i <= $#mo_out; $i++) {
       #  if ($mo_out[$i] =~ /{/) { $mo_out[$i] = $`; }
       #} 
      foreach $mo (@mo_out) {
        if ($mo =~ /{/) { print $`, ' ', $word, "\n"; }
      }
    }
  }
  else { print STDERR "Unrecognized format: $_\n"; }
}
}
1;
