#!/usr/bin/perl
# Breaks each line into lines of specified length 
# (default 60). Break is put after a space.

# Input variables set by the calling routine:
#  $lim		- Number of coloumns after which break to be put.
#  $brkmark	- Char string to be put at the break point
#  $forcesw	- true means break the line even if white space not fnd.
sub breakcol {
  local ($in, $out) = @_;
  local ($oldfile, $limplus);
  $oldfile = select ($out);
  $limplus = 0.12 *  $lim;
  while (<$in>) {
    while ($_ ne "") {  # Has the current line been processed?
      if (length($_) < ($lim + $limplus)) { print; $_ = ""; }
      else {
         $slim = substr ($_, 0, $lim); # Select and print line upto $lim length.
         print $slim;
  
         $_ = substr ($_, $lim);       # Set to remaining line.
         if (/ /) {  # Find the next available space 
           if ( length($`) < (2 * $limplus) ) {  
             print "$`$&";
             $_ = $';
             # If there is a punctuation mark etc. include in this line.
             if (/^(\.|\,|!|\?|\+`|\)|\]|\}|\:) */) { # Punctuation mark etc.
               print "$&$brkmark\n";   $_ = $';
             }
             else { print "$brkmark\n"; }
           }
           else { # Break without waiting for a space.
             if ($forcesw) { # Force the break even if no white space.
               if ($brkmark ne '') { print $brkmark, "\n"; }
               else { print '~', "\n"; }
             }
           }
         }
         else  { 
           if ($forcesw) { # Force the break even if no white space.
             if ($brkmark ne '') { print $brkmark, "\n"; }
             else { print '~', "\n"; }
           }
           else { # Do not force break.
             print STDERR "No white space in line $.. Not splitting.\n";
             print; $_ = ""; 
           }
         }
      }
    }
  }
  select ($oldfile);
}
1; 
