#!/usr/bin/perl
# Takes a field name and a keyword and retrieves records
# from the database which satisfy the field name and keyword.
#   Example: db_find.cmd -fFLDNAMES -kKEYWRDS -nDBNAME > my.id
# The above retrieves record IDs which have the field names
# called FLDNAMES, keywords KEYWRDS in the database 
# which has the name DBNAME.
# Options:
# -f	-  List of field names which will be searched. 
# -k	-  List of keywords which will be searched in the fields. 
# -n	-  Database name using which file names are constructed (e.g. 'prvrb.db' name is constructed if given 'prvrb'). (Default: 'my') 
# -F	-  End of field marker. Default: '{'  
# -V	-  End marker for value of a field. Default: '}' 
# -R	-  End of record marker. Default: "@\n" 
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $Finname, $Foutname);
local ($fld_list, $keywords, $db_name, $fld_end, $val_end, $rec_end);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$fld_list = 'head source'; # Default  List of field names which will be searched. 
$keywords = ''; # Default  List of keywords which will be searched in the fields. 
$db_name = 'my'; # Default  Database name using which file names are constructed (e.g. 'prvrb.db' name is constructed if given 'prvrb'). (Default: 'my') 
$fld_end = '{'; # Default  End of field marker. Default: '{'  
$val_end = '}'; # Default  End marker for value of a field. Default: '}' 
$rec_end = "@\n"; # Default  End of record marker. Default: "@\n" 

# Look up .db_findrc file. Override defaults.
if (open (RC, '.db_findrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/db_find.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'f') {
        $fld_list = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  'k') {
        $keywords = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  'n') {
        $db_name = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  'F') {
        $fld_end = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  'V') {
        $val_end = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  'R') {
        $rec_end = substr ($_, 2);
      }
      else { 
          print STDERR "db_find.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/db_find.cmd');
          die;
      }
    }
    else { # 
      print STDERR "db_find.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
     # Command line args override those of the rc file.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/db_find.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1, 1) eq  'f') {
      $fld_list = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'k') {
      $keywords = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'n') {
      $db_name = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'F') {
      $fld_end = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'V') {
      $val_end = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'R') {
      $rec_end = substr ($ARGV[$i], 2);
    }
    else { 
        print STDERR "db_find.cmd: Unrecognized param: $ARGV[$i].\n";
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/db_find.cmd');
        die;
    }
  }
  else { # Argument found. Parameters end.
    $argbegpos = $i;
    last;
  }
}

# Process the arguments in the command line then call db_find.
# 1. Can be used as a filter. OR
# 2. file args can be given instead of filters.

if ($argbegpos < 0) { # No args.
  &db_find(STDIN, STDOUT);
}
elsif (($#ARGV - $argbegpos) == 0) { # Only one arg given. Execute the command with arg and stdout.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "db_find.cmd: Unable to open file $ARGV[$argbegpos]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = '';
  &db_find(INFILE, STDOUT);
  close (INFILE);
}
elsif (($#ARGV - $argbegpos) == 1) { # Two args given. Execute the command with arg1 and arg2.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "db_find.cmd: Unable to open input file $ARGV[$argbegpos]\n" ;
  open (OUTFILE, ">$ARGV[$argbegpos + 1]") ||
     die "db_find.cmd: Unable to open output file $ARGV[$argbegpos + 1]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = $ARGV[$argbegpos+1];  
  &db_find(INFILE, OUTFILE);
  close (OUTFILE);
  close (INFILE);
}
else { print STDERR "db_find.cmd: Too many arguments.\n"; }
#------------------------------------------------------

sub db_find {
local ($W_fin, $W_fout) = @_;
select($W_fout);
#%COMMENT
# Takes a field name and a keyword and retrieves records
# from the database which satisfy the field name and keyword.
#   Example: db_find.cmd -fFLDNAMES -kKEYWRDS -nDBNAME > my.id
# The above retrieves record IDs which have the field names
# called FLDNAMES, keywords KEYWRDS in the database 
# which has the name DBNAME.
#%CMD TYPE 11




@flds = split (/\s+/, $fld_list);
foreach $f (@flds) {
   #print STDERR "DEBUG:field=$f,\n";
  $ndx_filename = $db_name . '_' . $f . '.ndx';
  @keys = split (/\s+/, $keywords);
  foreach $ky (@keys) {
     #print STDERR "keyword=$ky,\n";  # DEBUG
     #print STDERR "Issuing...  grep $ky $ndx_filename\n";
    $res = `grep $ky $ndx_filename`;
    ($w, $rem) = split (/\s+/, $res, 2);
    if ($rem ne '') { print $rem;}
  }
   #print STDERR "\n";  # DEBUG
}
}
1;
