#!/usr/bin/perl
#%COMMENT
# Takes a file of records and builds one main index file on the words
# occurring in the specified fields. Output is a single index file.
# Also creates a db (database) file containing the records
# with unique IDs.
#   Example usage: db_ndx_main.pl  < my.txt  > my.ndx
#   Example usage: db_ndx_main.cmd  < my.txt  > my.ndx
# The above creates an index file my.ndx and a database file my.db.
# (Note: The single index file is broken up into separate index files,
# by db_ndx_sup.pl, db_ndx_sup.cmd.)
#%CMD TYPE 11

$fld_list = '*';  # List of field names (in which words will be indexed). (Default: '*' all) #OPT -f
$db_file = ''; # Filename in which records are stored after giving ID numbers. (Default: 'my.db') #OPT -d
$db_name = 'my'; # Database name using which file names are constructed (e.g. 'prvrb.db' name is constructed if given 'prvrb'). (Default: 'my') #OPT -n

$fld_end = '{'; # End of field marker. Default: '{'  #OPT  -F
$val_end = '}'; # End marker for value of a field. Default: '}' #OPT  -V
$rec_end = "@\n"; # End of record marker. Default: "@\n" #OPT  -R

$fld_names_file = '.myrc'; # Name of file containing field names which are to be indexed. #OP_T -r

if ($db_file eq '') { 
  if ($db_name ne '') {$db_file = $db_name . '.db';}
}
open (DB, ">$db_file") || die "Unable to open file $db_file:$!\n";

#@words = split (/[\",:;'`\\\/\?><\]\[}{\)\(\-_ \n\r\t=!\@\#%&\$\|\*\+\.\xea0-9]+/);  # Not using: ~
$punct =         '[\",:;\'`\\\/\?><\]\[}{\)\(\-_ \n\r\t=!\@\#%&\$\|\*\+\.\xea0-9]';  # No: ~

$old_eol = $/;
$/ = "$rec_end";
$rec_cnt = 0;
while (<>) {
  $rec_cnt++;
  for ($i=1; $i <= length($rec_end); $i++) { chop; }
     # chop; chop; # Remove trailing "@\n" #if (/@/) { #  print STDERR "db_ndx_main.pl: Continuing though spurious '\@' found in:$_"; #} 
  if (/^\s*$/) {   # White space only.
    print STDERR "db_ndx_main.pl: Empty record found ID=$rec_cnt: $_\n";
    next; 
  }
  print DB "ID{$rec_cnt}\n";   #print DB $_, $rec_end;  # Below print only those fields which got processed.
  @field_values = split (/$val_end/);
   #for ($k=0;  $k <= $#field_values;  $k++) { #$field_value = $field_values[$k];
  foreach $field_value (@field_values) {
    if ($field_value =~ /^\s+/) { $field_value = $'; }
    if ($field_value =~ /\s+$/) { $field_value = $`; }
    @fld_val = split (/\s*$fld_end\s*/, $field_value);
    if ($#fld_val > 1) { # if $#fld_val == 0 then OK. val is nil.
      print STDERR "  db_ndx_main.pl: Ignoring. Unexpected field end '$fld_end' found in ID=$rec_cnt:$field_value\n";
      foreach $item (@fld_val) { print STDERR "   item=$item\n"; } #DEBUG
    }
    $fld  = $fld_val[0];
     #print STDERR "DEBUG: Field found in recID=$rec_cnt:$fld,\n";
    if ($fld =~ /\s/) { 
      print STDERR 'db_ndx_main.pl: Ignoring unusual field name ',
                   "in recID=$rec_cnt (with white space): $fld,\n";
      #print DB 'db_ndx_main.pl:DEBUG: Unusual field name in recID:', 
      #          $rec_cnt, ':', $fld, "\n";
      next;
    }
    $val = $fld_val[1];
     #print STDERR "DEBUG: Value found in recID=$rec_cnt:$val,\n";
    if (($fld eq '') && ($val eq '')) {  next; }
    if (($fld_list eq '*') || ($fld_list =~ /$fld/)) { #build ndx on words for specified fields
      @words = split (/$punct+/, $val);
          #@words = split (/[\s,\.;\?\!:]/, $val); # simpler punct.
      if ($#words < 0) { # There was no punct mark. No split occurred.
        $words[0] = $val;
      }
      foreach $w (@words) {
        print $fld, ' : ', $w, '  ', $rec_cnt, "\n";
      }
    }
    if ($fld eq '') { print STDERR "db_ndx_main.pl: empty field in recID=$rec_cnt: $fld\n"; }
    print DB $fld, '{', $val, '}', "\n";
  }
  print DB $rec_end;
}
close DB;
