#!/usr/bin/perl
# Takes a file of fieldname followed by word and indices,
# and builds an index on the words, separate for each fieldname.
#   Example usage: db_ndx_sup.pl -dDBNAME < prvrb.ndx
# The above creates an index file for each of the fieldnames.
# The index files are named by DBNAME_FLDNAME.ndx. DBNAME is
# given separately by -d option. (Default: my.db)
# Options:
# -d	-  Name of database file in which records are stored. (Defaut: 'my.db') 
# -n	-  Database name using which file names are constructed (e.g. 'prvrb.db' name is constructed if given 'prvrb'). (Default: 'my') 
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $Finname, $Foutname);
local ($db_file, $db_name);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$db_file = ''; # Default  Name of database file in which records are stored. (Defaut: 'my.db') 
$db_name = 'my'; # Default  Database name using which file names are constructed (e.g. 'prvrb.db' name is constructed if given 'prvrb'). (Default: 'my') 

# Look up .db_ndx_suprc file. Override defaults.
if (open (RC, '.db_ndx_suprc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/db_ndx_sup.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'd') {
        $db_file = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  'n') {
        $db_name = substr ($_, 2);
      }
      else { 
          print STDERR "db_ndx_sup.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/db_ndx_sup.cmd');
          die;
      }
    }
    else { # 
      print STDERR "db_ndx_sup.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
     # Command line args override those of the rc file.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/db_ndx_sup.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1, 1) eq  'd') {
      $db_file = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'n') {
      $db_name = substr ($ARGV[$i], 2);
    }
    else { 
        print STDERR "db_ndx_sup.cmd: Unrecognized param: $ARGV[$i].\n";
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/db_ndx_sup.cmd');
        die;
    }
  }
  else { # Argument found. Parameters end.
    $argbegpos = $i;
    last;
  }
}

# Process the arguments in the command line then call db_ndx_sup.
# 1. Can be used as a filter. OR
# 2. file args can be given instead of filters.

if ($argbegpos < 0) { # No args.
  &db_ndx_sup(STDIN, STDOUT);
}
elsif (($#ARGV - $argbegpos) == 0) { # Only one arg given. Execute the command with arg and stdout.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "db_ndx_sup.cmd: Unable to open file $ARGV[$argbegpos]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = '';
  &db_ndx_sup(INFILE, STDOUT);
  close (INFILE);
}
elsif (($#ARGV - $argbegpos) == 1) { # Two args given. Execute the command with arg1 and arg2.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "db_ndx_sup.cmd: Unable to open input file $ARGV[$argbegpos]\n" ;
  open (OUTFILE, ">$ARGV[$argbegpos + 1]") ||
     die "db_ndx_sup.cmd: Unable to open output file $ARGV[$argbegpos + 1]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = $ARGV[$argbegpos+1];  
  &db_ndx_sup(INFILE, OUTFILE);
  close (OUTFILE);
  close (INFILE);
}
else { print STDERR "db_ndx_sup.cmd: Too many arguments.\n"; }
#------------------------------------------------------

sub db_ndx_sup {
local ($W_fin, $W_fout) = @_;
select($W_fout);
#%COMMENT
# Takes a file of fieldname followed by word and indices,
# and builds an index on the words, separate for each fieldname.
#   Example usage: db_ndx_sup.pl -dDBNAME < prvrb.ndx
# The above creates an index file for each of the fieldnames.
# The index files are named by DBNAME_FLDNAME.ndx. DBNAME is
# given separately by -d option. (Default: my.db)
#%CMD TYPE 11


if ($db_file eq '') { 
  if ($db_name ne '') {$db_file = $db_name . '.db';}
}

open (NDXFILE, ">/dev/null") ||
  print STDERR "db_ndx_sup.pl: Unable to open file /dev/null:$! ...Continuing.\n";
while (<$W_fin>) {
  chop;
  @fld_w_ndx = split (/ : /);
  if ($#fld_w_ndx != 1) {
    print STDERR "db_ndx_sup.pl: Invalid field. Skipping line:$_\n";
    next;
  }
  $fldname = $fld_w_ndx[0];
  if ($fldname ne $prev_fldname) {
    print NDXFILE $prev_word, '  ', $prev_index, "\n";
    close (NDXFILE);
    $prev_word = '';  $prev_index = '';
    $ndx_filename = $db_name . '_' . $fldname . '.ndx' ;
    open (NDXFILE, ">$ndx_filename") ||
      die "db_ndx_sup.pl: Unable to open file $ndx_filename:$!\n";
    $prev_fldname = $fldname;
  }
  $w_ndx =  $fld_w_ndx[1];
  if ($w_ndx =~ /\s\s/) { $word = $`;  $index = $'; }
  else { 
    print STDERR "db_ndx_sup.pl: Invalid word. Skipping line:$w_ndx\n";
    next;
  }
  if ($word ne $prev_word) { 
    print NDXFILE $prev_word, '  ', $prev_index, "\n";
    $prev_word = $word;  $prev_index = $index;
  }
  else { $prev_index .= ' ' . $index; }
}
print NDXFILE $prev_word, '  ', $prev_index, "\n"; # print the last entry
close NDXFILE;
}
1;
