#!/usr/bin/perl
#%COMMENT
# Takes a file each of whose lines contains:
# a fieldname followed by word and indices (record ids),
# and builds an index on the words, separate for each fieldname.
#   Example usage: db_ndx_sup.pl -nDBNAME < prvrb.ndx
# The above creates an index file for each of the fieldnames.
# The index files are named by DBNAME_FLDNAME.ndx. DBNAME is
# given separately by -n option. (Default: my)
#%CMD TYPE 11

$db_file = ''; # Name of database file in which records are stored. (Defaut: 'my.db') #OPT -d
$db_name = 'my'; # Database name using which file names are constructed (e.g. 'prvrb.db' name is constructed if given 'prvrb'). (Default: 'my') #OPT -n

if ($db_file eq '') { 
  if ($db_name ne '') {$db_file = $db_name . '.db';}
}

open (NDXFILE, ">/dev/null") ||
  print STDERR "db_ndx_sup.pl: Unable to open file /dev/null:$! ...Continuing.\n";
while (<>) {
  chop;
  @fld_w_ndx = split (/ : /);
  if ($#fld_w_ndx != 1) {
    print STDERR "db_ndx_sup.pl: Invalid field. Skipping line:$_\n";
    next;
  }
  $fldname = $fld_w_ndx[0];
  if ($fldname ne $prev_fldname) {
    print NDXFILE $prev_word, '  ', $prev_index, "\n";
    close (NDXFILE);
    $prev_word = '';  $prev_index = '';
    $ndx_filename = $db_name . '_' . $fldname . '.ndx' ;
    open (NDXFILE, ">$ndx_filename") ||
      die "db_ndx_sup.pl: Unable to open file $ndx_filename:$!\n";
    $prev_fldname = $fldname;
  }
  $w_ndx =  $fld_w_ndx[1];
  if ($w_ndx =~ /\s\s/) { $word = $`;  $index = $'; }
  else { 
    print STDERR "db_ndx_sup.pl: Invalid word. Skipping line:$w_ndx\n";
    next;
  }
  if ($word ne $prev_word) { 
    print NDXFILE $prev_word, '  ', $prev_index, "\n";
    $prev_word = $word;  $prev_index = $index;
  }
  else { $prev_index .= ' ' . $index; }
}
print NDXFILE $prev_word, '  ', $prev_index, "\n"; # print the last entry
close NDXFILE;
