#!/usr/bin/perl
# Takes a file of record numbers and retrieves records with the
# given numbers from the database of records specified by  -d 
# parameter (default database file: 'my.db').
#   Example usage: db_show.pl   < my.in  > my.out
#   Example usage: db_show.cmd  < my.in  > my.out
# Options:
# -d	-  Name of database file containing records. (Default: 'my.db') 
# -n	-  Database name using which file name is constructed (e.g. 'prvrb.db' name is constructed if given 'prvrb'). (Default: 'my') 
# -F	-  End of field marker. Default: '{'  
# -V	-  End marker for value of a field. Default: '}' 
# -R	-  End of record marker. Default: "@\n" 
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $Finname, $Foutname);
local ($db_file, $db_name, $fld_end, $val_end, $rec_end);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$db_file = ''; # Default  Name of database file containing records. (Default: 'my.db') 
$db_name = 'my'; # Default  Database name using which file name is constructed (e.g. 'prvrb.db' name is constructed if given 'prvrb'). (Default: 'my') 
$fld_end = '{'; # Default  End of field marker. Default: '{'  
$val_end = '}'; # Default  End marker for value of a field. Default: '}' 
$rec_end = "@\n"; # Default  End of record marker. Default: "@\n" 

# Look up .db_showrc file. Override defaults.
if (open (RC, '.db_showrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/db_show.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'd') {
        $db_file = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  'n') {
        $db_name = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  'F') {
        $fld_end = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  'V') {
        $val_end = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  'R') {
        $rec_end = substr ($_, 2);
      }
      else { 
          print STDERR "db_show.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/db_show.cmd');
          die;
      }
    }
    else { # 
      print STDERR "db_show.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
     # Command line args override those of the rc file.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/db_show.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1, 1) eq  'd') {
      $db_file = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'n') {
      $db_name = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'F') {
      $fld_end = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'V') {
      $val_end = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'R') {
      $rec_end = substr ($ARGV[$i], 2);
    }
    else { 
        print STDERR "db_show.cmd: Unrecognized param: $ARGV[$i].\n";
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/db_show.cmd');
        die;
    }
  }
  else { # Argument found. Parameters end.
    $argbegpos = $i;
    last;
  }
}

# Process the arguments in the command line then call db_show.
# 1. Can be used as a filter. OR
# 2. file args can be given instead of filters.

if ($argbegpos < 0) { # No args.
  &db_show(STDIN, STDOUT);
}
elsif (($#ARGV - $argbegpos) == 0) { # Only one arg given. Execute the command with arg and stdout.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "db_show.cmd: Unable to open file $ARGV[$argbegpos]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = '';
  &db_show(INFILE, STDOUT);
  close (INFILE);
}
elsif (($#ARGV - $argbegpos) == 1) { # Two args given. Execute the command with arg1 and arg2.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "db_show.cmd: Unable to open input file $ARGV[$argbegpos]\n" ;
  open (OUTFILE, ">$ARGV[$argbegpos + 1]") ||
     die "db_show.cmd: Unable to open output file $ARGV[$argbegpos + 1]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = $ARGV[$argbegpos+1];  
  &db_show(INFILE, OUTFILE);
  close (OUTFILE);
  close (INFILE);
}
else { print STDERR "db_show.cmd: Too many arguments.\n"; }
#------------------------------------------------------

sub db_show {
local ($W_fin, $W_fout) = @_;
select($W_fout);
#%COMMENT
# Takes a file of record numbers and retrieves records with the
# given numbers from the database of records specified by  -d 
# parameter (default database file: 'my.db').
#   Example usage: db_show.pl   < my.in  > my.out
#   Example usage: db_show.cmd  < my.in  > my.out
#%CMD TYPE 11


# Not being used:
#$rec_id_list = '';  # List of record ids which will be retrieved. #OP_T -r
#$rec_id_file = 'my.srch'; # Name of file containing record ids which are to be retrieved. (Default: my.srch) #OP_T -f
#$fld_names_file = '.myrc'; # Name of file containing field names which are to be indexed. #O_PT -f
#$fld_list = 'key_words head source';  # List of field names which will be indexed on. #O_PT -i

if ($db_file eq '') { 
  if ($db_name ne '') {$db_file = $db_name . '.db';}
}
open (DB, "$db_file") || 
  die "Unable to open database file $db_file:$!\n";

if ($rec_id_list eq '') {  # record ids not given, look at the file.
  while (<$W_fin>) {
    chop;
    $rec_id_list .= $_;
    $rec_id_list =~ s/\s+/ /;
  }
}

#@words = split (/[\",:;'`\\\/\?><\]\[}{\)\(\-_ \n\r\t=!\@\#%&\$\|\*\+\.\xea0-9]+/);  # Not using: ~
$punct =         '[\",:;\'`\\\/\?><\]\[}{\)\(\-_ \n\r\t=!\@\#%&\$\|\*\+\.\xea0-9]';  # No: ~

$old_eol = $/;
$/ = "$rec_end";
$rec_cnt = 0;
while (<DB>) {
  $rec_cnt++;
     #for ($i=1; $i <= length($rec_end); $i++) { chop; }
  if (/^\s*$/) {   # White space only.
    print STDERR "db_ndx.pl: Empty record found rec num=$rec_cnt: $_\n";
    next; 
  }
     #print STDERR "DEBUG:rec num{$rec_cnt}, $_\n";
  if ($rec_id_list =~ /$rec_cnt/) { print $_; }
}
close DB;

#  @field_values = split (/$val_end/);
#   #for ($k=0;  $k <= $#field_values;  $k++) { #$field_value = $field_values[$k];
#  foreach $field_value (@field_values) {
#    if ($field_value =~ /^\s+/) { $field_value = $'; }
#    if ($field_value =~ /\s+$/) { $field_value = $`; }
#    @fld_val = split (/\s*$fld_end\s*/, $field_value);
#    if ($#fld_val > 1) { # if $#fld_val == 0 then OK. val is nil.
#      print STDERR "  db_ndx.pl: Ignoring. Unexpected field end '$fld_end' found in ID=$rec_cnt:$field_value\n";
#      foreach $item (@fld_val) { print STDERR "   item=$item\n"; } #DEBUG
#    }
#    $fld  = $fld_val[0];
#     #print STDERR "DEBUG: Field found in recID=$rec_cnt:$fld,\n";
#    if ($fld =~ /\s/) { 
#      print STDERR 'db_ndx.pl: Ignoring unusual field name ',
#                   "in recID=$rec_cnt (with white space): $fld,\n";
#      #print DB 'db_ndx.pl:DEBUG: Unusual field name in recID:', 
#      #          $rec_cnt, ':', $fld, "\n";
#      next;
#    }
#    $val = $fld_val[1];
#    if (($fld eq '') && ($val eq '')) {  next; }
#    if (($fld_list eq '*') || ($fld_list =~ /$fld/)) { #build ndx on words for specified fields
#      @words = split (/$punct+/, $val);
#          #@words = split (/[\s,\.;\?\!:]/, $val); # simpler punct.
#      foreach $w (@words) {
#        print $fld, ' : ', $w, '  ', $rec_cnt, "\n";
#      }
#    }
#    if ($fld eq '') { print STDERR "db_ndx.pl: empty field in recID=$rec_cnt: $fld\n"; }
#    print DB $fld, '{', $val, '}', "\n";
#  }
#  print DB $rec_end;
#}
}
1;
