#!/usr/bin/perl
# Re-formats a given text record file by writing field names along with
# their values, and putting each record in a new line. Can also select
# the names fields (using -s option).
# Useful for seeing structured data such as morph output, lwg output, etc.
#   Example usage: fmtrecsel.cmd  -fFIELDS -d';'  -r'@'   infile  outfile
# Options:
# -f	Specifies a file name containing the field names. Each field on a new 
#	line. (Default file: FIELDS).  In case, this switch is not used, it
#	looks for the file name in .fmtrecselrc file in the current directory.
# -s	Specifies the field names that are to be selected from each record.
# -d	Specifies field delimeter in the input file. Default: ','.
# -r	Specifies record delimeter in the input file.  Default: \n.
# -D	Specifies field delimeter for the output file. Default: \n.
# -R	Specifies record delimeter for the output file.  Default: \n\n.
# -=	Says that the output delimters should be the same as for the 
#	input. (Overrides -D and -R options.)
# -nf	Do not put field names in the output.
# -sf	Strip off field names contained in the input file.
# -nor	Do not use record delimiters while reading. Count field 
#	delimiters to determine end of record. (As a default, no record
#	delimiter is used in the output either.)
# -yesR	Turn on the record delimiter in the output, even when the -nor
#	option is on.(Otherwise, -nor option turns off the output record
#	delimiter.)
# -new	Specifies that a new record is to be input, print field names.
# (Note: Record delimiter should NEVER be a substring of the field delimter.
# More generally, a field delimiter should not occur in a field value,
# and a record delimiter should not occur anywhere in the record value,
# including inside a field delimiter.)
# These optons can also be set in .fmtrecselrc file in the current directory.
# Command line parameters always override those in the rc file.
# A sample .fmtrecrselrc file.
#   # This is a comment in the rc file. '-' should begin on col 1.
#   -sName Addr
#   -fFIELDS
#   -d;
#   -r\n
#   -D|
#   -R@@
# cmd assumes 1 to 1.
# $Id: fmtrecsel.cmd,v 1.2 1996/05/13 14:40:32 sangal Exp $
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/fmtrecsel.pl");
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/fmtrecsf.pl");
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/fmtrecnew.pl");
require ("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $attr, $val);
local ($fldnamefile, $dlimtr, $rlimtr,  $dlimtrout, $rlimtrout, $sameinout, $rmfldnames, $noreclimtr, $yesreclimout, $newrec);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$fldnamefile = 'FIELDS';  # Name of the file containing field names.
$dlimtr = ',';  # default delimiter for field values in an input record.
$rlimtr = '\n';  # Default delimiter for input records.
$dlimtrout = '\n';  # Default delimiter for output of field values in a record.
$rlimtrout = '\n\n';  # Default delimiter for output of records.
$sameinout = 0;  # Flag for Delimiters for input and output should be the same.default no.
$rmfldnames = 0;  # Flag to remove field names from the input file.default no.
$nofldnames = 0;  # Flag to say do not put fieldnames in output
$noreclimtr = 0;  # Flag to say there is no input record delimiter. default is no.
$yesreclimout = 0; # Flag to say output record delimiter should be used,
                # even when $noreclimtr is set. Default is no, which is
                # consulted only with -nor option.
$newrec = 0;  # Flag to say that a new record is to be input. Default no.

# Look up .fmtrecselrc file. Override defaults.
if (open (RC, '.fmtrecselrc')) {
  while (<RC>) {
    chop;
    if (/\w*#/) {  next; }   # Ignore. A comment line.
    # if (substr($_,0,1) eq '-') {  # A parameter found.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
       #if (substr($_,1,1) eq 'h') { # print help info.
       #  &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/fmtrecsel.cmd');
       #  die;
       #}
      if (substr($_,1,1) eq 'f') { # Name of field names file.
        $fldnamefile = substr ($_,2);
      }
      elsif (substr($_,1,1) eq 'd') { # delimitor for fields in input
        $dlimtr = substr ($_,2);
      }
      elsif (substr($_,1,1) eq 'r') { # delimitor for records in input
        $rlimtr = substr ($_,2);
      }
      elsif (substr($_,1,1) eq 'D') { # delimitor for output fields
        $dlimtrout = substr ($_,2);
      }
      elsif (substr($_,1,1) eq 'R') { # delimitor for output records
        $rlimtrout = substr ($_,2);
      }
      elsif (substr($_,1,1) eq '=') { # in and out delimiters same
        $sameinout = 1;
      }
      elsif (substr($_,1,2) eq 'sf') { # remove field names from input
        $rmfldnames = 1;
      }
      elsif (substr($_,1,2) eq 'nf') { # do not put field names in output
        $nofldnames = 1;
      }
      elsif (substr($_,1,1) eq 's') { # field names to be selected.
        $fldnamesel = substr ($_, 2);
      }
      elsif (substr($_,1,3) eq 'nor') { # no input record delimiter.
        $noreclimtr = 1;
      }
      elsif (substr($_,1,4) eq 'yesR') { # use output record delimiter.
        $yesreclimout = 1;
      }
      elsif (substr($_,1,3) eq 'new') { # new record to be input
        $newrec = 1;
      }
      else { 
          print STDERR "Unrecognized param in rc file: $_. Ignoring.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/fmtrecsel.cmd');
      }
    }
    else { # 
      print STDERR "Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
     # Command line args override those of the rc file.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/fmtrecsel.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1,1) eq 'f') { # Name of field names file.
      $fldnamefile = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1,1) eq 'd') { # delimitor for fields
      $dlimtr = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1,1) eq 'r') { # delimitor for records
      $rlimtr = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1,1) eq 'D') { # delimitor for output fields
      $dlimtrout = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1,1) eq 'R') { # delimitor for output records
      $rlimtrout = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1,1) eq '=') { # in and out delimiters same
      $sameinout = 1;
    }
    elsif (substr($ARGV[$i],1,2) eq 'sf') { # remove field names from input
      $rmfldnames = 1;
    }
    elsif (substr($ARGV[$i],1,2) eq 'nf') { # do not print field names in output
      $nofldnames = 1;
    }
    elsif (substr($ARGV[$i],1,1) eq 's') { # field names to be selected.
      $fldnamesel = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1,3) eq 'nor') { # no input record delimiter
      $noreclimtr = 1;
    }
    elsif (substr($ARGV[$i],1,4) eq 'yesR') { # use output record delimiter.
      $yesreclimout = 1;
    }
    elsif (substr($ARGV[$i],1,3) eq 'new') { # new record to be input
      $newrec = 1;
    }
    else { 
        print STDERR "Unrecognized param: $ARGV[$i]. Ignoring.\n";
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/fmtrecsel.cmd');
    }
  }
  else { # Argument found. Parameters end.
    $argbegpos = $i;
    last;
  }
}
#print STDERR "DEB params end: $parmlastpos;  args begin: $argbegpos; total: $#ARGV;\n";
if ($rlimtrout eq '') { $rlimtrout = '\n\n'; }
# Change rlimtr, dlimtr etc. to new line octal, if '\n' is set by the user.
$dlimtr =~ s/\\n/\012/g; 
$rlimtr =~ s/\\n/\012/g;
if ($sameinout) {  $dlimtrout = $dlimtr; $rlimtrout = $rlimtr; } # -s flag overrides.
else {
  $dlimtrout =~ s/\\n/\012/g;
  $rlimtrout =~ s/\\n/\012/g;
}

# Process the arguments in the command line then call fmtrecsel.
# 1. Can be used as a filter. OR
# 2. file args can be given instead of filters.

if ($argbegpos < 0) { # No args.
  if ($newrec) { &fmtrecnew (STDIN, STDOUT); }
  elsif ($rmfldnames) { &fmtrecsf (STDIN, STDOUT); }
  else { &fmtrecsel (STDIN, STDOUT); }
}
elsif (($#ARGV - $argbegpos) == 0) { # Only one arg given. Execute the command with arg and stdout.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "Unable to open file $ARGV[$argbegpos]\n" ;
  if ($newrec) { &fmtrecnew (INFILE, STDOUT); }
  elsif ($rmfldnames) { &fmtrecsf (INFILE, STDOUT); }
  else { &fmtrecsel (INFILE, STDOUT); }
  close (INFILE);
}
elsif (($#ARGV - $argbegpos) == 1) { # Two args given. Execute the command with arg1 and arg2.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "Unable to open input file $ARGV[$argbegpos]\n" ;
  open (OUTFILE, ">$ARGV[$argbegpos + 1]") ||
     die "Unable to open output file $ARGV[$argbegpos + 1]\n" ;
  if ($newrec) { &fmtrecnew (INFILE, OUTFILE); }
  elsif ($rmfldnames) { &fmtrecsf (INFILE, OUTFILE); }
  else { &fmtrecsel (INFILE, OUTFILE); }
  close (OUTFILE);
  close (INFILE);
}
else { print STDERR "Too many arguments.\n"; }

