# Called by fmtrecsel.cmd.
# $fldnamesel   names of the fields that are to be selected from each record
# $fldnamefile	Name of file containing the field names each in a new line.
# $dlimtr	delimiter for fields in the input file.
# $rlimtr	delimiter for records in the input file.
# $dlimtrout	delimiter for fields in the output file.
# $rlimtrout	delimiter for records in the output file.
# $noreclimtr	Flag to say that no input record delimiter is used. Use
#		counting of fields to determine record boundary.
# $nofldnames	Flag to say do not put fieldnames in output
# $yesreclimout	Flag to say that output record delimiter should be used
#		even when $noreclimtr flag is set.
# $newrec	New record is to be input. Print field names. (Not used here.)

sub fmtrecsel {
  local ($fin, $fout) = @_;
  local ($oldlimtr, $eachfld, $eachval, $fnum, $dout, $rout, $i, $tf, $tv);
  # print STDERR "DEBUG: FieldNameFile:$fldnamefile,  FieldDelimter:$dlimtr,  RecDelimiter:$rlimtr.\n";
   # First read the field names of the record.
  open (FLDS, $fldnamefile) ||
    die "fmtrecsel.pl: Unable to open file containing field names: $fldnamefile.\n";
  $oldlimtr = $/;
  $/ = '%#adfh5$67';  # To read the entire file of fieldnames.
  $_ = <FLDS>;
  @fields = split (/\n/);
  for ($i=0;  $i <= $#fields;  $i++) {
    $fieldloc{$fields[$i]} = $i;  # This is for fast lookup from field name
                                  # to field location.
  }
  close (FLDS);

  # print STDERR "DEBUG: Fields to be selected: $fldnamesel\n";
  @fnamesel = split (/\s+/, $fldnamesel);
  for ($i=0;  $i <= $#fnamesel;  $i++) {
    $flocsel[$i] = $fieldloc{$fnamesel[$i]};
    # print STDERR "DEB: $fnamesel[$i], $flocsel[$i]\n";
  }

  if (!($noreclimtr)) { # Record delimiters are present.
    $/ = $rlimtr;
    while (<$fin>) {
      for ($i=1; $i <= length($/); $i++) { chop; }   # Remove the delimiter.
      if (/^\s*$/) { # A record with white line or new flag set
        for ($i=0; $i < $#flocsel; $i++) {
          print $fout $fields[$flocsel[$i]], ': ', $dlimtrout;
        }
        print $fout $fields[$flocsel[$#flocsel]], ': ';
        print $fout $rlimtrout;
      }
      else {
        @vals = split (/$dlimtr/);
        if ($#fields != $#vals) {
          print STDERR "fmtrecsel.pl: Mismatch in number of fields $#fields and number of values $#vals for record $_. Ignoring.\n";
        }
        # $tf = $#fields + 1;   # For debugging.
        # $tv = $#vals + 1;
        # print STDERR "fmtrecsel.pl debug: $tf fields are @fields\n $tv values are @vals\n";
        $fnum = 0;
        for ($i=0; $i < $#flocsel; $i++) {
          if ($nofldnames) { # Do not put field names in output
            print $fout  $vals[$flocsel[$i]], $dlimtrout;
          }
          else {
            print $fout $fields[$flocsel[$i]], ': ', $vals[$flocsel[$i]], $dlimtrout;
          }
        }
        if ($nofldnames) { # Do not put field names in output
          print $fout  $vals[$flocsel[$#flocsel]];
        }
        else {
          print $fout $fields[$flocsel[$#flocsel]], ': ', $vals[$flocsel[$#flocsel]];
        }
        print $fout $rlimtrout;
      }
    }
  }
  else { # There is no record delimiter.
    $/ = $dlimtr;  # Read using field delimiters.
    $fnum = 0;
    while (<$fin>) {
      for ($i=1; $i <= length($/); $i++) { chop; } # Remove delimiter.
      if ($nofldnames) { # Do not put field names in output
        print $fout  $_, $dlimtrout;
      }
      else {
        print $fout $fields[$fnum], ': ', $_, $dlimtrout;
      }
      if ($fnum == $#fields) { 
        if ($yesreclimout) { print $fout $rlimtrout; }
        $fnum = 0; 
      }
      else { $fnum++; }
    }
    if ($fnum != 0) {
      print STDERR "fmtrecsel.pl: Premature end of record. $fnum fields read.\nProbably the input file ends. $_.\n";
    }
  }
  $/ = $oldlimtr;
}
1;
