# Called by fmtrec.cmd when user has given command option -sf.
# Strips off field names from the input file besides formatting it.
# $fldnamefile	Contains the names of the fields each in a new line.
# $dlimtr	Contains the delimiter for fields in the input file.
# $rlimtr	Contains the delimiter for records in the input file.
# $dlimtrout	Contains the delimiter for fields in the output file.
# $rlimtrout	Contains the delimiter for records in the output file.
# $noreclimtr	Flag to say that no input record delimiter is used. Use
#		counting of fields to determine record boundary.
# $yesreclimout	Flag to say that output record delimiter should be used
#		even when $noreclimtr flag is set.
# $newrec	New record is to be input. Print field names. (Not used here.)
# $Id: fmtrecsf.pl,v 1.1 1996/05/11 14:28:32 sangal Exp $

sub fmtrecsf {
  local ($fin, $fout) = @_;
  local ($oldlimtr, $eachfld, $eachval, $dout, $rout, $i);
  # print STDERR "FieldNameFile:$fldnamefile,  FieldDelimter:$dlimtr,  RecDelimiter:$rlimtr.\n";
  # First read the field names of the record.
  open (FLDS, $fldnamefile) ||
    die "fmtrecsf.pl: Unable to open file containing field names: $fldnamefile.\n";
  $oldlimtr = $/;
  $/ = '%#adfh5$67';  # To read the entire file of fieldnames.
  $_ = <FLDS>;
  @fields = split (/\n/);
  close (FLDS);

  if (!($noreclimtr)) { # Record delimiters are present.
    $/ = $rlimtr;
    while (<$fin>) {
      for ($i=1; $i <= length($/); $i++) { chop; }   # Remove the delimiter.
      @vals = split (/$dlimtr/);
      if ($#fields != $#vals) {
        print STDERR "fmtrecsf.pl: Mismatch in number of fields $#fields and number of values $#vals for record $_. Ignoring.\n";
      }
      # print STDERR "fmtrecsf.pl debug: $#fields fields are @fields\n $#vals values are @vals\n";
      for ($i=0; $i <= $#fields; $i++) {
        ($fld, $v) = split (/:\s*/, $vals[$i]);
        if ($fields[$i] ne $fld) {
          print STDERR "fmtrecsf.pl: Field name in input file does not match with specified one:$fields[$i]: =!= $fld\n";
        }
        if ($i < $#fields) { print $fout $v, $dlimtrout; }
        else { print $fout $v; }
      }
      print $fout $rlimtrout;
    }
  }
  else { # There is no record delimiter.
    $/ = $dlimtr;  # Read using field delimiters.
    $fnum = 0;
    while (<$fin>) {
      if (/^$/) { last; }
      for ($i=1; $i <= length($/); $i++) { chop; } # Remove delimiter.
      ($fld, $v) = split (/:\s*/);
      if ($fields[$fnum] ne $fld) {
        print STDERR "fmtrecsf.pl: Field name in input file does not match with specified one:$fields[$fnum]: =!= $fld\n";
      }
      print $fout  $v, $dlimtrout;
      if ($fnum == $#fields) { 
        if ($yesreclimout) { print $fout $rlimtrout; }
        $fnum = 0; 
      }
      else { $fnum++; }
    }
    if ($fnum != 0) {
      print STDERR "fmtrecsf.pl: Premature end of record. $fnum fields read.\nProbably the input file ends. $_.\n";
    }
  }
  $/ = $oldlimtr;
}
1;
