# Called by fmtrec.cmd when user has given command option -part (for partial).
# This signifies that only some of the fields in the record have been
# given values. Fields with nil values are not mentioned.
# $fldnamefile	Filename containing the names of the fields each in a new line.
# $dlimtr	the delimiter for fields in the input file.
# $rlimtr	the delimiter for records in the input file.
# $dlimtrout	the delimiter for fields in the output file.
# $rlimtrout	the delimiter for records in the output file.
# $yesreclimout	Flag to say that output record delimiter should be used
#		even when $noreclimtr flag is set.
# $newrec	New record is to be input. Print field names. (Not used here.)

sub fmtrecpart {
  local ($fin, $fout) = @_;
  local (@fields, %fieldloc, @pairs, @vals);
  local ($oldlimtr, $eachfld, $eachval, $dout, $rout, $i);
  # print STDERR "DEBUG: FieldNameFile:$fldnamefile, Partial: $part\n",
                       "flddelim--$dlimtr--, recdelim--$rlimtr--\n";
  # First read the field names of the record.
  open (FLDS, $fldnamefile) ||
    die "fmtrpart.pl: Unable to open file containing field names: $fldnamefile.\n";
  $oldlimtr = $/;
  $/ = '%#adfh5$67';  # To read the entire file of fieldnames.
  $_ = <FLDS>;
  @fields = split (/\n/);
  for ($i=0;  $i <= $#fields;  $i++) {
    $fieldloc{$fields[$i]} = $i;  # This is for fast lookup from
                                  # field name to field location.
  }
  close (FLDS);

  $/ = $rlimtr;  # record delimiter must be present (for records
                 #  with partial fields)
  while (<$fin>) {  # Read a record.
    # print STDERR "fmtrpart.pl DEB: Record is ---$_---\n";
    for ($i=1; $i <= length($/); $i++) { chop; }   # Remove the record delimiter.
    @pairs = split (/$dlimtr/);
    # print STDERR "fmtrpart.pl debug: $#fields fields are @fields\n $#pairs pairs are @pairs\n";
    for ($i=0;  $i <= $#fields;  $i++) { $vals[$i] = ''; } # Initialize to nil.
    for ($i=0; $i <= $#pairs; $i++) { # Store each field val approp.
      ($fld, $v) = split (/:\s*/, $pairs[$i]);
      if (($fieldloc{$fld} < 0) || ($fieldloc{$fld} > $#fields)) {
        print STDERR "fmtrpart.pl: Invalid field name $fld in input file.\n";
      }
      else {
        $vals[$fieldloc{$fld}] = $v;
      }
    }
    for ($i=0; $i <= $#fields; $i++) { # Output the entire record with all field names and values.
      if ($i < $#fields) { print $fout $fields[$i], ':', $vals[$i], $dlimtrout; }
      else { print $fout $fields[$i], ':', $vals[$i]; }
    }
    print $fout $rlimtrout;
  }
  $/ = $oldlimtr;
}
1;
