#!/usr/bin/perl
# Takes as its input a file containing output of 'ls -l'.
# Present date and cutoff dates are given by means of command line args.
# Filters those entries of the input file which are later than
# the cutoff date. (Uses the default cutoff date as 1.1.1901.)
#   Example usage: lsfiltdt.cmd  -p'15.5.96'  -c'14.2.95' < ls.lst  > ls2.lst
# The above takes ls.lst as its input, and produces ls2.lst. Only those
# lines from the input file, which have their date after or equal to '14.2.95' 
# are written to the output. (The present date is given by -p option.)
# cmd assumes 1 to 1.
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/lsfiltdt.pl");
require ("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");


# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$cutoffdate =  '1.1.1901';  # Default string is an early date.
$curdate = ''; # Default null string.
for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/lsfiltdt.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1,1) eq 'c') { # Get the cutoff date
      $cutoffdate = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1,1) eq 'p') { # Get the present date
      $curdate = substr ($ARGV[$i],2);
    }
    else { print STDERR "Unrecognized param: $ARGV[$i]. Ignoring.\n";
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/lsfiltdt.cmd');
    }
  }
  else { # Argument found. Parameters end.
    $argbegpos = $i;
    last;
  }
}
#print STDERR "DEB params end: $parmlastpos;  args begin: $argbegpos; total: $#ARGV;\n";

# Process the arguments in the command line then call lsfiltdt.
# 1. Can be used as a filter. OR
# 2. file args can be given instead of filters.

if ($argbegpos < 0) { # No args.
  &lsfiltdt(STDIN, STDOUT);
}
elsif (($#ARGV - $argbegpos) == 0) { # Only one arg given. Execute the command with arg and stdout.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "Unable to open file $ARGV[$argbegpos]\n" ;
  &lsfiltdt(INFILE, STDOUT);
  close (INFILE);
}
elsif (($#ARGV - $argbegpos) == 1) { # Two args given. Execute the command with arg1 and arg2.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "Unable to open input file $ARGV[$argbegpos]\n" ;
  open (OUTFILE, ">$ARGV[$argbegpos + 1]") ||
     die "Unable to open output file $ARGV[$argbegpos + 1]\n" ;
  &lsfiltdt(INFILE, OUTFILE);
  close (OUTFILE);
  close (INFILE);
}
else { print STDERR "Too many arguments.\n"; }

