#! /usr/bin/perl
# Takes a file of filenames, and puts a command in front and at the tail of
# every line. (A filename per line in the input file.) Lines marked by
# the comment symbol '#' are ignored, unless -yes# option is on.
#   Example usage: names2cmds.cmd  -f'mkdir '   -l' &'   fnames  fnames.sh
# The above creates a file 'fnames.sh' with 'mkdir ' command 
# at the beginning of each line, followed by the line with ' &' at the end of
# the line.
#  Options:
#   -fFRONT	Puts the string FRONT in front of every line containing filename
#   -lLAST	Puts the string LAST at end of every line containing filename
#   -yes#	Does not ignore comment  lines beginning with '#'.
#               (Normally ignores filenames preceded by '#' in the line.)
require ("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/names2cmds.pl");
require ("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$tail = $front = ''; # Default value of params.
$yescomment = 0; # Default value of param.
$fout = '';
for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/names2cmds.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1,1) eq 'f') { # front
      $front = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1,1) eq 'l') { # tail
      $tail = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1,4) eq 'yes#') { # tail
      $yescomment = 1;
    }
    else { 
      print STDERR "Unrecognized param: $ARGV[$i]. Ignoring.\n";
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/names2cmds.cmd');
    }
  }
  else { # Argument found. Parameters end.
    $argbegpos = $i;
    last;
  }
}
#print STDERR "DEB params end: $parmlastpos;  args begin: $argbegpos; total: $#ARGV;\n";

# Process the arguments in the command line then call names2cmds.pl.
# 1. Can be used as a filter. OR
# 2. file args can be given instead of filters.

if ($argbegpos < 0) { # No args.
  &names2cmds (STDIN, STDOUT);
}
elsif (($#ARGV - $argbegpos) == 0) { # Only one arg given. Execute the command with arg and stdout.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "Unable to open file $ARGV[$argbegpos]\n" ;
  &names2cmds (INFILE, STDOUT);
  close (INFILE);
}
elsif (($#ARGV - $argbegpos) == 1) { # Two args given. Execute the command with arg1 and arg2.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "Unable to open input file $ARGV[$argbegpos]\n" ;
  open (OUTFILE, ">$ARGV[$argbegpos + 1]") ||
     die "Unable to open output file $ARGV[$argbegpos + 1]\n" ;
  &names2cmds (INFILE, OUTFILE);
  close (OUTFILE);
  close (INFILE);
}
else { die "Too many arguments.\n"; }

