#!/usr/bin/perl
# Adds the outputs of /usr/bin/time commands in the input file.
#  Example usage: timeadd.pl < time.log  > totaltime
#
# Date: Sat Dec 21 10:02:35 GMT+0500 1996
# 36.77user 0.57system 0:39.46elapsed 94%CPU (0avgtext+0avgdata 0maxresident)k
# 0inputs+0outputs (122major+1701minor)pagefaults 0swaps
# -------- Sorting .ndx file ------
# 1.41user 0.35system 0:02.40elapsed 73%CPU (0avgtext+0avgdata 0maxresident)k
# 0inputs+0outputs (23major+315minor)pagefaults 0swaps

local ($user, $system, $elpasedmin, $elapsedsec, $cpu,
  $totuser, $totsystem, $totelpasedmin, $totelapsedsec, $totcpu,
  $cpuerr, $firstlinefound );
$cpuerr = 0;
while (<>) {
  if (/^([0-9\.]+)user ([0-9\.]+)system ([0-9]+):([0-9\.]+)elapsed /) {
    if ($firstlinefound) { # previous line was first line of time cmd.
      print STDERR "Second line of /usr/bin/time command not found: $_";
    }
    $user =  $1;   $system = $2;  $elapsedmin = $3;   $elapsedsec = $4; 
    $_ = $';
    if (/^([0-9]+)%CPU/) {  $cpu = $1; }
    else { 
      print STDERR "Format of %CPU in time output not matching: $_";
      $cpuerr = 1;
    }
    $totuser += $user;  $totsystem += $system;
    $totelapsedmin += $elapsedmin;  $totelapsedsec += $elapsedsec;
    $totcpu += $cpu;
    $firstlinefound = 1;
  }
  elsif ( /^([0-9]+)inputs\+([0-9]+)outputs \(([0-9]+)major\+([0-9]+)minor\)pagefaults/) {
    $inputs = $1; $outputs = $2;  $major = $3;  $minor = $4;
    $totinputs += $inputs; $totoutputs += $outputs;
    $totmajor += $major;  $totminor += $minor;
    if ($firstlinefound != 1) {
      print STDERR "First line of /usr/bin/time not found before: $_";
    }
    $firstlinefound = 0;
  }
  # else { print STDERR "DEB: Not a /usr/bin/time output: $_"; }
}

if ($cpuerr == 1) { $percentcpu = '-'; }
else { $cpupercent = $totcpu/$.;}
print STDOUT sprintf ("%6.2r user %6.2r system ", $totuser, $totsystem); 
  #print $totuser, 'user ', $totsystem, 'system '; 
print $elapsedmin, ':',
      $elapsedsec, 'elapsed ', $cpupercent, '%CPU', "\n",
      $totinputs, 'inputs+', $totoutputs, 'outputs (', $totmajor, 'major+',
      $totminor, "minor)pagefaults\n";

# 36.77user 0.57system 0:39.46elapsed 94%CPU (0avgtext+0avgdata 0maxresident)k
# 0inputs+0outputs (122major+1701minor)pagefaults 0swaps

