#!/usr/bin/perl
# Adds the outputs of /usr/bin/time commands in the input file.
#  Example usage: timeadd.pl < time.log  > totaltime
#
# Date: Sat Dec 21 10:02:35 GMT+0500 1996
# 36.77user 0.57system 0:39.46elapsed 94%CPU (0avgtext+0avgdata 0maxresident)k
# 0inputs+0outputs (122major+1701minor)pagefaults 0swaps
# -------- Sorting .ndx file ------
# 1.41user 0.35system 0:02.40elapsed 73%CPU (0avgtext+0avgdata 0maxresident)k
# 0inputs+0outputs (23major+315minor)pagefaults 0swaps

local ($user, $system, $elapsedmin, $elapsedsec, $cpu,
  $tot_user, $tot_system, $tot_elapsedmin, $tot_elapsedsec, $tot_cpu,
  $cpuerr, $firstlinefound );
   #$cpuerr = 0;
while (<>) {
  if (/^\s*([0-9\.]+)user\s+([0-9\.]+)system\s+([0-9]+):([0-9\.]+)elapsed\s+/) {
    if ($firstlinefound) { # previous line was first line of time cmd.
      print STDERR "Missing 2nd line of /usr/bin/time command.  Found another 1st line: $_";
    }
    $user =  $1;   $system = $2;  $elapsedmin = $3;   $elapsedsec = $4; 
    $_ = $';
       #if (/^([0-9]+)%CPU/) {  $cpu = $1; }
       #else { 
       #  print STDERR "Format of %CPU in time output not matching: $_";
       #  $cpuerr = 1;
       #}
    $tot_user += $user;  $tot_system += $system;
    $tot_elapsedmin += $elapsedmin;  $tot_elapsedsec += $elapsedsec;
       #$tot_cpu += $cpu;
    $firstlinefound = 1;
  }
  elsif ( /^([0-9]+)inputs\+([0-9]+)outputs\s+\(([0-9]+)major\+([0-9]+)minor\)pagefaults/) {
    $inputs = $1; $outputs = $2;  $major = $3;  $minor = $4;
    $tot_inputs += $inputs; $tot_outputs += $outputs;
    $tot_major += $major;  $tot_minor += $minor;
    if ($firstlinefound != 1) {
      print STDERR "First line of /usr/bin/time not found before: $_";
    }
    $firstlinefound = 0;
  }
  # else { print STDERR "DEB: Not a /usr/bin/time output: $_"; }
}

   #if ($cpuerr == 1) { $cpupercent = '-'; }
   #else { $cpupercent = $tot_cpu/($. - 1); }
$cpupercent = '-';
if ($tot_elapsedsec >= 60) { # convert to minutes.
  $secs = $tot_elapsedsec % 60;
  $mins = ($tot_elapsedsec - $secs) / 60;
  $tot_elapsedmin += $mins;
  $tot_elapsedsec = $secs;
}
print STDOUT sprintf ("%6.2fuser %6.2fsystem %3d:%2delapsed ", 
                       $tot_user, $tot_system, $tot_elapsedmin, $tot_elapsedsec); 
          # $tot_elapsedmin, ':', $tot_elapsedsec, 'elapsed ', 
print $cpupercent, '%CPU', "\n",
      $tot_inputs, 'inputs+', $tot_outputs, 'outputs (', $tot_major, 'major+',
      $tot_minor, "minor)pagefaults\n";

# 36.77user 0.57system 0:39.46elapsed 94%CPU (0avgtext+0avgdata 0maxresident)k
# 0inputs+0outputs (122major+1701minor)pagefaults 0swaps

