#!/usr/bin/perl
# Detects the notation in a given input file (for Indian language files)
#   Example usage: find_code.pl < TEXT
#   Example usage: find_code.cmd  -v  -w200  TEXT
# The answers are: i8, i8p, i7, ra, English etc. for 8-bit ISCII,
#  8-bit PC ISCII, 7-bit ISCII, 
#  IITK Roman alphabetic notation, English, binary file etc.
# Prints characteristics of file on STDERR if option is on.
# Options:
# -v	-  Verbose print the characteristics of file 
# -b	-  Briefly print the characteristics of file 
# -w	-  Number of words to look at. 
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $Finname, $Foutname);
local ($v_charac, $b_charac, $word_max);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$v_charac = 0; #  Verbose print the characteristics of file  Default value as shown.
$b_charac = 0; #  Briefly print the characteristics of file  Default value as shown.
$word_max = 1000; # Default  Number of words to look at. 

# Look up .find_coderc file. Override defaults.
if (open (RC, '.find_coderc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/scrpt/find_code.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'v') {
        $v_charac = 1;
      }
      elsif (substr($_,1, 1) eq  'b') {
        $b_charac = 1;
      }
      elsif (substr($_,1, 1) eq  'w') {
        $word_max = substr ($_, 2);
      }
      else { 
          print STDERR "find_code.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/scrpt/find_code.cmd');
          die;
      }
    }
    else { # 
      print STDERR "find_code.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
     # Command line args override those of the rc file.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/scrpt/find_code.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1, 1) eq  'v') {
      $v_charac = 1;
    }
    elsif (substr($ARGV[$i],1, 1) eq  'b') {
      $b_charac = 1;
    }
    elsif (substr($ARGV[$i],1, 1) eq  'w') {
      $word_max = substr ($ARGV[$i], 2);
    }
    else { 
        print STDERR "find_code.cmd: Unrecognized param: $ARGV[$i].\n";
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/scrpt/find_code.cmd');
        die;
    }
  }
  else { # Argument found. Parameters end.
    $argbegpos = $i;
    last;
  }
}

# Process the arguments in the command line then call find_code.
# 1. Can be used as a filter. OR
# 2. file args can be given instead of filters.

if ($argbegpos < 0) { # No args.
  &find_code(STDIN, STDOUT);
}
elsif (($#ARGV - $argbegpos) == 0) { # Only one arg given. Execute the command with arg and stdout.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "find_code.cmd: Unable to open file $ARGV[$argbegpos]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = '';
  &find_code(INFILE, STDOUT);
  close (INFILE);
}
elsif (($#ARGV - $argbegpos) == 1) { # Two args given. Execute the command with arg1 and arg2.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "find_code.cmd: Unable to open input file $ARGV[$argbegpos]\n" ;
  open (OUTFILE, ">$ARGV[$argbegpos + 1]") ||
     die "find_code.cmd: Unable to open output file $ARGV[$argbegpos + 1]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = $ARGV[$argbegpos+1];  
  &find_code(INFILE, OUTFILE);
  close (OUTFILE);
  close (INFILE);
}
else { print STDERR "find_code.cmd: Too many arguments.\n"; }
#------------------------------------------------------

sub find_code {
local ($W_fin, $W_fout) = @_;
select($W_fout);
#%COMMENT
# Detects the notation in a given input file (for Indian language files)
#   Example usage: scrpt_anal.pl < TEXT
# The answers are: i8, i8p, i7, ra, English etc. for 8-bit ISCII,
#  8-bit PC ISCII, 7-bit ISCII, 
#  IITK Roman alphabetic notation, English, binary file etc.
# Prints characteristics of file on STDERR if option is on.
#%CMD TYPE 11

# In var names: _ccnt - means char counts
# In var names: _wcnt - means word counts
$uc_ccnt = $lc_ccnt = $tot_ccnt = $i8_ccnt = $i8p_ccnt = 0;  # Counts of chars: upper case, lc, total, iscii8, iscii8 PC
$word_wcnt = 0; # Count of total words.
$x_wcnt = 0;    # Count of words beginning with 'x'.
$ucword_ccnt = 0; # Count of uc chars when all upper case letters in word.
$ucword_wcnt = 0; # Count of words with all upper case letters in them.
$mixed_wcnt = 0; # Count of words with mixed case (other than the first char)
while (<$W_fin>)  {
  @words = split (/\s+/);
  $word_wcnt += $#words + 1;
  for ($i=0;  $i <= $#words;  $i++)  {
    $uc_in_word = ($words[$i] =~ tr/A-Z/A-Z/);
    $uc_ccnt += $uc_in_word; 
    $lc_in_word = ($words[$i] =~ tr/a-z/a-z/);
    $lc_ccnt += $lc_in_word; 
    $i8_in_word = ($words[$i] =~ tr/\xA1-\xEA/\xA1-\xEA/);
    $i8_ccnt += $i8_in_word; 
    $i8p_in_word = ($words[$i] =~ tr/\x80-\xAF\xE1-\xFB/\x80-\xAF\xE1-\xFB/);
      # $i8p_in_word += ($words[$i] =~ tr/\xE1-\xFB/\xE1-\xFB/);
    $i8p_ccnt += $i8p_in_word; 
    $tot_in_word = length ($words[$i]);
    $tot_ccnt += $tot_in_word; 
    if ($lc_in_word == 0) {  # Word without lower case letters
      if ($uc_in_word != 0) { # Word has some uc letters
        $ucword_ccnt += $uc_in_word;  # For use in iitk roman notation
        $ucword_wcnt++;
      }
    }
    if ($words[$i] =~ /^x/) { $x_wcnt++; }  # For 7bit ISCII
    $rest_word = substr ($words[$i], 1);    # For IITK roman
    if ( ($rest_word =~ /[A-Z]/) && ($rest_word =~ /[a-z]/) ) {
      $mixed_wcnt++;
    }
  }
  if ( ($word_wcnt > $word_max) || ($tot_ccnt > (20 * $word_max)) )
    { last; }
}

$uclc_ccnt = $uc_ccnt + $lc_ccnt;
if ($tot_ccnt != 0)  { $uclc_tot_ratio = $uclc_ccnt / $tot_ccnt;
                       $i8_tot_ratio = $i8_ccnt / $tot_ccnt;
                       $i8p_tot_ratio = $i8p_ccnt / $tot_ccnt;
                     }
               else  { $uclc_tot_ratio = '-';
                       $i8_tot_ratio = '-';
                       $i8p_tot_ratio = '-';
                     }
if ($uclc_ccnt != 0) { $uc_uclc_ratio = $uc_ccnt / $uclc_ccnt; }
               else  { $uc_uclc_ratio = '-'; }
if ($word_wcnt != 0) { $x_ratio = $x_wcnt / $word_wcnt; }
               else  { $x_ratio = '-'; }
if ($word_wcnt != 0) { $mixed_ratio = $mixed_wcnt / $word_wcnt; }
               else  { $mixed_ratio = '-'; }
# The following ignores words with all capitals.
if ($tot_ccnt != 0)  { $uclc_tot_ratio2 = ($uclc_ccnt - $ucword_ccnt)
                                        / ($tot_ccnt - $ucword_ccnt); }
               else  { $uclc_tot_ratio2 = '-'; }
if ($uclc_ccnt != 0) { $uc_uclc_ratio2 = ($uc_ccnt - $ucword_ccnt)
                                        / ($uclc_ccnt - $ucword_ccnt); }
               else  { $uc_uclc_ratio2 = '-'; }
$word_wcnt2 = $word_wcnt - $ucword_wcnt;

if ($b_charac || $v_charac) {
  print STDERR sprintf 
    ("UCLC/TOT=  %3.2f, UC/UCLC=  %3.2f, I8/TOT=%3.2f, I8P/TOT=%3.2f, X/WRDS=%3.2f, MXD/WRDS= %3.2f,", 
       $uclc_tot_ratio, $uc_uclc_ratio, $i8_tot_ratio, $i8p_tot_ratio, $x_ratio, $mixed_ratio);
  print STDERR "\n";
}
if ($v_charac) {
   #--- The following are the results of ignoring words with all capitals
   #print STDERR sprintf ("UCLC2/TOT2=%3.2f, UC2/UCLC2=%3.2f,",
   #     $uclc_tot_ratio2, $uc_uclc_ratio2);
   #print STDERR "\n";
  print STDERR " Chars: UC,LC,TOT,I8,I8P,UCC=($uc_ccnt,$lc_ccnt,$tot_ccnt,$i8_ccnt,$i8p_ccnt,$ucword_ccnt)";
  print STDERR " Wrds: W,XW,MXD=($word_wcnt,$x_wcnt,$mixed_wcnt),\n";
   #print STDERR " Wrds: W,XW,W2,UCW=($word_wcnt,$x_wcnt,$word_wcnt2,$ucword_wcnt),\n"; #DEBUG
}

#--- For returning name of notation:
$ans = 'UNDEF';
if ($x_ratio > .60) { $ans = 'i7'; }
elsif ($i8_tot_ratio > .60) { 
  if ($i8_tot_ratio > $i8p_tot_ratio ) { $ans = 'i8'; }
  else { $ans = 'i8p'; }
}
elsif ($i8p_tot_ratio > .60) { $ans = 'i8p'; }
elsif (($uclc_tot_ratio > .60) && ($mixed_ratio > .40)) { $ans = 'ra'; }
                                   # ^^ For Punjabi the above is .50
elsif ($uclc_tot_ratio > .60) { $ans = 'english'; }
elsif (($uclc_tot_ratio > .03) && ($uc_uclc_ratio > .2)) { $ans = 'bin';}
print $ans, "\n";
}
1;
