#!/usr/bin/perl
# This program identifies the language of the text in the input
# file (which is assumed to be in iitk roman alphabetic notation).
#   Example usage: find_lang.pl < INFILE
#   Example usage: find_lang.cmd  -v  -w100 < INFILE
# Prints characteristics of file on STDERR if option -v is on.
# Therefore, it can also be used to try out different parameters
# for making the program run more effectively.
# Options:
# -v	-  Verbose print the characteristics of file 
# -w	-  Number of words to look at in the input file. 
require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $Finname, $Foutname);
local ($v_charac, $word_max);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$v_charac = 0; #  Verbose print the characteristics of file  Default value as shown.
$word_max = 500; # Default  Number of words to look at in the input file. 

# Look up .find_langrc file. Override defaults.
if (open (RC, '.find_langrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/scrpt/find_lang.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'v') {
        $v_charac = 1;
      }
      elsif (substr($_,1, 1) eq  'w') {
        $word_max = substr ($_, 2);
      }
      else { 
          print STDERR "find_lang.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/scrpt/find_lang.cmd');
          die;
      }
    }
    else { # 
      print STDERR "find_lang.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
     # Command line args override those of the rc file.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/scrpt/find_lang.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1, 1) eq  'v') {
      $v_charac = 1;
    }
    elsif (substr($ARGV[$i],1, 1) eq  'w') {
      $word_max = substr ($ARGV[$i], 2);
    }
    else { 
        print STDERR "find_lang.cmd: Unrecognized param: $ARGV[$i].\n";
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/scrpt/find_lang.cmd');
        die;
    }
  }
  else { # Argument found. Parameters end.
    $argbegpos = $i;
    last;
  }
}

# Process the arguments in the command line then call find_lang.
# 1. Can be used as a filter. OR
# 2. file args can be given instead of filters.

if ($argbegpos < 0) { # No args.
  &find_lang(STDIN, STDOUT);
}
elsif (($#ARGV - $argbegpos) == 0) { # Only one arg given. Execute the command with arg and stdout.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "find_lang.cmd: Unable to open file $ARGV[$argbegpos]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = '';
  &find_lang(INFILE, STDOUT);
  close (INFILE);
}
elsif (($#ARGV - $argbegpos) == 1) { # Two args given. Execute the command with arg1 and arg2.
  open (INFILE, "$ARGV[$argbegpos]") ||
     die "find_lang.cmd: Unable to open input file $ARGV[$argbegpos]\n" ;
  open (OUTFILE, ">$ARGV[$argbegpos + 1]") ||
     die "find_lang.cmd: Unable to open output file $ARGV[$argbegpos + 1]\n" ;
  $Finname = $ARGV[$argbegpos];  $Foutname = $ARGV[$argbegpos+1];  
  &find_lang(INFILE, OUTFILE);
  close (OUTFILE);
  close (INFILE);
}
else { print STDERR "find_lang.cmd: Too many arguments.\n"; }
#------------------------------------------------------

sub find_lang {
local ($W_fin, $W_fout) = @_;
select($W_fout);
#%COMMENT
# This program identifies the language of the text in the input
# file (which is assumed to be in iitk roman alphabetic notation).
#   Example usage: find_lang.pl < INFILE
#   Example usage: find_lang.cmd  -v  -w100 < INFILE
# Prints characteristics of file on STDERR if option -v is on.
# Therefore, it can also be used to try out different parameters
# for making the program run more effectively.
#%CMD TYPE 11

open(BNG, "/home/bharathi/PNJ1/usr3/sangal/tools/src/scrpt/lang_data/word_bng.lst") || 
  die "can't open file /home/bharathi/PNJ1/usr3/sangal/tools/src/scrpt/lang_data/word_bng.lst: $!\n";
open(TLG, "/home/bharathi/PNJ1/usr3/sangal/tools/src/scrpt/lang_data/word_tlg.lst") || 
  die "can't open file /home/bharathi/PNJ1/usr3/sangal/tools/src/scrpt/lang_data/word_tlg.lst: $!\n";
open(PNJ, "/home/bharathi/PNJ1/usr3/sangal/tools/src/scrpt/lang_data/word_pnj.lst") ||
  die "can't open file /home/bharathi/PNJ1/usr3/sangal/tools/src/scrpt/lang_data/word_pnj.lst: $!\n";
open(HND, "/home/bharathi/PNJ1/usr3/sangal/tools/src/scrpt/lang_data/word_hnd.lst") ||
  die "can't open file /home/bharathi/PNJ1/usr3/sangal/tools/src/scrpt/lang_data/word_hnd.lst: $!\n";
open(MRT, "/home/bharathi/PNJ1/usr3/sangal/tools/src/scrpt/lang_data/word_mrt.lst") ||
  die "can't open file /home/bharathi/PNJ1/usr3/sangal/tools/src/scrpt/lang_data/word_mrt.lst: $!\n";
open(ORY, "/home/bharathi/PNJ1/usr3/sangal/tools/src/scrpt/lang_data/word_ory.lst") ||
  die "can't open file /home/bharathi/PNJ1/usr3/sangal/tools/src/scrpt/lang_data/word_ory.lst: $!\n";
while(<BNG>) { chop; $bng{$_} = 1; }
while(<TLG>) { chop; $tlg{$_} = 1; }
while(<PNJ>) { chop; $pnj{$_} = 1; }
while(<HND>) { chop; $hnd{$_} = 1; }
while(<MRT>) { chop; $mrt{$_} = 1; }
while(<ORY>) { chop; $ory{$_} = 1; }
close(BNG); close(TLG); close(PNJ); close(HND);close(MRT); close(ORY);
#SEPARATOR = [\",:;'\?><\]\[\}\{\)\( \n\t_\-=!\@#%&\*\.\\/]
$bng_cnt=0; $tlg_cnt=0; $pnj_cnt=0; $hnd_cnt=0; $mrt_cnt=0; $ory_cnt=0; $tot_wrd=0;
while(<$W_fin>) {
  s/[\",:;'\?><\]\[\}\{\)\(_\-\=\!\@\#\%\&\*\.\\\/]//g;
  s/^\s+//;
  if(/^\s*$/) { 
    # print STDERR "WHITE SPACE ONLY\n"; # Debugging
    next;
  }
  chop;
  @word_lst = split(/\s+|\t+/);
  $tot_wrd += $#word_lst;
  for($i=0; $i<$#word_lst; $i++) {
    if($bng{$word_lst[$i]} == 1) { $bng_cnt++;
      # print STDERR "BNG :$word_lst[$i]:\n"; #DEBUGGING
    }
    if($tlg{$word_lst[$i]} == 1) { $tlg_cnt++;
      # print STDERR "TLG :$word_lst[$i]:\n"; #DEBUGGING
    }
    if($pnj{$word_lst[$i]} == 1) { $pnj_cnt++;
      # print STDERR "PNJ :$word_lst[$i]:\n"; #DEBUGGING
    }
    if($hnd{$word_lst[$i]} == 1) { $hnd_cnt++;
      # print STDERR "HND :$word_lst[$i]:\n"; #DEBUGGING
    }
    if($mrt{$word_lst[$i]} == 1) { $mrt_cnt++;
      # print STDERR "MRT :$word_lst[$i]:\n"; #DEBUGGING
    }
    if($ory{$word_lst[$i]} == 1) { $ory_cnt++;
      # print STDERR "ORY :$word_lst[$i]:\n"; #DEBUGGING
    }
  }
  if ($tot_wrd >= $word_max) { last; } # Exit while loop.
} # End of while loop

if ($tot_wrd == 0) { 
  print STDERR "Warn: Zero words in file.\n";
  $output1 = 'UNDEF';
}
else {
  $bng=$bng_cnt*100/$tot_wrd;
  $tlg=$tlg_cnt*100/$tot_wrd;
  $pnj=$pnj_cnt*100/$tot_wrd;
  $hnd=$hnd_cnt*100/$tot_wrd;
  $mrt=$mrt_cnt*100/$tot_wrd;
  $ory=$ory_cnt*100/$tot_wrd;
  if ($v_charac) {
    print STDERR "WORDS:$tot_wrd -> ";
    #print STDERR sprintf("PNJ:%d% HND:%d% BNG:%d% ORY:%d% MRT:%d% TLG:%d% \n",
    print STDERR sprintf("PNJ:%d HND:%d BNG:%d ORY:%d MRT:%d TLG:%d /",
                            $pnj, $hnd, $bng, $ory, $mrt, $tlg);
     #print STDERR sprintf("HND:%d% PNJ:%d% BNG:%d% TLG:%d% MRT:%d% ORY:%d% \n",
     #                      $hnd, $pnj, $bng, $tlg, $mrt, $ory);
  }

  # Highest goes to output. (Error check 1. Percentage more than 30.
  # check 2: Highest and second highest numbers are more than 20% apart.
  @list = ($pnj, $hnd, $bng, $ory, $mrt, $tlg);
  @newlist = sort numerically @list;
   #foreach $l (@list) { print $l, " "; }  print "\n";    #DEBUG
   #foreach $l (@newlist) { print $l, " "; }  print "\n"; #DEBUG
  if    ($newlist[$#list] == $hnd) { $output1 = 'hindi'; }
  elsif ($newlist[$#list] == $pnj) { $output1 = 'punjabi'; }
  elsif ($newlist[$#list] == $bng) { $output1 = 'bengali'; }
  elsif ($newlist[$#list] == $tlg) { $output1 = 'telugu'; }
  elsif ($newlist[$#list] == $mrt) { $output1 = 'marathi'; }
  elsif ($newlist[$#list] == $ory) { $output1 = 'oriya'; }
  else { $output1 = 'UNDEF'; }

  if ($tot_wrd <= 30) { print STDERR "Warn: Less than 30 words in file. Check answer.\n"; }
  if    ($newlist[$#list] <= 25) { $output1 = 'UNDEF'; }
  elsif ($newlist[$#list - 1] >= (0.8 * $newlist[$#list])) {
    print STDERR "Warn: Not sure of the identified language. Check answer. \n";
  }
}
print $output1, "\n";

sub numerically { $a <=> $b; }

}
1;
