#!/usr/bin/perl
# Called by gencmd.cmd.
# Takes a description of command line and rc file options, 
# and generates the cmd file.
# $suff		- Suffix to be used for naming the output file. Default '.cmd'.
# $funcname	- Name of the function to be called.

sub gencmd {
  local ($fin, $fout) = @_;
  local ($fileframe, $comment, $require, @optlines, @optname, @optsw, @optswvalue, @optinit, @optvar, @optcomment);
  local ($v, $keyword, $i, $len);
  # print STDERR "gencmd.pl DEBUG: funcname = $funcname,\n";
  $/ = '#%';
  $_ = <$fin>;   chop;  chop;  print $fout $_;  # Read-print the first part
  while (<$fin>) { # Read in the option description.  In the given .opt file.
    chop;  chop;
    if (/^CMD\s+TYPE/) { # Choose many to many, many to one, one to one etc.
      $_ = $';
      if (/^\s+11/) { $fileframe = '/home/bharathi/PNJ1/usr3/sangal/tools/src/SUPP/cmd11.frm.gen'; }
      elsif (/^\s+M1/) { $fileframe = '/home/bharathi/PNJ1/usr3/sangal/tools/src/SUPP/cmdm1.frm.gen'; }
      elsif (/^\s+MM/) { $fileframe = '/home/bharathi/PNJ1/usr3/sangal/tools/src/SUPP/cmdmm.frm.gen'; }
      else { die "Illegal command type specified: $'"; }
    }
    elsif ($keyword =~ /^END/) { print $fout $_; }
    elsif (/^SPECIAL/) {
      print STDERR "Not handling SPECIAL keyword in *.opt file yet. Handle manually.\n";
    }
    elsif (/^COMMENT\s*\n/) { $comment = $'; }
    elsif (/^REQUIRE\s*\n/) { $require = $'; }
    elsif (/^OPTIONS\s*\n/) { 
      $_ = $';
      @optlines = split (/\n/);  # Separates each opt line.
      for ($i=0;  $i <= $#optlines;  $i++) { # Process each opt line.
        # FORMAT for opt line:       -OPT sw0 $VAR=INIT;  # COMMENT
        #  optional parts in opt line -->  ^^     ^  ^    ^ ^
        # Example:
        #    -f       $fldnamefile = '';  # Name of the file containing field names.
        #    -yes sw0 $Dlimflag = 1; # Print output Dlimiter.
        $_ = $optlines[$i];
        # Process the option name
        if (/\s+/) { # left part is option name, right part is remaining.
          if (substr($`,0,1) eq '-') { $optname[$i] = substr ($`, 1); }
          else { 
            $optname[$i] = $`;
            print STDERR "Option $optname[$i] must begin with a '-'. Continuing.\n";
          }
          $_ = $';
          # Process the 'sw' option with 0, 1 or space.
          if (/^sw/) { 
            $optsw[$i] = 1;  
            $_ = $';
            if (/^0/) { $optswvalue[$i] = 0;  $_ = $'; }
            elsif (/^1/) { $optswvalue[$i] = 1;  $_ = $'; }
            elsif (/^\s/) { $optswvalue[$i] = 1; $_ = $'; }
            else { print STDERR "Error: sw not followed by 0, 1, space: $_"; }
            if (/^\s+/) { $_ = $'; }
          }
          else { $optsw[$i] = 0; }
          # Process the variable naming and initialization.
          if (/;\s*/) { # Found ';'. 
            $optinit[$i] = $` ;  
            $_ = $';
            # Now separate the var name from the rest of initialization.
            if ($optinit[$i] =~ /\s*=\s*/) { 
              $optvar[$i] = $`; 
              $optinit[$i] = $optinit[$i] . ';'; 
            }
            else { # Not an initialization. Only var name given.
              $optvar[$i] = $optinit[$i];  $optinit[$i] = '';
            } 
          }
          else { 
            print STDERR "Error: ';' expected after var name or initialization. $_";
            $optinit[$i] = '';
          }
          # Process the comment.
          if (/^#/) { $optcomment [$i] = $'; }
          elsif (!(/^$/)) { print STDERR "Error: Comment or new line expected: $_"; }
          else { $optcomment[$i] = ''; }
        }
        else { print STDERR "Error: No space found: $_\n"; }
        # print STDERR "gencmd.pl DEBUG: $optlines[$i]\n$optname[$i]\t$optsw[$i]\t$optinit[$i]\n";
        # print STDERR "$optname[$i]\t$optsw[$i]:$optswvalue[$i]:\t$optvar[$i]\n";
      }  # End of for loop for each opt line.
    }
    else { print STDERR "Error: Keyword not recognized in *.opt file: $_"; }
  } # End of while loop.
  # print STDERR "gencmd.pl DEBUG: Comment ----$comment----\n";

  open (FRAME, $fileframe) ||
    die "gencmd.pl: Unable to open file $fileframe.\n";
  $_ = <FRAME>;   chop;  chop; 
  s/DUMMY/$funcname/g;  
  print $fout $_;  # Read-print the first part
  while (<FRAME>) {  # Read in chunks delimited by '#%'
    chop;  chop;
    s/DUMMY/$funcname/g;  
    if (/\n/) { $keyword = $`;  $_ = $'; }
    else { print $fout $_;   next; }
    if ($keyword =~ /^COMMENT\s*/) { 
      print $fout $comment;
      &prt_optcomment;
      print $fout $_;
    }
    elsif ($keyword =~ /^REQUIRE\s*/) { 
      if (!($require =~ /printusage/)) {
        print $fout 'require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");', "\n";
      }
      print $fout $require, $_;
    }
    elsif ($keyword =~ /^LOCAL\s*/) {  # Generate local var declaration.
      if ($#optlines >= 0) {
        print $fout "local ($optvar[0]";
        for ($j=1;  $j <= $#optvar;  $j++) { print $fout ', ', $optvar[$j]; }
        print $fout ");\n";
      }
      print $fout $_;
    }
    elsif ($keyword =~ /^OPTIONS\s+DEFAULTS\s*/) {  # generate initializations.
      for ($i=0; $i <= $#optlines;  $i++) { # do for each option line.
        # print $fout "DEBUG: $optinit[$i]\n";
        if ($optinit[$i] ne '') { # There is an initialization.
          if (!($optsw[$i])) { print $fout $optinit[$i], ' # Default ', $optcomment[$i], "\n"; }
          else { print $fout $optinit[$i], ' # ', $optcomment[$i], 
                       " Default as shown.\n"; }
        }
        else { print $fout "# $optvar[$i] - $optcomment[$i]\n"; }
      }
      print $fout $_;
    }
    elsif ($keyword =~ /^RC\s+ELSIF\s+STMTS\s*/) { # generate RC elsif for options.
      for ($i=0; $i <= $#optlines;  $i++) { # do for each option line.
        $len = length ($optname[$i]);
        if (!($optsw[$i])) { 
          print $fout "      elsif (substr(\$_,1, $len) eq  '$optname[$i]') {\n";
          $len1 = $len + 1;
          print $fout "        $optvar[$i] = substr (\$_, $len1);\n      }\n";
        }
        else {
          print $fout "      elsif (substr(\$_,1, $len) eq  '$optname[$i]') {\n";
          print $fout "        $optvar[$i] = $optswvalue[$i];\n      }\n";
        }
      }
      print $fout $_;
    }
    elsif ($keyword =~ /^CMD\s+LINE\s+ELSIF\s+STMTS\s*/) { # generate cmdline elsif for options.
      for ($i=0; $i <= $#optlines;  $i++) { # do for each option line.
        $len = length ($optname[$i]);
        if (!($optsw[$i])) { 
          print $fout "    elsif (substr(\$ARGV[\$i],1, $len) eq  '$optname[$i]') {\n";
          $len1 = $len + 1;
          print $fout "      $optvar[$i] = substr (\$ARGV[\$i], $len1);\n    }\n";
        }
        else {
          print $fout "    elsif (substr(\$ARGV[\$i],1, $len) eq  '$optname[$i]') {\n";
          print $fout "      $optvar[$i] = $optswvalue[$i];\n    }\n";
        }
      }
      print $fout $_;
    }
    elsif ($keyword =~ /^BEGIN/) { print $fout $_; }
    elsif ($keyword =~ /^END/) { print $fout $_; }
    elsif ($keyword =~ /^SPECIAL/) {  # Do nothing. print $fout $_; 
      print STDERR "Not handling SPECIAL keyword in *.frm file yet.\n";
    }
    else { print STDERR "Error: Unknown keyword in *.frm file: $keyword $_"; }
  }
}
sub prt_optcomment {
  local ($i);
  # print $fout "prt_optcomment DEBUG: opts = $#optlines\n";
  print $fout "# Options:\n";
  for ($i=0;  $i <= $#optlines;  $i++) {
    print $fout '# -', $optname[$i], "\t- ", $optcomment[$i], "\n";
  }
}
1;
