# Called by gencmdmenu.pl.
# Takes a description of menu options and generates the .menu file.
# Generates DUMMY.menu function. (Where DUMMY is replaced by $funcname.)

# The values of variables shown below are already set as shown by example:
# FORMAT for opt line:       -OPT sw0 $VAR=INIT;  # COMMENT
#  optional parts in opt line -->  ^^     ^  ^    ^ ^
# Example:
#    -f       $fldnamefile = '';  # Name of the file containing field names.
#    -yes sw0 $Dlimflag = 1; # Print output Dlimiter.

# FORMAT for menu spec line:  -OPT {ask|noask} VALSPECIF
# Example:
#   -f noask  list('<.>', '\n',)
#   -yes ask

# Produces the following data structure
#  @optname = { 'f', 'yes'}
#  @optsw   = {  0,  1 }     # Indicates non switch option 'f', and sw option 'yes'.
#  @optswvalue = { '', 0 }   # Indicates 0 of 'sw0'
#  @optvar = { '$fldnamefile', '$Dlimflag' }
#  @optinit = { '$fldnamefile = \'\';', '$Dlimflag = 1;' } 
#  @optcomment = { 'Name of the file containing field names.', 'Print output Dlimiter.' }

# If menu specs were found $found_menu is set to 1, and the following are set:
#  @ask = { 'noask', 'ask' }
#  @valspec = { 'list(\'<.>\', \'\n\',)', '' }
#  @opt_name = {'f', 'yes'}        #      Names of options
#  @opt_val  = User given inputs for the above options

# $suff		- Suffix to be used for naming the output file. Default '.cmd'.
# $funcname	- Name of the function to be called.

sub genmenu {
  open (GENOUT, ">$funcname.menu") ||
    die "genmenu.pl: Unable to open out file /home/bharathi/PNJ1/usr3/sangal/tools/src/$funcname.menu\n";
  open (FRAME, '/home/bharathi/PNJ1/usr3/sangal/tools/src/menu.frm') ||
    die "genmenu.pl: Unable to open file /home/bharathi/PNJ1/usr3/sangal/tools/src/menu.frm\n";
  $_ = <FRAME>;   chop;  chop; 
  s/DUMMY/$funcname/g;  
  print GENOUT $_;  # Read-print the beginning part of FRAME file.
  while (<FRAME>) {  # Read in chunks delimited by '#%'
    chop;  chop;
    s/DUMMY/$funcname/g;  
    if (/\n/) { $keyword = $`;  $_ = $'; }
    else { print GENOUT $_;   next; }
    if ($keyword =~ /^COMMENT\s*/) { 
      print GENOUT $optcomment;
      &prt_optcomment;
      print GENOUT $_; # print any remaining stuff read from FRAME.
    }
    elsif ($keyword =~ /^LOCAL\s*/) {  # Generate local var declaration.
      print GENOUT "local ($optvar[0]";
      for ($i=1;  $i <= $#optvar;  $i++) { print GENOUT ', ', $optvar[$i]; }
      print GENOUT ");\n";
      print GENOUT "\# Generate inits to store opt names.\n";
      for ($i=0;  $i <= $#optvar;  $i++) { 
        print GENOUT "\$opt_name[$i] = '$optname[$i]';\n";
      }
      print GENOUT $_; # print any remaining stuff read from FRAME.
    }
    elsif ($keyword =~ /^TOASK\s*/) {  # Set var @toask and %asked.
      print GENOUT '@toask = (';
      $firstout = 1;
      for ($j=1;  $j <= ($#optvar + 1);  $j++) { 
        if ($ask[$j - 1] eq 'ask') { 
          if ($firstout) { $firstout = 0; }
          else { print GENOUT ','; }
          print GENOUT '\'"', $j, '"\'';
        }
      }
      print GENOUT ");\n";
      # Set %asked variable.
      for ($i=0;  $i <= $#optvar;  $i++) { 
        if ($ask[$i] eq 'ask') { 
          print GENOUT "\$asked{$i} = 1;\n";
        }
      }
      print GENOUT $_; # print any remaining stuff read from FRAME.
    }
    elsif ($keyword =~ /^PROCESS\s+INPUT/) {  # Issue call for getting user input.
      $indent = '    ';
      $firstout = 1;
      for ($j=1;  $j <= ($#optvar + 1);  $j++) { 
        if ($firstout) { $firstout = 0; print GENOUT $indent; }
        else { print GENOUT $indent, 'els'; }
        print GENOUT "if (\$toask[\$i] == '\"$j\"')  {\n";
        $j1 = $j - 1;
        print GENOUT $indent, "  \$res = &ask_opt_val_$j1;\n";
        print GENOUT $indent, '  if ($res != 0) { print "Skipping to next menu ...\n"; next; }', "\n"; 
        print GENOUT $indent, "  else { \$opt_val[\$i] = &get_val; }\n";
        print GENOUT $indent, "}\n";
      }
      print GENOUT $_; # print any remaining stuff read from FRAME.
    }
    elsif ($keyword =~ /^CONFIRM/) {
      $indent = '           ';
      print GENOUT '  ', $#optvar + 1, '\\\\', "\n";
      for ($j=1;  $j <= ($#optvar + 1);  $j++) { 
        $tmpcomnt = $optcomment[$j - 1];
        $j1 = $j - 1;
        print GENOUT $indent, "\\\"$j\\\" \\\"\$opt_val[$j1] - $tmpcomnt\\\" off\\\\", "\n";
      }
      print GENOUT $_; # print any remaining stuff read from FRAME.
    }
    elsif ($keyword =~ /^CALLCMD/) {
      print GENOUT
       "\$cmdissue = $funcname . '.cmd ';\n",
       "for (\$i=0;  \$i <= $#optvar;  \$i++) {\n",
       "  if (\$asked{\$i}) {\n",
       "    if (\$optname[\$i] eq 'ARGIN') { \$cmdissue = \$cmdissue . \$opt_val[\$i] . ' '; }\n",
       "    elsif (\$optname[\$i] eq 'ARGOUT') { \$cmdissue = \$cmdissue . \$opt_val[\$i] . ' '; }\n",
       "    elsif (\$optname[\$i] eq 'BGFG') { \n",
       "      if (\$opt_val[\$i] eq 'BG') { \$cmdissue = \$cmdissue . '&'; }\n",
       "    }\n",
       "    else { \$cmdissue = \$cmdissue . '-' . \$opt_name[\$i] . \$opt_val[\$i] . ' '; }\n",
       #       "    if (!(\$optname[\$i] =~ /^(ARGIN|ARGOUT|BGFG)\$/)) {\n",
       #       "      \$cmdissue = \$cmdissue . '-' . \$opt_name[\$i] . \$opt_val[\$i] . ' ';\n",
       #       "    }\n",
       "  }\n",
       "};\n";
      print GENOUT 'print STDERR "Initiating... $cmdissue\\n";', "\n";
      print GENOUT 'print LOG "$cmdissue\\n";', "\n";
      print GENOUT 'close (LOG);', "\n";
      print GENOUT 'system ("$cmdissue");', "\n";

      print GENOUT $_; # print any remaining stuff read from FRAME.
    }
    elsif ($keyword =~ /^SUBROUTINES\s+MENU/) {
      print STDERR "SUBROUTINES MENU yet to be implemented.\n";
    }
    else { print STDERR "Error: Unknown keyword in *.frm file: $keyword $_"; }
  }
}

#%SUBROUTINES MENU
# sub ask_opt_val_I {
#   local ($res);
#   $res = system('dialog --menu "Type of string you want to search for:" 10 55 4 \
#             "1" "String as it is (specified by you later)" \
#             "2" "Word   (delimited by white space on both sides)" \
#             "3" "Prefix (delimited by leading white space)" \
#             "4" "Suffix (delimited by trailing white space)" \
#          2>> DUMMY.ans');
#   open (ANS, ">>DUMMY.ans") ||
#     print STDERR "Unable to open file DUMMY.ans for appending.\n";
#   print ANS "\n";
#   close (ANS);
#   return $res;
# }
# sub find_str {
#   local ($res);
#   $res = system('dialog --inputbox "Item to be searched for (preferably avoid spaces):[Use ^h for backspace]:" 9 50 \
#           2>> DUMMY.ans');
#   open (ANS, ">>DUMMY.ans") ||
#     print STDERR "Unable to open file DUMMY.ans for appending.\n";
#   print ANS "\n";
#   close (ANS);
#   return $res;
# }
sub prt_optcomment {
  local ($i);
  # print GENOUT "prt_optcomment DEBUG: opts = $#optlines\n";
  print GENOUT "# Options:\n";
  for ($i=0;  $i <= $#optlines;  $i++) {
    print GENOUT '# -', $optname[$i], "\t- ", $optcomment[$i], "\n";
  }
}
1;
