#!/usr/bin/perl
# This is a mail handler which takes e-mail messages
# and removes message header corresponding to forwarding of mail,
# from sangal@.*iitk.ernet.in and subject Forwarded_mail_Message.
# To execute it with respect to mail messages in in.mail do:
#   Example usage:      mail_rmfwd.pl < in.mail  > toread.mail

$msghd = 0;   # 1 means message header found.
$msgbody = 1; # 1 means message body continuing.
$msgbody1line = 0; # 1 means only a carriage return found after the 
                  # message header. Before the next message header begins. 
                  # This indicates that it is possibly a forwarded message.
$deb = 0;    # debugging flag.

# Now  start processing the input mail message
$bufcnt = -1;  # @buf is an array where messages are sent until
               #   the body is found.
while (<>) {
  if ($msghd) {  # Inside the message header.
    if ($_ eq "\n") { # Header terminates. Body to start.
      &terminate_header ();
      $msgbody1line = 1; # The message is a single line yet.
    }
    elsif (/^[^\s]+: /) {  # A possible field found
      $bufcnt++; $buf[$bufcnt] = $_;
      if ($deb) { print "--A field found:$_"; }
      $fld = $&;   chop ($fld);   $rem = $';  chop($rem);
         # Save the values of some fields
      if ($fld eq 'From:') { $fldfrom = $rem; }
      elsif ($fld eq 'Subject:') { $fldsubj = $rem; }
      elsif ($fld eq 'Cc:') { $fldcc = $rem; }
    }
    else {  # An ordinary header line (without field in beginning)
      $bufcnt++; $buf[$bufcnt] = $_;
      if ($deb) { print "--in header:$_"; }
    }
  }
  elsif ($msgbody) {  # We are inside the message body
    if (/^From /) {  # A new header found. Terminates message body.
      if ($msgbody1line == 1) { # A message of single line only.
        $msgbody1line = 0; # Message of single line is over.
        if (($fldfrom =~ /sangal@.*iitk\.ernet\.in/) &&
            ($fldsubj =~ /Forwarded_message_Follows/)) {
          $bufcnt = -1;  # Flush the buffer without printing.
        }
        else { 
          print STDERR "Forwarded message not from sangal@(iitk|csesun1).\n";
          #print "DEB-------Forwarded message but not from sangal@(iitk|csesun1).\n";
          #print "DEB       fldfrom = $fldfrom,  fldsubj = $fldsubj\n";
          &terminate_header1; # Print the previous header
                 # because the forwarded message is from somebody else.
        }
      }
      else { &terminate_header1; }
      &terminate_body ();
    }
    elsif (/^>From /) {  # A forwarded message
      if (($fldfrom =~ /sangal@.*iitk\.ernet\.in/) &&
          ($fldsubj =~ /Forwarded_message_Follows/)) {
        $bufcnt = -1;  # Flush the buffer without printing.
        # Now store line after removing '>'.
        $bufcnt++; $buf[$bufcnt] = substr($_, 1); 
        $msghd = 1;  $msgbody = 0;
      }
      else { 
          $bufcnt++; $buf[$bufcnt] = $_;  # Add to buffer
          print STDERR "Forwarded message not from sangal\@iitk\n";
          #print "DEB-------Forwarded message but not from sangal@(iitk|csesun1).\n";
          #print "DEB       fldfrom = $fldfrom,  fldsubj = $fldsubj\n";
      }
      if ($msgbody1line == 1) { # A message of single line only.
        $msgbody1line = 0; # Message of single line is over.
      }
    }
    else { # Usual line in the message body.
      if ($deb) { print "--Body of message:$_"; }
      if ($msgbody1line == 1) { &terminate_header1; $msgbody1line = 0; }
      print;
    }
  }
  else { 
    if ($deb) { print "--Neither body nor header:$_"; } 
    print;
  }
} # End of main while loop reading from input file

if ($msghd) { # Means message never got terminated. Premature end.
  &terminate_header ();
  &terminate_header1 ();  # Empty the buffer also.
}

#-----------------------------------------------------------------
# Message header has terminated. Body expected to start. Do the needful.
# in two parts. terminate_header simply changes the flags. but does
# not empty the buffer into STDOUT.
# terminate_header1 only empties the buffer does not change flags.
sub terminate_header {
  $msghd = 0;  $msgbody = 1; 
  if ($deb) { print "--Header over:$_"; }
    # The following has been moved to terminate_header1
    #for ($j = 0;  $j <= $bufcnt;  $j++){# Empty the buffer
    #  print $buf[$j];
    #}
    #$bufcnt = -1;
    #print;  # print this line.
}

# Message header has terminated. Body expected to start. Do the needful.
# in two parts. terminate_header simply changes the flags. but does
# not empty the buffer into STDOUT.
# terminate_header1 only empties the buffer does not change flags.
sub terminate_header1 {
  local ($j);
  for ($j = 0;  $j <= $bufcnt;  $j++){ # Empty the buffer
    print $buf[$j];
  }
  $bufcnt = -1;
  print "\n";  # Add a trailing new line.
}

# Beginning of message header encountered. Terminates body. Do needful.
sub terminate_body {
  $msghd = 1;   $msgbody = 0;
  $index = -1;
  if ($deb) { print "--Header starts.\n"; }
  $bufcnt++; $buf[$bufcnt] = $_;
}
