#!/usr/bin/perl
# Author: Rajeev Sangal, IIT Kanpur. Version: 0.5  10 August 1997
# (C) Copyright Rajeev Sangal, 1997. Permission to copy given
#     under GNU General Public License.
# This is a mail handler which takes e-mail messages
# and for each message if it satisfies a pre-specified property,
# saves it in an associated folder, otherwise sends it to stdout.
# It can also be used to automatically distribute mails to other users.
# All log messages are appended to $HOME/mailf.log by default.
#   Example usage:  mailf.pl < inbox
# Where inbox contains several messages.
# Alternatively, it can be installed in your .forward file.
#
# SPECIFYING PROPERTIES:
# The properties are pre-specified through $HOME/.mailfrc file. 
# Suppost the file contains the following lines (without '#' in front)
#   To: blug-list	           Folder: blug.mail
#   Return-Path: owner-blug-list   Folder: blug.mail
#   Subject: MTG	           Folder: mtg.recd
# The first line will cause mail addressed to
# blug mail group to go to the folder blug.mail.
# Similary, the third line in .mailfrc will send messages having
# the string MTG somewhere in the subject field to folder mtg.recd
# Note: The above specification would also match the following in the
# appropriate fields: 
#   blug-list@orion.svrec.ernet.in   | <blug-list@orion> | MTG, | MTG | <MTG>  etc.
# If more than one property is satisifed by a message, the first
# property it satisfies decides the folder to which it goes.
#
# DISTRIBUTING MAILS TO OTHER USERS:
# If the subject field contains in it somewhere:
#       USR::<name1> <name2> ... ::
# remails the message to the specified users: <name1>, <name2>, etc. 
# Messages with the above condition are not saved in the folder, 
# even if a relevant property is satisfied. They however are put
# in the mail spool.
# (If mails of only selected users are to be given this facility, 
# change line containing PERMITTED in this file appropriately.
# In distant future: Permissions are to be specified through .mailprc.)
#
# INSTALLING AS .forward FILE:
# If the program is kept in your bin area named as: mailf.pl,
# create .forward file as shown:
#    "| ./bin/mailf.pl  >> /usr/spool/mail/USERNAME   2>> mailf.log"
# Substitute USERNAME appropriately, and change './bin/mailf.pl' if 
# it is kept at another location.
#
# BUGS
# 1. If forwarded messages are received, mailf.pl can get fooled,
# and might separate the forwarding-headers as messages with
# a null body. The message would not get lost, however.


if ($ENV{HOME} ne '') { # Name of home dir.
  $mailfrcdir = $ENV{HOME}; # Name of the directory containing .mailfrc, etc.
  $mailflogdir = $ENV{HOME};  # Name of the directory where log file,
   # and some temporary files will be created. Default: home dir.
}
else {  # If in .forward file, HOME will have nil value.
  $mailfrcdir = $mailflogdir =  '.';
}

#print STDERR "DEBUG ENV: mailflogdir: $mailflogdir, mailfrcdir: $mailfrcdir\n";

$msghdr = 0;  # 1 means message header found.
$msgbody = 1; # 1 means message body continuing. (Before the first message 
         #  header is found, the lines will go to stdout, as no folder is open.)
$tousr = 0;  # 1 means the message is to be remailed to a user.
$deb = 0;    # debugging flag.


if (!(open (LOG, ">> $mailflogdir/mailf.log"))) {
  print STDERR 
    "mailf.pl: Unable to open appnd file $mailflogdir/mailf.log: $!\n";
  exit_grace;
}


#--------------------------------------------------------------
# Get the list of pre-specified addresses/patterns and their folders:
$frccnt = -1;  # Counter for number of entries in .mailfrc file.
if (!(open (FRC, "$mailfrcdir/.mailfrc"))) {
  print LOG
    "mailf.pl: Unable to open file $mailfrcdir/.mailfrc: $!\n";
  exit_grace;
}
while (<FRC>) { # Read all the properties from .mailfrc.
  if (!(/^\s*#/) && !(/^\s*$/) ) { # Not a comment or blank line.Process.
    $frccnt++;
    if (/^\s+/) { $_ = $'; }       # Remove opening space.
    ( $field[$frccnt], $pat[$frccnt], $const[$frccnt], $folder[$frccnt] ) = split (/\s+/, $_);
    if ($const[$frccnt] ne "Folder:") {
      print LOG 
        "Folder not specified in file $mailfrcdir/.mailfrc in line $.:$_";
      $frccnt--;  # discard entries due to this line.
    }
    #print STDERR "DEB: Folder: $folder[$frccnt]\n";
  }
}
if ($frccnt == -1) { 
  print LOG "mailf.pl: $mailfrcdir/.mailfrc file is empty.\n"; 
  exit_grace;
}
close (FRC);
#--------------------------------------------------------------
# Not yet incorporated.
# # Get the list of permitted users who can execute commands.
# $prccnt = -1;  # Counter for number of entries in mailprc file.
# if (-e "$mailfrcdir/.mailprc") {  # if the file of permissions exists
#   open (PRC, "$mailfrcdir/.mailprc") 
#     || print LOG "Unable to open file: $mailfrcdir/.mailprc: $!\n";
#   while (<PRC>) {
#     if (/^\s+/) { $_ = $'; }  # remove opening white space
#     if (!(/^#/)) { # Not a comment line. Process.
#       if ( /^([\w\-\.\@\!]+)/ ) {
#         $prccnt++;
#         $permitted[$prccnt] = $1;
#       }
#       else { print LOG "User addr not found in file .mailprc :$_"; }
#     }
#   }
#   if ($prccnt == -1) { print LOG "mailprc file is empty.\n"; }
#   close (PRC);
# }
#--------------------------------------------------------------
# Now  start processing the input mail message
$bufcnt = -1;  # @buf is an array where messages are sent until
               #   the body is found.
$index = -1; # Gives the position of matched folder found in message.
while (<>) {
  if ($msghdr) {  # Inside the message header.
    if ($_ eq "\n") { # Header terminates. Body to start.
      &terminate_header ();
    }
    elsif (/^[^\s]+: /) {  # A possible field found
      $bufcnt++; $buf[$bufcnt] = $_;
      if ($deb) { print "--A field found:$_"; }
      $fld = $&;   chop ($fld);   $rem = $';  chop($rem);
         # Save the values of some fields
      if ($fld eq 'From:') { $fldfrom = $rem; }
      elsif ($fld eq 'Subject:') { $fldsubj = $rem; }
      elsif ($fld eq 'Cc:') { $fldcc = $rem; }

      if ($index == -1) { # Check if from the pre-specified list.
        #print STDERR "DEB:$_";
        for  ($i = 0; $i <= $frccnt; $i++) {
          #print STDERR "   DEB: field: $field[$i], val: $pat[$i],\n";
          if (($fld eq $field[$i]) && 
              ( /[\s]*[\s<]($pat[$i])[>,@\s][\s]*/i )) {
              #    if (($fld eq $field[$i]) && 
              #        (/([\s<])[\s]*$pat[$i]([>,\s])[\s]*/) &&
              #        (($1 ne '<') || ($2 eq '>')) #ensure matching <.>
              #       ) {
            $index = $i; # Store loc of matched item in pre-specfd list.
            #print STDERR "DEBUG: Found.\n";
            last;  # Quit for-loop after the first pattern found.
          }
        }  # End for loop
      }
    }
    else {  # An ordinary header line (without field in beginning)
      $bufcnt++; $buf[$bufcnt] = $_;
      if ($deb) { print "--in header:$_"; }
    }
  }
  elsif ($msgbody) {  # We are inside the message body
    if (/^From /) {  # A new header found. Terminates message body.
      &terminate_body ();
    }
    else { # Usual line in the message body.
      if ($deb) { print "--Body of message:$_"; }
      print;
      if ($tousr) { print USRF ; }
    }
  }
  else { 
    if ($deb) { print "--Neither body nor header:$_"; } 
    print;
    if ($tousr) { print USRF ; }
  }
} # End of main while loop reading from input file

if ($msghdr) { # Means message never got terminated. Premature end.
  &terminate_header ();
}

if ($tousr) {  # if the last message was for remailing.
  close (USRF);
  system "mailx $usrname $put_subj < $mailflogdir/mailf.tmp";
  $curtime = `date`;  chop($curtime);
  #print  LOG "On $curtime mailing to $usrname\. Subj: $fldsubj\n";
  print  LOG "On $curtime from $fldfrom. Subj: $fldsubj\n";
}
print "\n";   # Put an extra new line (needed for .forwarding)
close (LOG);
#-----------------------------------------------------------------
# Message header has terminated. Body expected to start. Do the needful.
sub terminate_header {
  $msghdr = 0;  $msgbody = 1; 
  if ($deb) { print "--Header over:$_"; }
  if ($index != -1) { # Folder name found. Select folder.
    $success = 1;
    if (-e $folder[$index]) {
      if (!(open (FOLDER, ">>$folder[$index]") )) {
        $success = 0 ;
        print LOG "Unable to open output append file: $folder[$index]. Ignoring. $!\n";
      }
    }
    else {
      if (!(open (FOLDER, ">$folder[$index]") )) {
        $success = 0;
        print LOG "Unable to open output file: $folder[$index]. Ignoring. $!\n";
      }
    }
    if ($success) { select (FOLDER); }
  } # End select folder. If no success, stdout will be used
   #if (($fldfrom =~ /PERMITTED/) && ($fldsubj =~ /USR::/)) { 
    # If only permitted users' mail is to be redirected activate
    # the line above instead of the one below.
    # Change 'PERMITTED' to the desired username(s) 
    #   e.g., ('ram@baikuntha' | 'mohan')
    # In future to be changed to handle .mailprc
  if ($fldsubj =~ /USR::/) {
    $prefix_subj = $`;
    $poss_usrname = $';
    if ($poss_usrname =~ /::/) {   # trailing '::'
      $usrname = $`;   $prefix_subj .= ' ' . $';
    }
    $tousr = 1;
    if (!(open (USRF, ">$mailflogdir/mailf.tmp"))) {
      print LOG "mailf.pl: Unable to open file: $mailflogdir/mailf.tmp: $!\n";
      $tousr = 0;
    }
    elsif ($usrname =~ /^\s*$/) {  # Nil usernames
      print LOG "User name not given. Message from: $fldfrom\. Subj:$fldsubj\n";
      $tousr = 0;
    }
    else { 
      #print USRF "\~s $prefix_subj\n";  # If ~s option does not work in your mailer.
                          # the following 3 lines should be activated.
      $prefix_subj =~ s/['`]/"/g;
      if ($prefix_subj ne '') { $put_subj = "-s '".$prefix_subj."'"; }
      else { $put_subj = ''; }
      print USRF "--- Mail forwarded automatically using mailf program.\n";
    }
  }
  if ($tousr) { print USRF '>'; }  # Fwrd message, put '>' on 1st 'From'
  for ($j = 0;  $j <= $bufcnt;  $j++){  # Empty the buffer
    print $buf[$j];
    if ($tousr) { print USRF $buf[$j]; }
  }
  $bufcnt = -1;
  print;  # print this line.
  if ($tousr) { print USRF ; }
}

# Beginning of message header encountered. Terminates body. Do needful.
sub terminate_body {
  $msghdr = 1;   $msgbody = 0;
  $index = -1;
  $curfile = select(STDOUT);
  if ($curfile ne STDOUT) { close (FOLDER); }
  if ($deb) { print "--Header starts.\n"; }
  $bufcnt++; $buf[$bufcnt] = $_;
  if ($tousr) {  # Remail the last message in mailf.tmp to named userid.
    close (USRF);
    system "mailx $usrname $put_subj < $mailflogdir/mailf.tmp";
    $curtime = `date`;  chop($curtime);
    print  LOG "On $curtime mailing to $usrname\. Subj: $fldsubj\n";
    $tousr = 0;
  }
}
sub exit_grace {
  # Copy the message(s) as it is to stdout, then exit gracefully.
  while (<>) { print; }
  exit;
}
1;
