#!/usr/local/perl/perl
# This is a mail handler which takes e-mail messages
# and for each message if it has a pre-specified property
# saves it in an associated folder, otherwise send them to stdout.
# To execute it with respect to mail messages in in.t do:
#         perl mailsel.pl < in.t

# The properties are pre-specified through mailfrc file. 
# The following line in mailfrc will cause mail addressed to 
# cg mail group to go the file folder cg.msg
#   To: cg@cs.umn.edu	Folder: cg.msg
# Similary, the following line in mailfrc will send messages having
# the string MTG somewhere in the subject field to folder mtgf.snd
#   Subject: MTG	Folder: mtgf.snd
# The above patterns would also match: cg@cs.umn.edu, | <cg@cs.umn.edu>
#         MTG, | MTG | <MTG>  etc.

# If the subject field contains USR::<name>, remails the message to
# the specified userid <name>.
# Works only if the message comes from a permitted source.
# Currently from guraosh only. 
# (For future: Permissions are to be specified in mailprc.)

$msghd = 0;  # 1 means message header found.
$msgbody = 1; # 1 means message body continuing. (Before the first message 
         #  header is found, the lines will go to stdout, as no folder is open.)
$tousr = 0;  # 1 means the message is to be remailed to a user.
$deb = 0;    # debugging flag.
if (-e 'mailfrc.log') {
  open (LOG, ">>mailfrc.log") || print "Unable to open appnd file: mailfrc.log\n";
}
else {
  open (LOG, ">mailfrc.log") || print "Unable to open file: mailfrc.log\n";
}


#--------------------------------------------------------------
# Get the list of pre-specified addresses/patterns and their folders:
$frccnt = -1;  # Counter for number of entries in mailfrc file.
open (FRC, "mailfrc") || die "Unable to open file: mailfrc\n";
while (<FRC>) {
  if (!(/^\s*#/) && !(/^\s*$/) ) {# Not a comment or blank line.Process.
    $frccnt++;
    if (/^\s+/) { $_ = $'; }  # Remove opening space.
    ( $field[$frccnt], $pat[$frccnt], $const[$frccnt], $folder[$frccnt] ) = split (/\s+/, $_);
    if ($const[$frccnt] ne "Folder:") {
      print LOG "Folder not specified in file mailfrc :$_";
      $frccnt--;
    }
  }
}
if ($frccnt == -1) { die "mailfrc file is empty.\n"; }
close (FRC);
#--------------------------------------------------------------
# # Get the list of permitted users who can execute commands.
# $prccnt = -1;  # Counter for number of entries in mailprc file.
# if (-e "mailprc") {  # if the file of permissions exists
#   open (PRC, "mailprc") || print LOG "Unable to open file: mailprc\n";
#   while (<PRC>) {
#     if (/^\s+/) { $_ = $'; }  # remove opening white space
#     if (!(/^#/)) { # Not a comment line. Process.
#       if ( /^([\w\-\.\@\!]+)/ ) {
#         $prccnt++;
#         $permitted[$prccnt] = $1;
#       }
#       else { print LOG "User addr not found in file mailprc :$_"; }
#     }
#   }
#   if ($prccnt == -1) { print LOG "mailprc file is empty.\n"; }
#   close (PRC);
# }
#--------------------------------------------------------------
# Now  start processing the input mail message
$bufcnt = -1;  # @buf is an array where messages are sent until
               #   the body is found.
$index = -1; # Gives the position of matched folder found in message.
while (<>) {
  if ($msghd) {  # Inside the message header.
    if ($_ eq "\n") { # Header terminates. Body to start.
      &terminate_header ();
    }
    elsif (/^[^\s]+: /) {  # A possible field found
      $bufcnt++; $buf[$bufcnt] = $_;
      if ($deb) { print "--A field found:$_"; }
      $fld = $&;   chop ($fld);   $rem = $';  chop($rem);
         # Save the values of some fields
      if ($fld eq 'From:') { $fldfrom = $rem; }
      elsif ($fld eq 'Subject:') { $fldsubj = $rem; }
      elsif ($fld eq 'Cc:') { $fldcc = $rem; }

      if ($index == -1) { # Check if from the pre-specified list.
        for  ($i = 0; $i <= $frccnt; $i++) {
          if (($fld eq $field[$i]) && ( /[\s]*[\s<]$pat[$i][>,@\s][\s]*/i )) {
              #    if (($fld eq $field[$i]) && 
              #        (/([\s<])[\s]*$pat[$i]([>,\s])[\s]*/) &&
              #        (($1 ne '<') || ($2 eq '>')) #ensure matching <.>
              #       ) {
            $index = $i; # Store loc of matched item in pre-specfd list.
            last;  # Quit for-loop after the first pattern found.
          }
        }  # End for loop
      }
    }
    else {  # An ordinary header line (without field in beginning)
      $bufcnt++; $buf[$bufcnt] = $_;
      if ($deb) { print "--in header:$_"; }
    }
  }
  elsif ($msgbody) {  # We are inside the message body
    if (/^From /) {  # A new header found. Terminates message body.
      &terminate_body ();
    }
    else { # Usual line in the message body.
      if ($deb) { print "--Body of message:$_"; }
      print;
      if ($tousr) { print USRF ; }
    }
  }
  else { 
    if ($deb) { print "--Neither body nor header:$_"; } 
    print;
    if ($tousr) { print USRF ; }
  }
} # End of main while loop reading from input file

if ($msghd) { # Means message never got terminated. Premature end.
  &terminate_header ();
}

if ($tousr) {  # if the last message was for remailing.
  close (USRF);
  #  system "mail $usrname  < usrf.t";
  print  LOG "mailing to $usrname  Subj: $fldsubj\n";
}
close (LOG);   # End of main program

#-----------------------------------------------------------------
# Message header has terminated. Body expected to start. Do the needful.
sub terminate_header {
  $msghd = 0;  $msgbody = 1; 
  if ($deb) { print "--Header over:$_"; }
  if ($index != -1) { # Folder name found. Select folder.
    $success = 1;
    if (-e $folder[$index]) {
      if (!(open (FOLDER, ">>$folder[$index]") )) {
        $success = 0 ;
        print LOG "Unable to open out append file: $folder[$index]. Ignoring.\n";
      }
    }
    else {
      if (!(open (FOLDER, ">$folder[$index]") )) {
        $success = 0;
        print LOG "Unable to open output file: $folder[$index]. Ignoring.\n";
      }
    }
    if ($success) { select (FOLDER); }
  } # End select folder. If no success, stdout will be used
  if (($fldfrom =~ /guraosh/) && ($fldsubj =~ /USR::/)) {
    $tousr = 1;
    open (USRF, ">usrf.t") || print LOG "Unable to open file: usrf.t\n";
    print USRF "\~s $fldsubj\n";
    $usrname = $';
    if ($usrname =~ /^\s*([^\s]+)/ ) { $usrname = $1; }
    else { 
      $usrname = '';
      print LOG "User name not given. Message from: $fldfrom Subj:$fldsubj\n";
    }
  }
  for ($j = 0;  $j <= $bufcnt;  $j++){# Empty the buffer
    print $buf[$j];
    if ($tousr) { print USRF $buf[$j]; }
  }
  $bufcnt = -1;
  print;  # print this line.
  if ($tousr) { print USRF ; }
}

# Beginning of message header encountered. Terminates body. Do needful.
sub terminate_body {
  $msghd = 1;   $msgbody = 0;
  $index = -1;
  $curfile = select(STDOUT);
  if ($curfile ne STDOUT) { close (FOLDER); }
  if ($deb) { print "--Header starts.\n"; }
  $bufcnt++; $buf[$bufcnt] = $_;
  if ($tousr) {  # Remail the message in usrf.t to named userid.
    close (USRF);
    #  system "mail $usrname  < usrf.t";
    print  LOG "mailing to $usrname  Subj: $fldsubj\n";
    $tousr = 0;   
  }
}
