#!/usr/local/perl/perl
# This is a mail handler which takes e-mail messages
# and for each message if it has a pre-specified property
# saves it in an associated folder, otherwise send them to stdout.
# To execute it with respect to mail messages in in.t do:
#         perl mailsel.pl < in.t

# The properties are pre-specified through mailfrc file. 
# The following line in mailfrc will cause mail addressed to 
# cg mail group to go the file folder cg.msg
#   To: cg@cs.umn.edu	Folder: cg.msg
# Similary, the following line in mailfrc will send messages having
# the string MTG somewhere in the subject field to folder mtg.msg
#   Subject: MTG	Folder: mtg.msg
# The above patterns would also match: cg@cs.umn.edu, | <cg@cs.umn.edu>
#      MTG | MTG, | <MTG> etc.

$msghdr = 0;  # 1 means message header found.
$msgbody = 1; # 1 means message body continuing. (Before the first message 
         #  header is found, the lines will go to stdout, as no folder is open.)
$deb = 0;
if (-e 'mailfrc.log') {
  open (LOG, ">>mailfrc.log") || warn "Unable to open appnd file: mailfrc.log\n";
}
else {
  open (LOG, ">mailfrc.log") || warn "Unable to open file: mailfrc.log\n";
}

#--------------------------------------------------------------
# Get the list of pre-specified addresses/patterns and their folders:
$frccnt = -1;  # Counter for number of entries in mailfrc file.
open (FRC, "mailfrc") || die LOG "Unable to open file: mailfrc\n";
while (<FRC>) {
  if (!(/^\s*#/) && !(/^\s*$/) ) {# Not a comment or blank line.Process.
    $frccnt++;
    if (/^\s+/) { $_ = $'; }  # Remove opening space.
    ( $field[$frccnt], $pat[$frccnt], $const[$frccnt], $folder[$frccnt] ) = split (/\s+/, $_);
    if ($const[$frccnt] ne "Folder:") {
      print LOG "Folder not specified in file mailfrc :$_";
      $frccnt--;
    }
  }
}
if ($frccnt == -1) { die LOG "mailfrc file is empty.\n"; }
close (FRC);
#--------------------------------------------------------------
# Now  start processing the input mail message
$bufcnt = -1;  # @buf is an array where messages are sent until
               #   the body is found.
$index = -1; # Gives the position of matched folder found in message.
while (<>) {
  if ($msghd) {  # Inside the message header.
    if ($_ eq "\n") { # Header over.
      &terminate_header ();
    }
    elsif (/^[\w\-]+: /) {  # A possible field found
      $bufcnt++; $buf[$bufcnt] = $_;
      if ($deb) { print "--A field found:$_"; }
      $fld = $&;   $rem = $';  chop ($fld);
         # Save the values of some fields
      if ($fld eq 'From:') { $fldfrom = $rem; }
      elsif ($fld eq 'Subject:') { $fldsubj = $rem; }
      elsif ($fld eq 'Cc:') { $fldcc = $rem; }

      if ($index == -1) { # Check if from the pre-specified list.
        for  ($i = 0; $i <= $frccnt; $i++) {
          if (($fld eq $field[$i]) && (/[\s<][\s]*$pat[$i][>,\s][\s]*/)) {
              #    if (($fld eq $field[$i]) && 
              #        (/([\s<])[\s]*$pat[$i]([>,\s])[\s]*/) &&
              #        (($1 ne '<') || ($2 eq '>')) #ensure matching <.>
              #       ) {
            $index = $i;  # Store loc of matched item in pre-specfd list.
            last;  # Quit for-loop after the first pattern found.
          }
        }  # End for loop
      }
    }
    else {  # An ordinary header line (without field in beginning)
      $bufcnt++; $buf[$bufcnt] = $_;
      if ($deb) { print "--in header:$_"; }
    }
  }
  elsif ($msgbody) {  # We are inside the message body
    if (/^From /) {  # A new header found. Terminates message body.
      &terminate_body ();
    }
    else {
      if ($deb) { print "--Body of message:$_"; }
      print;
    }
  }
  else { 
    if ($deb) { print "--Neither body nor header:$_"; } 
    print;
  }
}  # End of main while loop - input over

if ($msghd) { # Means message never got terminated. Premature end.
  &terminate_header ();  # Sends it out of the buffer etc.
}
close (LOG);

#-----------------------------------------------------------------
# Message header has terminated. Body expected to start. Do the needful.
sub terminate_header {
  $msghd = 0;  $msgbody = 1; 
  if ($deb) { print "--Header over:$_"; }
  if ($index != -1) { # Folder name found. Select folder.
    $success = 1;
    if (-e $folder[$index]) {
      if (!(open (FOLDER, ">>$folder[$index]") )) {
        $success = 0 ;
        warn LOG "Unable to open out append file: $folder[$index]. Ignoring.\n";
      }
    }
    else {
      if (!(open (FOLDER, ">$folder[$index]") )) {
        $success = 0;
        warn LOG "Unable to open output file: $folder[$index]. Ignoring.\n";
      }
    }
    if ($success) { select (FOLDER); }
  } # End select folder. If no success, stdout will be used
  for ($j = 0;  $j <= $bufcnt;  $j++){# Empty the buffer
    print $buf[$j];
  }
  $bufcnt = -1;
  print;  # print this line.
}
#-----------------------------------------------------------------
# Beginning of message header encountered. Terminates body. Do needful.
sub terminate_body {
  $msghd = 1;   $msgbody = 0;
  $index = -1;
  $curfile = select(STDOUT);
  if ($curfile ne STDOUT) { close (FOLDER); }
  if ($deb) { print "--Header starts.\n"; }
  $bufcnt++; $buf[$bufcnt] = $_;
}
