#!/usr/bin/perl
# Used for restoring the accounts in /etc/passwd given 
# the 'ls -l' output of the users home dirs (from /usr3, /usr4, etc.)
#   Example usage: restore_passwd.pl < usr3.ls >> /etc/passwd
# LIMITATION. Assumes the following:
#  1. The name of the user is the same as his home dir.
#  2. Does not look at group id. Makes it '100'.
# Options:
# -d	-  Name of the home directory containing the user accounts. 
# -v	-  Verbose output. Print userids. 

require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");

# Assumes that the args begin after the params. (Each param begins by -)
local ($i, $parmlastpos, $argbegpos, $arg, $suff, $argsinfile, $Finname, $Finname);
local ($dirname, $verbose);
$parmlastpos = -1; # Gives pos of last param in ARGV. Default no params.
$argbegpos = -1;  # Gives position of 1st argument in ARGV. Default no args
$argsinfile = 0;  # Default args file names are not in a file.
$suff = '.new'; # Default suffix for output file if necessary.
$dirname = '/usr3'; # Default  Name of the home directory containing the user accounts. 
$verbose = 0; #  Verbose output. Print userids.  Default value as shown.

# Look up .restore_passwdrc file. Override defaults.
if (open (RC, '.restore_passwdrc')) {
  while (<RC>) {
    chop;
    if (/\s*#/) {  next; }   # Ignore. A comment line.
    if (/\s*-/) {  # A parameter found.
      $_ = '-' . $';
      if (substr($_,1,1) eq 'h') { # print help info.
        &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/sysapp/restore_passwd.cmd');
        die;
      }
      elsif (substr($_,1, 1) eq  'd') {
        $dirname = substr ($_, 2);
      }
      elsif (substr($_,1, 1) eq  'v') {
        $verbose = 1;
      }
      else { 
          print STDERR "restore_passwd.cmd: Unrecognized param in rc file: $_.\n";
          &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/sysapp/restore_passwd.cmd');
          die;
      }
    }
    else { # 
      print STDERR "restore_passwd.cmd: Param must begin with '-' in rc file: $_. Ignoring.\n";
    }
  }
  close (RC);
}

for ($i=0; $i <= $#ARGV; $i++) { # Extract the parameters and locate
                                 #  starting loc of command args.
  if (substr($ARGV[$i],0,1) eq '-') {  # A parameter found.
    $parmlastpos++;  # Change the following lines to suit your needs.
    if (substr($ARGV[$i],1,1) eq 'h') { # print help info.
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/sysapp/restore_passwd.cmd');
      die;
    }
    elsif (substr($ARGV[$i],1,1) eq 's') { # Suffix for output file names.
       $suff = substr ($ARGV[$i],2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'd') {
      $dirname = substr ($ARGV[$i], 2);
    }
    elsif (substr($ARGV[$i],1, 1) eq  'v') {
      $verbose = 1;
    }
    else  { print STDERR "restore_passwd.cmd: Parameter not recognized $ARGV[$i].\n";
      &printusage ('/home/bharathi/PNJ1/usr3/sangal/tools/src/sysapp/restore_passwd.cmd');
      die;
    }
  }
  else { # Argument found. Parameters end.
    if (substr($ARGV[$i],0,1) eq '@') {  # A file of args found
      $argsinfile = 1;
      $ARGV[$i] = substr($ARGV[$i], 1);
    }
    $argbegpos = $i;
    last;
  }
}
#print STDERR "restore_passwd.cmd: DEB params end: $parmlastpos;  args begin: $argbegpos; total: $#ARGV;\n";

# Process the arguments in the command line then call restore_passwd.
# 1. Can be used as a filter. OR
# 2. If one or more file arguments are given, they are treated as
#    inputs files, and the output file name is given
#    by the input file name suffixed by the default suffix
#    (which can be changed by approp flag).

if ($argbegpos < 0) { # No args.
  &restore_passwd(STDIN, STDOUT);
}
elsif ($argsinfile) {  # Argument filenames in file.
  open (ARGFILE, "$ARGV[$argbegpos]") ||
    die "restore_passwd.cmd: Unable to open args file $ARGV[$argbegpos]\n" ;
  while (<ARGFILE>) { # Read args.
    chop;
    if (!(open (INFILE, $_))) {
      print STDERR "restore_passwd.cmd: Unable to open file $_\n" ;
      next;
    }
    if (!(open (OUTFILE, ">$_$suff"))) {
      print STDERR "restore_passwd.cmd: Unable to open out file $_$suff\n";
      close (INFILE);
      next;
    }
    $Finname = $_;  $Foutname = $_ . $suff;
    &restore_passwd (INFILE, OUTFILE);
    close (INFILE);
    close (OUTFILE);
  }
  close (ARGFILE);
}
else { # Call the program with each arg. 
  for ($arg = $argbegpos; $arg <= $#ARGV; $arg++) {
    if (!(open (INFILE, "$ARGV[$arg]"))) {
      print STDERR "restore_passwd.cmd: Unable to open file $ARGV[$arg]\n" ;
      next;
    }
    if (!(open (OUTFILE, ">$ARGV[$arg]$suff"))) {
      print STDERR "restore_passwd.cmd: Unable to open out file $ARGV[$arg]$suff\n";
      close (INFILE);
      next;
    }
    $Finname = $ARGV[$arg];
    $Foutname = $ARGV[$arg] . $suff;  
    &restore_passwd (INFILE, OUTFILE);
    close (INFILE);
    close (OUTFILE);
  }
}
#------------------------------------------------------

sub restore_passwd {
local ($W_fin, $W_fout) = @_;
select($W_fout);
#%COMMENT
# Used for restoring the accounts in /etc/passwd given 
# the 'ls -l' output of the users home dirs (from /usr3, /usr4, etc.)
#   Example usage: restore_passwd.pl < usr3.ls >> /etc/passwd
# LIMITATION. Assumes the following:
#  1. The name of the user is the same as his home dir.
#  2. Does not look at group id. Makes it '100'.
#%CME TYPE 11


while (<$W_fin>) {
  @all = split(/\s+/);
   # print STDERR "DEB: $all[0]  $all[4]\n";
   #if (substr($all[0], 0, 1) eq '-') { # To check that the info is for a file, not directory.
   #}
  if ($#all < 0) { next; }  # Blank line.
  if ($all[0] =~ /:$/) {  chop($all[0]);  $dirname = $all[0];  } # Skip directories
  else {
    if ('total' eq substr($all[0], 0, 5)) { next; }  # skip this line. Directory.
    if (!($all[2] =~ /^[0-9]+$/)) {  next; } # userid already exists.
    if ('d' eq substr($all[0], 0, 1)) {  # Process.
      $fname = $all[8];
      #if ($fname =~ /\/$/ ) {  next; }
      if ($fname =~ /\*$/ ) {  chop ($fname);  }  # Remove '*' from +x files.
      if ($all[2] == 0) {
        $all[2] = 99;
        print STDERR "Warn: a root password found. Changing to 99.: $fname\n";
      }
      print  $fname, ':*:', $all[2], ':100:', $fname, ':',
                $dirname, '/', $fname, ':/bin/bash', "\n";
      if ($verbose) { print STDERR $fname, "\n"; }
    }
  }
}

}
1;
