#!/usr/bin/perl
# Cuts out mime part-attachements from the input email message
# and writes to output.
#   Example usage:  mime_read.pl < mail.msg  > mime.part
# 
# State transition:
#  await_marker_def -> marker_def_found -> cut_lines -> nlnl_found
#     -> print_lines

$marker = '^Not found yet zxytre$';
$state = 'await_marker_def';
while(<>) {
  if ($state eq 'await_marker_def') {
    if (/boundary="([^"]+)"/i) {  # Marker def found
      $marker = $1;
      $state = 'marker_def_found';
       #print "DEB: m def found: $_";
      next;
    }
  }
  elsif ($state eq 'marker_def_found') {  # await start-marker
    if (/^--$marker$/) { # Start-marker found
      $state = 'cut_lines';
       #print "DEB: start m found: $_";
      next;
    }
  }
  elsif ($state eq 'cut_lines') {  # Ignore text lines after the start marker
    if (/^$/) { # End of cut lines found.
      $state = 'nlnl_found';
       #print "DEB: cut lines over: $_";
      next;
    }
  }
  elsif ($state eq 'nlnl_found') {
    if (/^--$marker$/) { # Start-marker found (without any content in the prev.)
      $state = 'cut_lines';
    }
    elsif (/^$/) { next; }  # No change. Continue waiting for non-blank line
    else { print;  $state = 'print_lines'; }  # non-blank line found
  }
  elsif ($state eq 'print_lines') {
    if (/^--$marker--$/) {  # End marker found
       #print "DEB: end m found: $_";
      last;
    }
    elsif (/^--$marker$/) { # Another start-marker found. Handles 1 attach
      last;
    }
    else { print; }
  }
}
if ($state ne 'print_lines') {
  print STDERR "Invalid end found in input file. State=$state\n";
}
