#!/usr/bin/perl
# Takes a *.pl file, puts a wrapper around it, and puts it in the *.cmd file.
# A description of command line options can also be given, in which case the
# wrapper will also handle command line options.
#  Example usage: wrapcmd.pl  PERLFILE.pl
# The above produces PERLFILE.cmd which can handle multiple files as
# well as command line options.
#
# $suff		- Suffix to be used for naming the output file. Default '.cmd'.

# $funcname	- Name of the function to be called.
# TODO: After #%COMMENT in the input file, the comments should end on
#  the first non-comment line.

#sub wrapcmd {
  local ($fileframe, $comment, $require, @optlines, @optname, @optsw, @optswvalue, @optinit, @optvar, @optcomment);
  local ($v, $keyword, $i, $len);
  # print STDERR "wrapcmd.pl DEBUG: funcname = $funcname,\n";
  $Finname = $ARGV[0];
  open (INPL, $Finname) || die "Unable to open file $Finname: $!\n";
  if ($Finname =~ /\.[^.]*$/) { $Foutname = $` . '.cmd'; }
  else { $Foutname .= '.cmd'; }
  open (OUTCMD, ">$Foutname") || die "Unable to open output file $Foutname: $!\n";
  if ($Foutname =~ /([^\/]*)\.cmd$/) { $funcname = $1; }
  else { 
    print STDERR 'wrapcmd.pl: Unable to gen funcname from file name: ',
                 $Foutname, "Using the name WRAP.\n";
    $funcname = 'WRAP';
  }
  if ($Foutname =~ /^(\/|~)/) { $absname = $Foutname; } # Absolute path name given.
  else { $absname = `pwd`;  chop $absname;  
         $absname .= '/' . $Foutname;
  }

  # First extract the option names etc.
  $/ = '#%';
  # $_ = <INPL>;   
  # if (/#%$/) { chop;  chop; }
  $fileframe = '/home/bharathi/PNJ1/usr3/sangal/tools/src/SUPP/cmdmm.frm'; # Default many to many (MM).
  while (<INPL>) { # Read in the option description.  In the given input file.
    if (/#%$/) { chop;  chop; }
    if (/^CMD\s+TYPE/) { # Choose many to many, many to one, one to one etc.
      $_ = $';
      if (/^\s+11/) { $fileframe = '/home/bharathi/PNJ1/usr3/sangal/tools/src/SUPP/cmd11.frm'; }
      elsif (/^\s+M1/i) { $fileframe = '/home/bharathi/PNJ1/usr3/sangal/tools/src/SUPP/cmdm1.frm'; }
      elsif (/^\s+MM/i) { $fileframe = '/home/bharathi/PNJ1/usr3/sangal/tools/src/SUPP/cmdmm.frm'; }
      else { die "Illegal command type specified: $'"; }
    }
    elsif (/^END/) { last;}
    elsif (/^COMMENT\s*\n/) { $comment = $'; }
       #elsif (/^OPTIONS\s*\n/) { 
    if (/#OPT/) {
      # print STDERR "DEBUG: Processing options.\n";
      # print STDERR $_, "DONE\n";
      @optlines = split (/\n/);  # Separates possible opt lines.
      $onum = -1;   # Keeps count of options found.
      for ($i=0;  $i <= $#optlines;  $i++) { # Process each opt line.
        # FORMAT for opt line:       $VAR=INIT;  # COMMENT #OPT -XYZ sw0
        #  optional parts    -->         ^ ^                         ^ 
        # Example:
        #    $fldnamefile = '';  # Name of the file containing field names. #OPT -f
        #    $Dlimflag = 1; # Print output Dlimiter. #OPT -yes sw0 

        $_ = $optlines[$i];
        if (!(/#OPT /)) { next; }    # Not an option line 
        # Else option line found. Process variable naming and initialization.
        $onum++;
        if (/;\s*/) { # Found ';'. 
          $optinit[$onum] = $` ;  
          $_ = $';
          # Now separate the var name from the rest of initialization.
          if ($optinit[$onum] =~ /\s*=\s*/) { 
            $optvar[$onum] = $`; 
            $optinit[$onum] .=  ';'; 
          }
          else { # Not an initialization. Only var name given.
            $optvar[$onum] = $optinit[$onum];  $optinit[$onum] = '';
          } 
        }
        else { 
          print STDERR "Error: ';' expected after var name or initialization. $_";
          $onum--;  next;
        }
        if (/^#/) {  # A comment found.
          $_ = $';
          if (/#OPT\s+/) { $optcomment [$onum] = $`;  $_ = $'; }
          else {
            print STDERR "Error: '#OPT' expected after the comment in: $_";
            $onum--;  next;
          }
        }
        else {
          print STDERR "Error: Comment or new line expected: $_";
          $onum--;  next;
        }
        if (substr($_,0,1) eq '-') { 
          if (/\s+/) { # left part is option name, right part is remaining.
            $optname[$onum] = substr ($`, 1);    $_ = $';
          }
          else { $optname[$onum] = substr ($_, 1);   next; }
        }
        else {
          print STDERR "Option $optname[$onum] must begin with a '-'.\n";
          $onum--;
        }
        # Process the 'sw' option with 0, 1 or space.
        if (/^sw/) { 
          $optsw[$onum] = 1;  
          $_ = $';
          if (/^0/) { $optswvalue[$onum] = 0;  $_ = $'; }
          elsif (/^1/) { $optswvalue[$onum] = 1;  $_ = $'; }
          elsif (/^(\s|$)/) { $optswvalue[$onum] = 1; $_ = $'; }
          else { 
            print STDERR "Error: sw not followed by 0, 1, space: $_";
            $onum--;
          }
        }
        else { $optsw[$onum] = 0; }
        if (!(/^\s*$/)) { 
          print STDERR "Warn: Ignoring invalid chars starting from: $_\n";
        }

        #print STDERR "wrapcmd.pl DEBUG: $optlines[$i]\n$optname[$i]\t$optsw[$i]\t$optinit[$i]\n";
        #print STDERR "$optname[$i]\t$optsw[$i]:$optswvalue[$i]:\t$optvar[$i]\n";
      }  # End of for loop for each opt line.
    }
  } # End of while loop.
  # print STDERR "wrapcmd.pl DEBUG: Comment ----$comment----\n";
  close (INPL);

#-------------------- Now generate code ----------------------------
  open (FRAME, $fileframe) ||
    die "wrapcmd.pl: Unable to open file $fileframe: $!\n";
  $_ = <FRAME>;   chop;  chop; 
  s/DUMMY/$funcname/g;  
  s/ABSNAME/$absname/g;  
  print OUTCMD $_;  # Read-print the first part
  while (<FRAME>) {  # Read in chunks delimited by '#%'
    chop;  chop;
    s/DUMMY/$funcname/g;  
    s/ABSNAME/$absname/g;  
    if (/\n/) { $keyword = $`;  $_ = $'; }
    else { print OUTCMD $_;   next; }
    if ($keyword =~ /^COMMENT\s*/) { 
      print OUTCMD $comment;
      &prt_optcomment;
      print OUTCMD $_;
    }
    elsif ($keyword =~ /^REQUIRE\s*/) { 
      if (!($require =~ /printusage/)) {
        print OUTCMD 'require("/home/bharathi/PNJ1/usr3/sangal/tools/src/etc/printusage.pl");', "\n";
      }
      print OUTCMD $require, $_;
    }
    elsif ($keyword =~ /^LOCAL\s*/) {  # Generate local var declaration.
      if ($#optname >= 0) {
        print OUTCMD "local ($optvar[0]";
        for ($j=1;  $j <= $#optvar;  $j++) { print OUTCMD ', ', $optvar[$j]; }
        print OUTCMD ");\n";
      }
      print OUTCMD $_;
    }
    elsif ($keyword =~ /^OPTIONS\s+DEFAULTS\s*/) {  # generate initializations.
      for ($i=0; $i <= $#optname;  $i++) { # do for each option line.
        # print OUTCMD "DEBUG: $optinit[$i]\n";
        if ($optinit[$i] ne '') { # There is an initialization.
          if (!($optsw[$i])) { print OUTCMD $optinit[$i], ' # Default ', $optcomment[$i], "\n"; }
          else { print OUTCMD $optinit[$i], ' # ', $optcomment[$i], 
                       " Default value as shown.\n"; }
        }
        else { print OUTCMD "# $optvar[$i] - $optcomment[$i]\n"; }
      }
      print OUTCMD $_;
    }
    elsif ($keyword =~ /^RC\s+ELSIF\s+STMTS\s*/) { # generate RC elsif for options.
      for ($i=0; $i <= $#optname;  $i++) { # do for each option line.
        $len = length ($optname[$i]);
        if (!($optsw[$i])) { 
          print OUTCMD "      elsif (substr(\$_,1, $len) eq  '$optname[$i]') {\n";
          $len1 = $len + 1;
          print OUTCMD "        $optvar[$i] = substr (\$_, $len1);\n      }\n";
        }
        else {
          print OUTCMD "      elsif (substr(\$_,1, $len) eq  '$optname[$i]') {\n";
          print OUTCMD "        $optvar[$i] = $optswvalue[$i];\n      }\n";
        }
      }
      print OUTCMD $_;
    }
    elsif ($keyword =~ /^CMD\s+LINE\s+ELSIF\s+STMTS\s*/) { # generate cmdline elsif for options.
      for ($i=0; $i <= $#optname;  $i++) { # do for each option line.
        $len = length ($optname[$i]);
        if (!($optsw[$i])) { 
          print OUTCMD "    elsif (substr(\$ARGV[\$i],1, $len) eq  '$optname[$i]') {\n";
          $len1 = $len + 1;
          print OUTCMD "      $optvar[$i] = substr (\$ARGV[\$i], $len1);\n    }\n";
        }
        else {
          print OUTCMD "    elsif (substr(\$ARGV[\$i],1, $len) eq  '$optname[$i]') {\n";
          print OUTCMD "      $optvar[$i] = $optswvalue[$i];\n    }\n";
        }
      }
      print OUTCMD $_;
    }
    elsif ($keyword =~ /^BEGIN/) { print OUTCMD $_; }
    elsif ($keyword =~ /^END/) { print OUTCMD $_; }
    elsif ($keyword =~ /^SPECIAL/) {  # Do nothing. print OUTCMD $_; 
      print STDERR "Not handling SPECIAL keyword in *.frm file yet.\n";
    }
    else { print STDERR "Error: Unknown keyword in *.frm file: $keyword $_"; }
  }

  print OUTCMD "\n#------------------------------------------------------\n\nsub $funcname {\n", 
               'local ($W_fin, $W_fout) = @_;', "\n",
               'select($W_fout);', "\n";
  # Now write the input *.pl  after suitable modifications.
  open (INPL, $Finname) || die "Unable to open file $Finname: $!\n";
  $/ = "\xa";
  while (<INPL>) {
    if (/^#!/) { next; }  # Ignore shell invocation.
    if (/#OPT/) { next; }  # Ignore option line.
    if (/(\sseek\s|\sselect\s|\sread\s)/) {
      print STDERR "Found '$&' in $Finname. Generated program may not work.\n";
    }
    if (/<>/) { s/<>/<\$W_fin>/; }
    print OUTCMD $_;
  }
  print OUTCMD "}\n1;\n";
  close (OUTCMD);
  system ("chmod +x $Foutname");

sub prt_optcomment {
  local ($i);
  # print OUTCMD "prt_optcomment DEBUG: opts = $#optname\n";
  print OUTCMD "# Options:\n";
  for ($i=0;  $i <= $#optname;  $i++) {
    print OUTCMD '# -', $optname[$i], "\t- ", $optcomment[$i], "\n";
  }
}
1;
